import java.util.*;
import java.awt.Color;	
import java.awt.Graphics2D;	
import java.awt.RenderingHints;	
import java.awt.BasicStroke;
import java.awt.AlphaComposite;
import java.awt.image.*;
import java.awt.geom.*;

public class JBackground extends BufferedImage {

	private static final int EMPTY = 0;
	private static final int LINES = 1;
	private static final int KAROS = 2;
	private static final int NOTES = 3;

	private Graphics2D pencil, eraser;

	//-----------------------------------------------------------------
	public class JBackStyle {
		private int style, offset, distance, dLines;
		private Color color;
		
		public JBackStyle(int s, int o, int d, int dL, Color c) {
			this.style    = s;
			this.offset   = o;
			this.distance = d;
			this.dLines   = dL;
			this.color	  = c;
		}
		
		public String toString() {
			return "backgroundstyle "+style+" "+offset+" "+distance+" "+dLines+" "
					+color.getRed()+" "+color.getGreen()+" "+color.getBlue();
		}
		
	}

	public List<JBackStyle> styleList = new ArrayList<JBackStyle>();

	//----------------------------------------------------------------

	public JBackground(int x, int y) {
		super(x,y, BufferedImage.TYPE_INT_ARGB);
		
		pencil = (Graphics2D)getGraphics();
		pencil.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
								RenderingHints.VALUE_ANTIALIAS_ON);
		pencil.setStroke(new BasicStroke(2));
		
		eraser = (Graphics2D)getGraphics();
		eraser.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR, 0.0f));
		
	}

	//----------------------------------------------------------------

	public void setColor(Color c) {
		pencil.setColor(c);
	}

	//----------------------------------------------------------------

	public void clearFrom(int offset) {
		eraser.fill( new Rectangle2D.Double(
  		                 0,offset, getWidth(), getHeight()-offset) );

		styleList.add( new JBackStyle(EMPTY, offset, 0, 0, pencil.getColor() ) );
	}

	//----------------------------------------------------------------

	public void paintLines (int offset, int distance) {

		for (int i=offset+distance; i <= getHeight(); i=i+distance ) {
			pencil.drawLine(0, i, getWidth(), i);		
		}

		styleList.add( new JBackStyle(LINES, offset, distance,0, pencil.getColor() ) );
	}

	//----------------------------------------------------------------

	public void paintKaros (int offset, int distance) {

		for (int i=offset+distance; i <= getHeight(); i=i+distance ) {
			pencil.drawLine(0, i, getWidth(), i);		
		}

		for (int i=distance; i <= getWidth(); i=i+distance ) {
			pencil.drawLine(i, offset, i, getHeight());		
		}

		styleList.add( new JBackStyle(KAROS, offset, distance,0, pencil.getColor() ) );

	}

	//----------------------------------------------------------------

	public void paintNotes (int offset, int distance, int dLines) {

		for (int i=offset+distance/2; i <= getHeight(); i=i+distance+4*dLines ) {
			for (int j=i; j <= i+4*dLines; j=j+dLines ) {
				pencil.drawLine(0, j, getWidth(), j);
			}
		}
		styleList.add( new JBackStyle(NOTES, offset, distance, dLines, pencil.getColor() ) );
	}

	//----------------------------------------------------------------

	public String getAllStyles() {
		StringBuilder allstyles = new StringBuilder();
		
		for (int i=0; i<styleList.size(); i++) {
			allstyles.append( styleList.get(i).toString()+"\n");
		}
		
		return allstyles.toString();
	}
	
}
