import java.util.*;
import java.awt.Color;	
import java.awt.Graphics2D;	
import java.awt.RenderingHints;	
import java.awt.BasicStroke;
import java.awt.AlphaComposite;
import java.awt.image.*;
import java.awt.geom.*;

public class JBoardImage extends BufferedImage {

	private Graphics2D pencil, eraser;

	private static BasicStroke[] pens = new BasicStroke[3];
	private static BasicStroke[] penpoints = new BasicStroke[3];
	private int penStyle = 1;

	private Color PenColor = new Color(0,0,0);

	//----------------------------------------------------------------

	public JBoardImage(int x, int y) {
		super(x,y, BufferedImage.TYPE_INT_ARGB);
		
		pencil = (Graphics2D)getGraphics();
		pencil.setRenderingHint(RenderingHints.KEY_ANTIALIASING, 
								RenderingHints.VALUE_ANTIALIAS_ON);
		pencil.setStroke(new BasicStroke(2));
		
		eraser = (Graphics2D)getGraphics();
		eraser.setComposite(AlphaComposite.getInstance(AlphaComposite.CLEAR, 0.0f));

		pens[0] = new BasicStroke(1.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		pens[1] = new BasicStroke(2.3f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		pens[2] = new BasicStroke(4.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);

		penpoints[0] = new BasicStroke(2.3f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		penpoints[1] = new BasicStroke(3.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		penpoints[2] = new BasicStroke(6.0f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND);
		
	}

	public void setPenColor(Color c) 	{ PenColor = c;		}
	public Color getPenColor() 			{ return PenColor;	}

	public void setPenStyle(int style) 	{ penStyle = style;	}
	public int getPenStyle() 			{ return penStyle;	}

	public void setStroke() {
		pencil.setStroke(pens[penStyle]);
	}

	public void setPointStroke() {
		pencil.setStroke(penpoints[penStyle]);
	}

	public BasicStroke getStroke() {
		return pens[penStyle];
	}

	public void clear(int x, int y, int w, int h) {
		eraser.fill( new Rectangle2D.Double(x,y,w,h) );
	}

	public void drawLine(int x1, int y1, int x2, int y2) {
		pencil.drawLine(x1, y1, x2, y2);
	}

	public void drawImage(BufferedImage image, int x, int y) {
		pencil.drawImage(image, x, y, null);
	}

	//----------------------------------------------------------------

	public void setColor() {
		pencil.setColor(PenColor);
	}


	//----------------------------------------------------------------

	public void clearFrom(int offset) {
		eraser.fill( new Rectangle2D.Double(
  		                 0,offset, getWidth(), getHeight()-offset) );
	}

	//----------------------------------------------------------------

	public void cloneSettingsFrom( JBoardImage old ) {
		setPenColor( old.getPenColor() );
		setPenStyle( old.getPenStyle() );
		setStroke();
		setPointStroke();
	}
	
}
