import java.awt.*;
import java.awt.geom.*;
import java.awt.event.*;
import javax.swing.JComponent; 

public class JGeodreieck 
		extends JComponent
		implements 	MouseListener, MouseMotionListener {

	private JBoardPanel parent;

	int width;
	int height;
	double angle;

	boolean isOutLine, isInLine,
			isInDreieck, isInRotate, isDrawing;

	private static final int MODE_NONE   = 0;
	private static final int MODE_DRIN   = 1;
	private static final int MODE_ROTATE = 2;
	private static final int MODE_LINEAL = 3;
	private static final int MODE_LINE   = 4;
	private static final int MODE_MARK   = 5;
	
	int dMode = MODE_NONE;
	int lMode = MODE_NONE;

	int kante;
	
	private Cursor movCursor = Cursor.getPredefinedCursor(Cursor.HAND_CURSOR);
	private Cursor defCursor = Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR);
	private Cursor linCursor = Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
	private Cursor testCursor = Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR);

	private Point startPoint = null;
	private Point endPoint = null;

    public int getOrigX() { return 1; }

    public int getOrigY() { return 0; }

    public int getOrigWidth() { return 604; }

    public int getOrigHeight() { return 302; }

	public JGeodreieck(JBoardPanel p) {
		parent = p;
        width  = 640;
        height = Math.round(1.0f*width/getOrigWidth()*getOrigHeight());
		angle  = 0.0;
		
		setLayout(null);
		setBounds(0,0,720,720);
		setVisible(false);
		addMouseListener(this);
		addMouseMotionListener(this);
	}

	public void paintComponent (Graphics g) {
		Graphics2D g2d = (Graphics2D) g.create();

		g2d.translate(360, 360);
		g2d.rotate(Math.toRadians(angle));
		g2d.translate(-320, -320);
		double coef = (double) this.width / (double) getOrigWidth();
		g2d.scale(coef, coef);
		paint(g2d);
		g2d.dispose();

		if ( lMode == MODE_MARK && startPoint != null && endPoint != null) {
			Graphics2D g2dd = (Graphics2D) g.create();
			g2dd.setColor( parent.image.getPenColor() );
			g2dd.setStroke( parent.image.getStroke() );
			g2dd.drawLine(startPoint.x, startPoint.y, endPoint.x, endPoint.y);
			g2dd.dispose();
		}

		if ( lMode == MODE_LINE ) {

			boolean showLine = (lMode == MODE_LINE) && 
							   (getKante(startPoint) > 0) &&
						       (getKante(startPoint) == getKante(endPoint));


			if (showLine && startPoint != null && endPoint != null) {

				Point l1 = getKpoint(startPoint);
				Point l2 = getKpoint(endPoint);

				Graphics2D g2dd = (Graphics2D) g.create();
				g2dd.setColor( parent.image.getPenColor() );
				g2dd.setStroke( parent.image.getStroke() );
				g2dd.drawLine(	l1.x, l1.y, l2.x, l2.y  );

				double dx  = l1.x - l2.x;
				double dy  = l1.y - l2.y;
				parent.infolabel.setText(
				  String.format("%,4.1f", Math.sqrt(dx*dx+dy*dy)/40 ));
			}

		}
		
	}

	private Point rotPoint(Point p, int mx, int my, double alpha) {
		Point q = new Point(p);
		double dx,dy, r, phi;
		
		if ( alpha != 0.0 )  {
			dx  = p.x - mx;
			dy  = p.y - my;
			r   = Math.sqrt(dx*dx+dy*dy);
			phi = Math.toDegrees(Math.atan2(dy,dx)) + alpha;
			if ( phi < 0 ) { phi = phi+360.0; }
			q.x = mx + (int) Math.round( r * Math.cos( Math.toRadians(phi) ) );
			q.y = my + (int) Math.round( r * Math.sin( Math.toRadians(phi) ) );
		}

		return q;
	}
	
	private void checkPoint(Point p) {
		Point q = rotPoint(p, 360,360, -angle);
		double dx,dy, r;
		boolean drin;

		isInLine    = false;
		isInDreieck = false;
		isInRotate  = false;

		drin = q.y <= (40+320) + 20;
		if ( q.x <= 40+320 ) {
			isOutLine = drin && q.y >= (40+320+40) - 28 - q.x;
		} else {
			isOutLine = drin && q.y >= q.x - (40+320-40) - 28 ;
		}

		if ( isOutLine ) {
			drin = q.y <= (40+320);
			if ( q.x <= 40+320 ) {
				isInLine = drin && q.y >= (40+320+40) - q.x;
			} else {
				isInLine = drin && q.y >= q.x - (40+320-40);
			}
			
		}

		if ( isInLine ) {
			isOutLine = false;
			drin = q.y <= (40+320) - 20;
			if ( q.x <= (40+320) ) {
				isInDreieck = drin && q.y >= (40+320+40) + 28 - q.x;
			} else {
				isInDreieck = drin && q.y >= q.x - (40+320-40) + 28 ;
			}
		} 
		
		if ( isInDreieck ) {
			isInLine  = false;
			isDrawing = false;

			dx  = q.x - (40+320);
			dy  = q.y - (75 + 40);
			r   = Math.sqrt(dx*dx+dy*dy);
			isInRotate = ( r <= 10.0 );
		} else { isInRotate = false; }

	}

	private double dangle(Point p1, Point p2) {
		double dx,dy,a1,a2;

		dx  = p1.x - 360;
		dy  = p1.y - 360;
		a1 = Math.toDegrees(Math.atan2(dy,dx));
		dx  = p2.x - 360;
		dy  = p2.y - 360;
		a2 = Math.toDegrees(Math.atan2(dy,dx));

		return Math.round(2.0*(a2-a1))/2.0;
	}
	
	private double infoangle() {
		if ( angle < 0 ) {
			return -angle;
		} else if ( angle > 0 && angle <= 180 ) {
			return 180.0-angle;
		} else if ( angle > 180 ) {
			return 360.0-angle; 
		} else {
			return angle;
		}
	}

	private boolean isPointOutside(Point p) {
		Point q = rotPoint(p, 360,360, -angle);

		boolean draussen = q.y > (40+320);
		if ( q.x <= 40+320 ) {
			return draussen || q.y < (40+320+40) - q.x;
		} else {
			return draussen || q.y < q.x - (40+320-40);
		}

	}

	private int getKante(Point p) {
		Point q = rotPoint(p, 360,360, -angle);
		int kante = 0;
		
		if ( (q.y > (40+320)) && (q.x >= 40) && (q.x <= 40+640) ) {
			kante = 1;
		} 
		
		if ( kante == 0 && q.x <= (40+320) ) {
			Point q2 = rotPoint(q, 40, 360, 45.0);
			
			if ( (q2.y < (40+320)) && (q2.x >= 40) && (q2.x <= 40+453) ) {
				kante = 2;
			}
		}

		if ( kante == 0 && q.x > (40+320) ) {
			Point q2 = rotPoint(q, 40+640, 360, -45.0);
			
			if ( (q2.y < (40+320)) && (q2.x >= 40+640-453) && (q2.x <= 40+640) ) {
				kante = 3;
			}
		}

		return kante;
	}

	private Point getKpoint(Point p) {
		Point q = rotPoint(p, 360,360, -angle);

		if ( getKante(p) == 1 ) {
			q.y = 360;
		}

		if ( getKante(p) == 2 ) {
			Point q2 = rotPoint(q, 40, 360, 45.0);
			q2.y = 360;
			q = rotPoint(q2, 40, 360, -45.0);
		}

		if ( getKante(p) == 3 ) {
			Point q2 = rotPoint(q, 40+640, 360, -45.0);
			q2.y = 360;
			q = rotPoint(q2, 40+640, 360, 45.0);
		}
		
		return rotPoint(q, 360,360, angle);
	}
	
	public void mousePressed(MouseEvent e) {
		if ( dMode != MODE_NONE ) {
			startPoint 	= e.getPoint();
			endPoint 	= startPoint;
		}

		if ( dMode == MODE_ROTATE ) {
			parent.infolabel.showLabel(String.format("%,5.1f°", infoangle()),120);
		}

		if ( dMode == MODE_LINEAL ) {
			if (isOutLine) {
				lMode = MODE_LINE;
				parent.infolabel.showLabel(String.format("%,4.1f", 0.0),80);
			} else {
				lMode = MODE_MARK;
			}
		}

    }

    public void mouseReleased(MouseEvent e) {
		if ( dMode == MODE_ROTATE ) {
			parent.infolabel.hideLabel();
		}

		if ( dMode == MODE_LINEAL ) {
			endPoint = e.getPoint();

			if (lMode == MODE_MARK && isPointOutside(endPoint) ) {
				parent.image.setStroke();
				parent.image.setColor();
				parent.image.drawLine(
					getX()+startPoint.x, parent.offset+getY()+startPoint.y, 
					getX()+endPoint.x, parent.offset+getY()+endPoint.y
					);
				parent.repaint();
			}

			if ( lMode == MODE_LINE ) {
				parent.infolabel.hideLabel();
			}

			boolean showLine = (lMode == MODE_LINE) && isPointOutside(endPoint) &&
							   (getKante(startPoint) > 0) &&
						       (getKante(startPoint) == getKante(endPoint));

			if (showLine) {
				parent.image.setStroke();
				parent.image.setColor();
				parent.image.drawLine(
					getX()+getKpoint(startPoint).x, 
					parent.offset+getY()+getKpoint(startPoint).y, 
					getX()+getKpoint(endPoint).x, 
					parent.offset+getY()+getKpoint(endPoint).y
					);
				parent.repaint();
			}


			lMode = MODE_NONE;
			repaint();
		}

    }

    public void mouseEntered(MouseEvent e) {
		checkPoint(e.getPoint());
		dMode = MODE_NONE;
		
		if ( isOutLine || isInLine ) {
			setCursor(linCursor);
			dMode = MODE_LINEAL;
		}

		if ( isInDreieck ) {
			setCursor(movCursor);
			dMode = MODE_DRIN;
		}

		if ( isInRotate ) {
			setCursor(movCursor);
			dMode = MODE_ROTATE;
		}
    }

    public void mouseExited(MouseEvent e) {
		setCursor(defCursor);
		dMode = MODE_NONE;
    }

    public void mouseClicked(MouseEvent e) {
		checkPoint(e.getPoint());
		dMode = MODE_NONE;
		
		if ( isOutLine || isInLine ) {
			setCursor(linCursor);
			dMode = MODE_LINEAL;
		}

		if ( isInDreieck ) {
			setCursor(movCursor);
			dMode = MODE_DRIN;
		}

		if ( isInRotate ) {
			setCursor(movCursor);
			dMode = MODE_ROTATE;
		}
    }	

    public void mouseMoved(MouseEvent e) {
		checkPoint(e.getPoint());
		boolean rep = false;

		if ( !(isOutLine || isInLine || isInDreieck) && dMode != MODE_NONE ) {
			setCursor(defCursor);
			dMode = MODE_NONE;
		}

		if ( (isOutLine || isInLine) && dMode != MODE_LINEAL ) {
			if (dMode == MODE_ROTATE) {
				dMode = MODE_LINEAL;
				repaint();
			} else {
				dMode = MODE_LINEAL;
			}
			setCursor(linCursor);
		}

		if ( isInDreieck && !isInRotate ) {
			switch (dMode) {
			case MODE_NONE:		dMode = MODE_DRIN;
								setCursor(movCursor);	break;
			case MODE_LINEAL:	dMode = MODE_DRIN;
								setCursor(movCursor);	break;
			case MODE_ROTATE:	dMode = MODE_DRIN;
								repaint();				break;
			}
		}

		if ( isInDreieck && isInRotate && dMode != MODE_ROTATE ) {
			dMode = MODE_ROTATE;
			repaint();
		}
    }

    public void mouseDragged(MouseEvent e) {
		endPoint = e.getPoint();

		if ( dMode == MODE_DRIN ) {
			setLocation( getX()+endPoint.x-startPoint.x, 
						getY()+endPoint.y-startPoint.y  );
		}

		if ( dMode == MODE_ROTATE ) {
			angle = angle + dangle(startPoint, endPoint);
			parent.infolabel.setText(String.format("%,5.1f°", infoangle()));
			repaint();
			startPoint 	= e.getPoint();
		}

		if ( dMode == MODE_LINEAL ) {
				repaint();
		}


    }

	public boolean isRotating() {
		return dMode == MODE_ROTATE;
	}

	public double getAngle() {
		return angle;
	}

	/* * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	 * 
	 * original painting of svg image converted to java2d... 
	 * 
	 * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

	private float origAlpha = 1.0f;

	public void paint(Graphics2D g) {
		g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
        origAlpha = 1.0f;
        Composite origComposite = g.getComposite();
        if (origComposite instanceof AlphaComposite) {
            AlphaComposite origAlphaComposite = 
                (AlphaComposite)origComposite;
            if (origAlphaComposite.getRule() == AlphaComposite.SRC_OVER) {
                origAlpha = origAlphaComposite.getAlpha();
            }
        }
        
		// _0
		AffineTransform trans_0 = g.getTransform();
		paintRootGraphicsNode_0(g);
		g.setTransform(trans_0);

	}

	private void paintShapeNode_0_0_0_0_0(Graphics2D g) {
		GeneralPath shape0 = new GeneralPath();
		shape0.moveTo(0.141748, 79.9413);
		shape0.lineTo(79.9999, 0.083029);
		shape0.lineTo(159.858, 79.9413);
		shape0.closePath();
		g.setPaint(new Color(43, 42, 41, 255));
		g.setStroke(new BasicStroke(0.117439f,0,0,4.0f,null,0.0f));
		g.draw(shape0);
	}

	private void paintShapeNode_0_0_0_0_1(Graphics2D g) {
		GeneralPath shape1 = new GeneralPath();
		shape1.moveTo(10.0957, 76.7467);
		shape1.lineTo(10.0957, 79.941025);
		shape1.moveTo(20.07797, 76.7467);
		shape1.lineTo(20.07797, 79.941025);
		shape1.moveTo(30.060242, 76.7467);
		shape1.lineTo(30.060242, 79.941025);
		shape1.moveTo(40.04251, 76.7467);
		shape1.lineTo(40.04251, 79.941025);
		shape1.moveTo(50.02478, 76.7467);
		shape1.lineTo(50.02478, 79.941025);
		shape1.moveTo(60.00705, 76.7467);
		shape1.lineTo(60.00705, 79.941025);
		shape1.moveTo(69.98932, 76.7467);
		shape1.lineTo(69.98932, 79.941025);
		shape1.moveTo(79.97159, 76.7467);
		shape1.lineTo(79.97159, 79.941025);
		shape1.moveTo(89.95386, 76.7467);
		shape1.lineTo(89.95386, 79.941025);
		shape1.moveTo(99.93613, 76.7467);
		shape1.lineTo(99.93613, 79.941025);
		shape1.moveTo(109.918396, 76.7467);
		shape1.lineTo(109.918396, 79.941025);
		shape1.moveTo(119.900665, 76.7467);
		shape1.lineTo(119.900665, 79.941025);
		shape1.moveTo(129.88293, 76.7467);
		shape1.lineTo(129.88293, 79.941025);
		shape1.moveTo(139.8652, 76.7467);
		shape1.lineTo(139.8652, 79.941025);
		shape1.moveTo(149.84747, 76.7467);
		shape1.lineTo(149.84747, 79.941025);
		shape1.moveTo(15.086472, 76.94634);
		shape1.lineTo(15.086472, 79.941025);
		shape1.moveTo(25.06874, 76.94634);
		shape1.lineTo(25.06874, 79.941025);
		shape1.moveTo(35.05101, 76.94634);
		shape1.lineTo(35.05101, 79.941025);
		shape1.moveTo(45.03328, 76.94634);
		shape1.lineTo(45.03328, 79.941025);
		shape1.moveTo(55.01555, 76.94634);
		shape1.lineTo(55.01555, 79.941025);
		shape1.moveTo(64.99782, 76.94634);
		shape1.lineTo(64.99782, 79.941025);
		shape1.moveTo(74.98009, 76.94634);
		shape1.lineTo(74.98009, 79.941025);
		shape1.moveTo(84.96236, 76.94634);
		shape1.lineTo(84.96236, 79.941025);
		shape1.moveTo(94.944626, 76.94634);
		shape1.lineTo(94.944626, 79.941025);
		shape1.moveTo(104.926895, 76.94634);
		shape1.lineTo(104.926895, 79.941025);
		shape1.moveTo(114.909164, 76.94634);
		shape1.lineTo(114.909164, 79.941025);
		shape1.moveTo(124.89143, 76.94634);
		shape1.lineTo(124.89143, 79.941025);
		shape1.moveTo(134.8737, 76.94634);
		shape1.lineTo(134.8737, 79.941025);
		shape1.moveTo(144.85597, 76.94634);
		shape1.lineTo(144.85597, 79.941025);
		shape1.moveTo(11.093979, 77.94457);
		shape1.lineTo(11.093979, 79.941025);
		shape1.moveTo(12.092206, 77.94457);
		shape1.lineTo(12.092206, 79.941025);
		shape1.moveTo(13.090433, 77.94457);
		shape1.lineTo(13.090433, 79.941025);
		shape1.moveTo(14.08866, 77.94457);
		shape1.lineTo(14.08866, 79.941025);
		shape1.moveTo(16.08511, 77.94457);
		shape1.lineTo(16.08511, 79.941025);
		shape1.moveTo(17.083336, 77.94457);
		shape1.lineTo(17.083336, 79.941025);
		shape1.moveTo(18.081562, 77.94457);
		shape1.lineTo(18.081562, 79.941025);
		shape1.moveTo(19.079788, 77.94457);
		shape1.lineTo(19.079788, 79.941025);
		shape1.moveTo(21.076239, 77.94457);
		shape1.lineTo(21.076239, 79.941025);
		shape1.moveTo(22.074465, 77.94457);
		shape1.lineTo(22.074465, 79.941025);
		shape1.moveTo(23.072691, 77.94457);
		shape1.lineTo(23.072691, 79.941025);
		shape1.moveTo(24.070917, 77.94457);
		shape1.lineTo(24.070917, 79.941025);
		shape1.moveTo(26.067368, 77.94457);
		shape1.lineTo(26.067368, 79.941025);
		shape1.moveTo(27.065594, 77.94457);
		shape1.lineTo(27.065594, 79.941025);
		shape1.moveTo(28.06382, 77.94457);
		shape1.lineTo(28.06382, 79.941025);
		shape1.moveTo(29.062046, 77.94457);
		shape1.lineTo(29.062046, 79.941025);
		shape1.moveTo(31.058496, 77.94457);
		shape1.lineTo(31.058496, 79.941025);
		shape1.moveTo(32.056725, 77.94457);
		shape1.lineTo(32.056725, 79.941025);
		shape1.moveTo(33.05495, 77.94457);
		shape1.lineTo(33.05495, 79.941025);
		shape1.moveTo(34.053177, 77.94457);
		shape1.lineTo(34.053177, 79.941025);
		shape1.moveTo(36.049625, 77.94457);
		shape1.lineTo(36.049625, 79.941025);
		shape1.moveTo(37.04785, 77.94457);
		shape1.lineTo(37.04785, 79.941025);
		shape1.moveTo(38.046078, 77.94457);
		shape1.lineTo(38.046078, 79.941025);
		shape1.moveTo(39.044304, 77.94457);
		shape1.lineTo(39.044304, 79.941025);
		shape1.moveTo(41.040752, 77.94457);
		shape1.lineTo(41.040752, 79.941025);
		shape1.moveTo(42.03898, 77.94457);
		shape1.lineTo(42.03898, 79.941025);
		shape1.moveTo(43.037205, 77.94457);
		shape1.lineTo(43.037205, 79.941025);
		shape1.moveTo(44.03543, 77.94457);
		shape1.lineTo(44.03543, 79.941025);
		shape1.moveTo(46.03188, 77.94457);
		shape1.lineTo(46.03188, 79.941025);
		shape1.moveTo(47.030106, 77.94457);
		shape1.lineTo(47.030106, 79.941025);
		shape1.moveTo(48.02833, 77.94457);
		shape1.lineTo(48.02833, 79.941025);
		shape1.moveTo(49.026558, 77.94457);
		shape1.lineTo(49.026558, 79.941025);
		shape1.moveTo(51.023006, 77.94457);
		shape1.lineTo(51.023006, 79.941025);
		shape1.moveTo(52.021233, 77.94457);
		shape1.lineTo(52.021233, 79.941025);
		shape1.moveTo(53.01946, 77.94457);
		shape1.lineTo(53.01946, 79.941025);
		shape1.moveTo(54.017685, 77.94457);
		shape1.lineTo(54.017685, 79.941025);
		shape1.moveTo(56.014133, 77.94457);
		shape1.lineTo(56.014133, 79.941025);
		shape1.moveTo(57.01236, 77.94457);
		shape1.lineTo(57.01236, 79.941025);
		shape1.moveTo(58.010586, 77.94457);
		shape1.lineTo(58.010586, 79.941025);
		shape1.moveTo(59.008812, 77.94457);
		shape1.lineTo(59.008812, 79.941025);
		shape1.moveTo(61.00526, 77.94457);
		shape1.lineTo(61.00526, 79.941025);
		shape1.moveTo(62.003487, 77.94457);
		shape1.lineTo(62.003487, 79.941025);
		shape1.moveTo(63.001713, 77.94457);
		shape1.lineTo(63.001713, 79.941025);
		shape1.moveTo(63.99994, 77.94457);
		shape1.lineTo(63.99994, 79.941025);
		shape1.moveTo(65.99639, 77.94457);
		shape1.lineTo(65.99639, 79.941025);
		shape1.moveTo(66.99462, 77.94457);
		shape1.lineTo(66.99462, 79.941025);
		shape1.moveTo(67.99285, 77.94457);
		shape1.lineTo(67.99285, 79.941025);
		shape1.moveTo(68.99108, 77.94457);
		shape1.lineTo(68.99108, 79.941025);
		shape1.moveTo(70.98753, 77.94457);
		shape1.lineTo(70.98753, 79.941025);
		shape1.moveTo(71.98576, 77.94457);
		shape1.lineTo(71.98576, 79.941025);
		shape1.moveTo(72.98399, 77.94457);
		shape1.lineTo(72.98399, 79.941025);
		shape1.moveTo(73.98222, 77.94457);
		shape1.lineTo(73.98222, 79.941025);
		shape1.moveTo(75.978676, 77.94457);
		shape1.lineTo(75.978676, 79.941025);
		shape1.moveTo(76.976906, 77.94457);
		shape1.lineTo(76.976906, 79.941025);
		shape1.moveTo(77.975136, 77.94457);
		shape1.lineTo(77.975136, 79.941025);
		shape1.moveTo(78.973366, 77.94457);
		shape1.lineTo(78.973366, 79.941025);
		shape1.moveTo(80.96982, 77.94457);
		shape1.lineTo(80.96982, 79.941025);
		shape1.moveTo(81.96805, 77.94457);
		shape1.lineTo(81.96805, 79.941025);
		shape1.moveTo(82.96628, 77.94457);
		shape1.lineTo(82.96628, 79.941025);
		shape1.moveTo(83.96451, 77.94457);
		shape1.lineTo(83.96451, 79.941025);
		shape1.moveTo(85.96096, 77.94457);
		shape1.lineTo(85.96096, 79.941025);
		shape1.moveTo(86.95919, 77.94457);
		shape1.lineTo(86.95919, 79.941025);
		shape1.moveTo(87.95742, 77.94457);
		shape1.lineTo(87.95742, 79.941025);
		shape1.moveTo(88.95565, 77.94457);
		shape1.lineTo(88.95565, 79.941025);
		shape1.moveTo(90.9521, 77.94457);
		shape1.lineTo(90.9521, 79.941025);
		shape1.moveTo(91.95033, 77.94457);
		shape1.lineTo(91.95033, 79.941025);
		shape1.moveTo(92.94856, 77.94457);
		shape1.lineTo(92.94856, 79.941025);
		shape1.moveTo(93.94679, 77.94457);
		shape1.lineTo(93.94679, 79.941025);
		shape1.moveTo(95.943245, 77.94457);
		shape1.lineTo(95.943245, 79.941025);
		shape1.moveTo(96.941475, 77.94457);
		shape1.lineTo(96.941475, 79.941025);
		shape1.moveTo(97.939705, 77.94457);
		shape1.lineTo(97.939705, 79.941025);
		shape1.moveTo(98.937935, 77.94457);
		shape1.lineTo(98.937935, 79.941025);
		shape1.moveTo(140.86353, 77.94457);
		shape1.lineTo(140.86353, 79.941025);
		shape1.moveTo(141.86176, 77.94457);
		shape1.lineTo(141.86176, 79.941025);
		shape1.moveTo(100.934456, 77.94457);
		shape1.lineTo(100.934456, 79.941025);
		shape1.moveTo(142.86005, 77.94457);
		shape1.lineTo(142.86005, 79.941025);
		shape1.moveTo(101.93275, 77.94457);
		shape1.lineTo(101.93275, 79.941025);
		shape1.moveTo(143.85834, 77.94457);
		shape1.lineTo(143.85834, 79.941025);
		shape1.moveTo(102.93104, 77.94457);
		shape1.lineTo(102.93104, 79.941025);
		shape1.moveTo(103.92927, 77.94457);
		shape1.lineTo(103.92927, 79.941025);
		shape1.moveTo(145.85486, 77.94457);
		shape1.lineTo(145.85486, 79.941025);
		shape1.moveTo(146.85309, 77.94457);
		shape1.lineTo(146.85309, 79.941025);
		shape1.moveTo(105.92579, 77.94457);
		shape1.lineTo(105.92579, 79.941025);
		shape1.moveTo(147.85138, 77.94457);
		shape1.lineTo(147.85138, 79.941025);
		shape1.moveTo(106.92408, 77.94457);
		shape1.lineTo(106.92408, 79.941025);
		shape1.moveTo(148.84967, 77.94457);
		shape1.lineTo(148.84967, 79.941025);
		shape1.moveTo(107.92237, 77.94457);
		shape1.lineTo(107.92237, 79.941025);
		shape1.moveTo(108.9206, 77.94457);
		shape1.lineTo(108.9206, 79.941025);
		shape1.moveTo(110.91705, 77.94457);
		shape1.lineTo(110.91705, 79.941025);
		shape1.moveTo(111.91528, 77.94457);
		shape1.lineTo(111.91528, 79.941025);
		shape1.moveTo(112.91351, 77.94457);
		shape1.lineTo(112.91351, 79.941025);
		shape1.moveTo(113.91174, 77.94457);
		shape1.lineTo(113.91174, 79.941025);
		shape1.moveTo(115.908195, 77.94457);
		shape1.lineTo(115.908195, 79.941025);
		shape1.moveTo(116.906425, 77.94457);
		shape1.lineTo(116.906425, 79.941025);
		shape1.moveTo(117.904655, 77.94457);
		shape1.lineTo(117.904655, 79.941025);
		shape1.moveTo(118.902885, 77.94457);
		shape1.lineTo(118.902885, 79.941025);
		shape1.moveTo(120.89934, 77.94457);
		shape1.lineTo(120.89934, 79.941025);
		shape1.moveTo(121.89757, 77.94457);
		shape1.lineTo(121.89757, 79.941025);
		shape1.moveTo(122.8958, 77.94457);
		shape1.lineTo(122.8958, 79.941025);
		shape1.moveTo(123.89403, 77.94457);
		shape1.lineTo(123.89403, 79.941025);
		shape1.moveTo(125.89048, 77.94457);
		shape1.lineTo(125.89048, 79.941025);
		shape1.moveTo(126.88871, 77.94457);
		shape1.lineTo(126.88871, 79.941025);
		shape1.moveTo(127.88694, 77.94457);
		shape1.lineTo(127.88694, 79.941025);
		shape1.moveTo(128.88516, 77.94457);
		shape1.lineTo(128.88516, 79.941025);
		shape1.moveTo(130.8816, 77.94457);
		shape1.lineTo(130.8816, 79.941025);
		shape1.moveTo(131.87984, 77.94457);
		shape1.lineTo(131.87984, 79.941025);
		shape1.moveTo(132.87807, 77.94457);
		shape1.lineTo(132.87807, 79.941025);
		shape1.moveTo(133.8763, 77.94457);
		shape1.lineTo(133.8763, 79.941025);
		shape1.moveTo(135.87274, 77.94457);
		shape1.lineTo(135.87274, 79.941025);
		shape1.moveTo(136.87097, 77.94457);
		shape1.lineTo(136.87097, 79.941025);
		shape1.moveTo(137.8692, 77.94457);
		shape1.lineTo(137.8692, 79.941025);
		shape1.moveTo(138.86743, 77.94457);
		shape1.lineTo(138.86743, 79.941025);
		g.draw(shape1);
	}

	private void paintShapeNode_0_0_0_0_2(Graphics2D g) {
		GeneralPath shape2 = new GeneralPath();
		shape2.moveTo(122.76, 76.1998);
		shape2.lineTo(124.74894, 76.02581);
		shape2.moveTo(122.27745, 72.48615);
		shape2.lineTo(124.24384, 72.13947);
		shape2.moveTo(121.46877, 68.829475);
		shape2.lineTo(123.397644, 68.31257);
		shape2.moveTo(120.3456, 65.25641);
		shape2.lineTo(122.22227, 64.573326);
		shape2.moveTo(118.90874, 61.797436);
		shape2.lineTo(120.718636, 60.953583);
		shape2.moveTo(117.1784, 58.476044);
		shape2.lineTo(118.90786, 57.47764);
		shape2.moveTo(115.16239, 55.32029);
		shape2.lineTo(116.79796, 54.17497);
		shape2.moveTo(112.880325, 52.35133);
		shape2.lineTo(114.40984, 51.0679);
		shape2.moveTo(110.35194, 49.58935);
		shape2.lineTo(111.76384, 48.177452);
		shape2.moveTo(107.58996, 47.060963);
		shape2.lineTo(108.87339, 45.53144);
		shape2.moveTo(104.62112, 44.77901);
		shape2.lineTo(105.76638, 43.143272);
		shape2.moveTo(101.46514, 42.76289);
		shape2.lineTo(102.46366, 41.03343);
		shape2.moveTo(98.14374, 41.032547);
		shape2.lineTo(98.987595, 39.22265);
		shape2.moveTo(94.68488, 39.59581);
		shape2.lineTo(95.367966, 37.71903);
		shape2.moveTo(91.11182, 38.472572);
		shape2.lineTo(91.62866, 36.543644);
		shape2.moveTo(87.4549, 37.663944);
		shape2.lineTo(87.8017, 35.697315);
		shape2.moveTo(83.74138, 37.181385);
		shape2.lineTo(83.91537, 35.192326);
		shape2.moveTo(80.000435, 37.017265);
		shape2.lineTo(80.000435, 35.020634);
		shape2.moveTo(76.25938, 37.181385);
		shape2.lineTo(76.08539, 35.192326);
		shape2.moveTo(72.545906, 37.663937);
		shape2.lineTo(72.19911, 35.697426);
		shape2.moveTo(68.88899, 38.472557);
		shape2.lineTo(68.372086, 36.54363);
		shape2.moveTo(65.3158, 39.5958);
		shape2.lineTo(64.63284, 37.719128);
		shape2.moveTo(61.857063, 41.032417);
		shape2.lineTo(61.01315, 39.222626);
		shape2.moveTo(58.53555, 42.762867);
		shape2.lineTo(57.537144, 41.033405);
		shape2.moveTo(55.379684, 44.778984);
		shape2.lineTo(54.234425, 43.143364);
		shape2.moveTo(52.410725, 47.060936);
		shape2.lineTo(51.127415, 45.531536);
		shape2.moveTo(49.648624, 49.589325);
		shape2.lineTo(48.236954, 48.177536);
		shape2.moveTo(47.120342, 52.351295);
		shape2.lineTo(45.590942, 51.067986);
		shape2.moveTo(44.83828, 55.320255);
		shape2.lineTo(43.20289, 54.175056);
		shape2.moveTo(42.82215, 58.476128);
		shape2.lineTo(41.09293, 57.477722);
		shape2.moveTo(41.091816, 61.79764);
		shape2.lineTo(39.282146, 60.953728);
		shape2.moveTo(39.65507, 65.2565);
		shape2.lineTo(37.77858, 64.57348);
		shape2.moveTo(38.531887, 68.82945);
		shape2.lineTo(36.603077, 68.31267);
		shape2.moveTo(37.723206, 72.486374);
		shape2.lineTo(35.756874, 72.139694);
		shape2.moveTo(37.240654, 76.19995);
		shape2.lineTo(35.251892, 76.02596);
		shape2.moveTo(122.69119, 75.454094);
		shape2.lineTo(123.68431, 75.34963);
		shape2.moveTo(122.60611, 74.709595);
		shape2.lineTo(123.59729, 74.58793);
		shape2.moveTo(122.50822, 73.9668);
		shape2.lineTo(123.49717, 73.827866);
		shape2.moveTo(122.39988, 73.225525);
		shape2.lineTo(123.386246, 73.06933);
		shape2.moveTo(122.140045, 71.74985);
		shape2.lineTo(123.12031, 71.5593);
		shape2.moveTo(121.99214, 71.01533);
		shape2.lineTo(122.968994, 70.80769);
		shape2.moveTo(121.83277, 70.28327);
		shape2.lineTo(122.80587, 70.058556);
		shape2.moveTo(121.65603, 69.55504);
		shape2.lineTo(122.625015, 69.31347);
		shape2.moveTo(121.271, 68.10685);
		shape2.lineTo(122.23089, 67.831635);
		shape2.moveTo(121.05739, 67.38872);
		shape2.lineTo(122.01228, 67.09671);
		shape2.moveTo(120.8312, 66.674286);
		shape2.lineTo(121.78087, 66.365715);
		shape2.moveTo(120.594505, 65.96331);
		shape2.lineTo(121.53871, 65.63824);
		shape2.moveTo(120.08112, 64.55546);
		shape2.lineTo(121.013466, 64.19756);
		shape2.moveTo(119.80631, 63.858402);
		shape2.lineTo(120.732254, 63.48424);
		shape2.moveTo(119.521225, 63.165394);
		shape2.lineTo(120.44059, 62.775204);
		shape2.moveTo(119.22093, 62.47891);
		shape2.lineTo(120.133194, 62.072746);
		shape2.moveTo(118.58665, 61.121258);
		shape2.lineTo(119.48417, 60.683506);
		shape2.moveTo(118.25312, 60.450214);
		shape2.lineTo(119.14295, 59.996845);
		shape2.moveTo(117.90474, 59.786922);
		shape2.lineTo(118.78641, 59.318165);
		shape2.moveTo(117.546486, 59.128853);
		shape2.lineTo(118.41976, 58.64483);
		shape2.moveTo(116.79711, 57.83139);
		shape2.lineTo(117.653, 57.31707);
		shape2.moveTo(116.40392, 57.193523);
		shape2.lineTo(117.250595, 56.664463);
		shape2.moveTo(116.00123, 56.561703);
		shape2.lineTo(116.83856, 56.01802);
		shape2.moveTo(115.58925, 55.93605);
		shape2.lineTo(116.41689, 55.377747);
		shape2.moveTo(114.7263, 54.711105);
		shape2.lineTo(115.53387, 54.124382);
		shape2.moveTo(114.281265, 54.108585);
		shape2.lineTo(115.07844, 53.507828);
		shape2.moveTo(113.82537, 53.514053);
		shape2.lineTo(114.61198, 52.89944);
		shape2.moveTo(113.35721, 52.929153);
		shape2.lineTo(114.13295, 52.301094);
		shape2.moveTo(112.39492, 51.780724);
		shape2.lineTo(113.148285, 51.125885);
		shape2.moveTo(111.8978, 51.220425);
		shape2.lineTo(112.639595, 50.552494);
		shape2.moveTo(111.39057, 50.66911);
		shape2.lineTo(112.12063, 49.988377);
		shape2.moveTo(110.875305, 50.12537);
		shape2.lineTo(111.59332, 49.432014);
		shape2.moveTo(109.81601, 49.066017);
		shape2.lineTo(110.50937, 48.348);
		shape2.moveTo(109.27227, 48.55076);
		shape2.lineTo(109.95294, 47.82076);
		shape2.moveTo(108.720955, 48.04354);
		shape2.lineTo(109.38877, 47.301857);
		shape2.moveTo(108.16054, 47.546364);
		shape2.lineTo(108.81538, 46.793056);
		shape2.moveTo(107.01217, 46.584133);
		shape2.lineTo(107.64023, 45.80851);
		shape2.moveTo(106.427315, 46.115963);
		shape2.lineTo(107.04181, 45.329475);
		shape2.moveTo(105.83278, 45.660065);
		shape2.lineTo(106.43336, 44.86301);
		shape2.moveTo(105.230194, 45.21509);
		shape2.lineTo(105.81686, 44.407642);
		shape2.moveTo(104.00536, 44.352093);
		shape2.lineTo(104.56349, 43.52468);
		shape2.moveTo(103.37959, 43.94012);
		shape2.lineTo(103.92322, 43.10302);
		shape2.moveTo(102.74772, 43.537483);
		shape2.lineTo(103.276665, 42.690987);
		shape2.moveTo(102.11003, 43.14418);
		shape2.lineTo(102.624115, 42.28858);
		shape2.moveTo(100.81257, 42.394863);
		shape2.lineTo(101.29642, 41.521942);
		shape2.moveTo(100.15445, 42.036617);
		shape2.lineTo(100.623085, 41.1553);
		shape2.moveTo(99.4911, 41.688118);
		shape2.lineTo(99.94429, 40.798817);
		shape2.moveTo(98.82011, 41.354713);
		shape2.lineTo(99.25762, 40.45766);
		shape2.moveTo(97.46251, 40.72049);
		shape2.lineTo(97.86849, 39.80858);
		shape2.moveTo(96.7759, 40.4202);
		shape2.lineTo(97.16603, 39.501244);
		shape2.moveTo(96.082954, 40.135002);
		shape2.lineTo(96.45688, 39.20953);
		shape2.moveTo(95.3859, 39.8602);
		shape2.lineTo(95.74356, 38.928444);
		shape2.moveTo(93.97799, 39.346817);
		shape2.lineTo(94.30289, 38.40308);
		shape2.moveTo(93.26696, 39.110237);
		shape2.lineTo(93.57541, 38.16104);
		shape2.moveTo(92.5527, 38.884052);
		shape2.lineTo(92.844475, 37.92963);
		shape2.moveTo(91.8345, 38.670315);
		shape2.lineTo(92.109604, 37.711018);
		shape2.moveTo(90.38619, 38.28529);
		shape2.lineTo(90.62765, 37.31689);
		shape2.moveTo(89.65807, 38.108604);
		shape2.lineTo(89.88256, 37.136154);
		shape2.moveTo(88.92585, 37.94918);
		shape2.lineTo(89.13342, 36.97303);
		shape2.moveTo(88.19145, 37.801266);
		shape2.lineTo(88.38182, 36.821712);
		shape2.moveTo(86.71572, 37.541435);
		shape2.lineTo(86.871796, 36.555775);
		shape2.moveTo(85.97439, 37.433098);
		shape2.lineTo(86.11326, 36.44497);
		shape2.moveTo(85.23164, 37.335213);
		shape2.lineTo(85.353195, 36.344852);
		shape2.moveTo(84.48726, 37.250126);
		shape2.lineTo(84.591545, 36.25777);
		shape2.moveTo(82.994385, 37.122883);
		shape2.lineTo(83.063965, 36.12765);
		shape2.moveTo(82.246826, 37.075558);
		shape2.lineTo(82.29903, 36.07909);
		shape2.moveTo(81.49815, 37.046078);
		shape2.lineTo(81.532974, 36.049084);
		shape2.moveTo(80.74937, 37.02676);
		shape2.lineTo(80.76675, 36.02918);
		shape2.moveTo(79.251144, 37.02676);
		shape2.lineTo(79.233765, 36.02918);
		shape2.moveTo(78.50236, 37.046078);
		shape2.lineTo(78.46754, 36.049084);
		shape2.moveTo(77.75381, 37.075554);
		shape2.lineTo(77.70161, 36.079205);
		shape2.moveTo(77.006134, 37.122883);
		shape2.lineTo(76.936554, 36.12765);
		shape2.moveTo(75.51325, 37.25013);
		shape2.lineTo(75.408966, 36.257893);
		shape2.moveTo(74.76893, 37.335213);
		shape2.lineTo(74.64732, 36.344852);
		shape2.moveTo(74.02613, 37.433094);
		shape2.lineTo(73.88726, 36.445084);
		shape2.moveTo(73.28486, 37.541435);
		shape2.lineTo(73.12872, 36.55595);
		shape2.moveTo(71.80907, 37.80127);
		shape2.lineTo(71.6187, 36.821774);
		shape2.moveTo(71.07455, 37.949184);
		shape2.lineTo(70.86715, 36.973152);
		shape2.moveTo(70.34244, 38.108494);
		shape2.lineTo(70.11795, 37.13622);
		shape2.moveTo(69.61426, 38.285297);
		shape2.lineTo(69.37286, 37.317135);
		shape2.moveTo(68.166, 38.670326);
		shape2.lineTo(67.89102, 37.711205);
		shape2.moveTo(67.44781, 38.884064);
		shape2.lineTo(67.15609, 37.929817);
		shape2.moveTo(66.73349, 39.110138);
		shape2.lineTo(66.425095, 38.161236);
		shape2.moveTo(66.022514, 39.346836);
		shape2.lineTo(65.69768, 38.403336);
		shape2.moveTo(64.6146, 39.860214);
		shape2.lineTo(64.25706, 38.92869);
		shape2.moveTo(63.917484, 40.13502);
		shape2.lineTo(63.543736, 39.2099);
		shape2.moveTo(63.22448, 40.4201);
		shape2.lineTo(62.834583, 39.501556);
		shape2.moveTo(62.53811, 40.720398);
		shape2.lineTo(62.13224, 39.80884);
		shape2.moveTo(61.1804, 41.354683);
		shape2.lineTo(60.743, 40.457863);
		shape2.moveTo(60.509415, 41.688152);
		shape2.lineTo(60.056396, 40.799084);
		shape2.moveTo(59.846066, 42.036655);
		shape2.lineTo(59.3776, 41.155632);
		shape2.moveTo(59.18794, 42.39484);
		shape2.lineTo(58.704327, 41.522274);
		shape2.moveTo(57.890476, 43.144215);
		shape2.lineTo(57.376564, 42.28891);
		shape2.moveTo(57.252666, 43.5374);
		shape2.lineTo(56.72402, 42.691315);
		shape2.moveTo(56.62079, 43.940094);
		shape2.lineTo(56.07746, 43.103344);
		shape2.moveTo(55.995136, 44.352123);
		shape2.lineTo(55.437244, 43.525005);
		shape2.moveTo(54.770313, 45.215004);
		shape2.lineTo(54.18394, 44.407967);
		shape2.moveTo(54.167675, 45.660095);
		shape2.lineTo(53.56733, 44.863453);
		shape2.moveTo(53.573143, 46.115993);
		shape2.lineTo(52.95894, 45.329918);
		shape2.moveTo(52.98824, 46.58416);
		shape2.lineTo(52.36047, 45.80895);
		shape2.moveTo(51.839924, 47.546387);
		shape2.lineTo(51.18538, 46.793488);
		shape2.moveTo(51.279625, 48.043507);
		shape2.lineTo(50.612164, 47.302177);
		shape2.moveTo(50.728252, 48.550667);
		shape2.lineTo(50.04799, 47.82108);
		shape2.moveTo(50.18457, 49.06599);
		shape2.lineTo(49.491566, 48.348324);
		shape2.moveTo(49.125217, 50.125282);
		shape2.lineTo(48.407608, 49.432335);
		shape2.moveTo(48.609955, 50.669025);
		shape2.lineTo(47.88031, 49.98876);
		shape2.moveTo(48.102737, 51.22034);
		shape2.lineTo(47.361465, 50.55282);
		shape2.moveTo(47.605503, 51.780758);
		shape2.lineTo(46.852665, 51.126213);
		shape2.moveTo(46.64321, 52.929134);
		shape2.lineTo(45.868, 52.301308);
		shape2.moveTo(46.17516, 53.513977);
		shape2.lineTo(45.388966, 52.899773);
		shape2.moveTo(45.71926, 54.108513);
		shape2.lineTo(44.922558, 53.508167);
		shape2.moveTo(45.27417, 54.711147);
		shape2.lineTo(44.467075, 54.124775);
		shape2.moveTo(44.411175, 55.935974);
		shape2.lineTo(43.584057, 55.378082);
		shape2.moveTo(43.99914, 56.561863);
		shape2.lineTo(43.16239, 56.018417);
		shape2.moveTo(43.59656, 57.19357);
		shape2.lineTo(42.750416, 56.664803);
		shape2.moveTo(43.20332, 57.831203);
		shape2.lineTo(42.348072, 57.317352);
		shape2.moveTo(42.45394, 59.128902);
		shape2.lineTo(41.581314, 58.645172);
		shape2.moveTo(42.095695, 59.786972);
		shape2.lineTo(41.21479, 59.318512);
		shape2.moveTo(41.747314, 60.45021);
		shape2.lineTo(40.85819, 59.997192);
		shape2.moveTo(41.41379, 61.121372);
		shape2.lineTo(40.516914, 60.68397);
		shape2.moveTo(40.779564, 62.47908);
		shape2.lineTo(39.86783, 62.07321);
		shape2.moveTo(40.479153, 63.16568);
		shape2.lineTo(39.56049, 62.775665);
		shape2.moveTo(40.19407, 63.858624);
		shape2.lineTo(39.26883, 63.484756);
		shape2.moveTo(39.919262, 64.55568);
		shape2.lineTo(38.98774, 64.198135);
		shape2.moveTo(39.40588, 65.963585);
		shape2.lineTo(38.46232, 65.63863);
		shape2.moveTo(39.1693, 66.67451);
		shape2.lineTo(38.22028, 66.36617);
		shape2.moveTo(38.943115, 67.38895);
		shape2.lineTo(37.98887, 67.09717);
		shape2.moveTo(38.729378, 68.10708);
		shape2.lineTo(37.7702, 67.83204);
		shape2.moveTo(38.34447, 69.55539);
		shape2.lineTo(37.376072, 69.313934);
		shape2.moveTo(38.167667, 70.28362);
		shape2.lineTo(37.19528, 70.05908);
		shape2.moveTo(38.008247, 71.01562);
		shape2.lineTo(37.032215, 70.80816);
		shape2.moveTo(37.86039, 71.75008);
		shape2.lineTo(36.880836, 71.55977);
		shape2.moveTo(37.60056, 73.22587);
		shape2.lineTo(36.614956, 73.06973);
		shape2.moveTo(37.49228, 73.96714);
		shape2.lineTo(36.50415, 73.82833);
		shape2.moveTo(37.394333, 74.709946);
		shape2.lineTo(36.403915, 74.588394);
		shape2.moveTo(37.30919, 75.454445);
		shape2.lineTo(36.316833, 75.3501);
		g.draw(shape2);
	}

	private void paintShapeNode_0_0_0_0_3(Graphics2D g) {
		GeneralPath shape3 = new GeneralPath();
		shape3.moveTo(154.564, 78.6395);
		shape3.lineTo(158.48833, 78.57104);
		shape3.moveTo(153.30571, 77.38115);
		shape3.lineTo(157.16368, 77.24662);
		shape3.moveTo(152.08759, 76.16301);
		shape3.lineTo(155.88156, 75.96424);
		shape3.moveTo(150.90729, 74.982925);
		shape3.lineTo(154.63908, 74.72174);
		shape3.moveTo(148.65013, 72.7257);
		shape3.lineTo(152.26318, 72.34561);
		shape3.moveTo(147.56917, 71.64445);
		shape3.lineTo(151.12521, 71.20811);
		shape3.moveTo(146.51723, 70.59273);
		shape3.lineTo(150.0179, 70.1006);
		shape3.moveTo(145.49265, 69.56825);
		shape3.lineTo(148.9394, 69.022224);
		shape3.moveTo(143.51886, 67.59412);
		shape3.lineTo(146.86182, 66.9445);
		shape3.moveTo(142.56668, 66.64222);
		shape3.lineTo(145.85954, 65.94199);
		shape3.moveTo(141.63593, 65.7114);
		shape3.lineTo(144.87976, 64.9625);
		shape3.moveTo(140.72514, 64.800316);
		shape3.lineTo(143.92105, 64.003494);
		shape3.moveTo(138.9595, 63.034863);
		shape3.lineTo(142.06247, 62.145206);
		shape3.moveTo(138.1021, 62.17756);
		shape3.lineTo(141.15996, 61.242516);
		shape3.moveTo(137.26071, 61.33588);
		shape3.lineTo(140.27425, 60.356796);
		shape3.moveTo(136.434, 60.509525);
		shape3.lineTo(139.40408, 59.486454);
		shape3.moveTo(134.82175, 58.897266);
		shape3.lineTo(137.70691, 57.789356);
		shape3.moveTo(134.03442, 58.109905);
		shape3.lineTo(136.87825, 56.960716);
		shape3.moveTo(133.25879, 57.33429);
		shape3.lineTo(136.0617, 56.14429);
		shape3.moveTo(132.49399, 56.56936);
		shape3.lineTo(135.25667, 55.33954);
		shape3.moveTo(130.9943, 55.069607);
		shape3.lineTo(133.67805, 53.760517);
		shape3.moveTo(130.25801, 54.333324);
		shape3.lineTo(132.90314, 52.985775);
		shape3.moveTo(129.53012, 53.60538);
		shape3.lineTo(132.1369, 52.21955);
		shape3.moveTo(128.80998, 52.885605);
		shape3.lineTo(131.37883, 51.461723);
		shape3.moveTo(127.39068, 51.465893);
		shape3.lineTo(129.88478, 49.967552);
		shape3.moveTo(126.69045, 50.765724);
		shape3.lineTo(129.14767, 49.230274);
		shape3.moveTo(125.995804, 50.071075);
		shape3.lineTo(128.41656, 48.499165);
		shape3.moveTo(125.30633, 49.381714);
		shape3.lineTo(127.690796, 47.773155);
		shape3.moveTo(123.940926, 48.01643);
		shape3.lineTo(126.253525, 46.33624);
		shape3.moveTo(123.26412, 47.33963);
		shape3.lineTo(125.541084, 45.62379);
		shape3.moveTo(122.59055, 46.666122);
		shape3.lineTo(124.83205, 44.91476);
		shape3.moveTo(121.91993, 45.99549);
		shape3.lineTo(124.12613, 44.2089);
		shape3.moveTo(120.58548, 44.660686);
		shape3.lineTo(122.72151, 42.804157);
		shape3.moveTo(119.92089, 43.996037);
		shape3.lineTo(122.02192, 42.104515);
		shape3.moveTo(119.25765, 43.332867);
		shape3.lineTo(121.32369, 41.406166);
		shape3.moveTo(118.59501, 42.670628);
		shape3.lineTo(120.62634, 40.7087);
		shape3.moveTo(117.27089, 41.346153);
		shape3.lineTo(119.23241, 39.314873);
		shape3.moveTo(116.60836, 40.683914);
		shape3.lineTo(118.53506, 38.617523);
		shape3.moveTo(115.94501, 40.020214);
		shape3.lineTo(117.83677, 37.919186);
		shape3.moveTo(115.280426, 39.355694);
		shape3.lineTo(117.137184, 37.219894);
		shape3.moveTo(113.94608, 38.021355);
		shape3.lineTo(115.73261, 35.815273);
		shape3.moveTo(113.275444, 37.350662);
		shape3.lineTo(115.02663, 35.109463);
		shape3.moveTo(112.601875, 36.677155);
		shape3.lineTo(114.3176, 34.400314);
		shape3.moveTo(111.92508, 36.000355);
		shape3.lineTo(113.605156, 33.687874);
		shape3.moveTo(110.55956, 34.635075);
		shape3.lineTo(112.167946, 32.250423);
		shape3.moveTo(109.87009, 33.9457);
		shape3.lineTo(111.442116, 31.524881);
		shape3.moveTo(109.17555, 33.25111);
		shape3.lineTo(110.711, 30.79383);
		shape3.moveTo(108.47526, 32.55089);
		shape3.lineTo(109.973885, 30.056612);
		shape3.moveTo(80.000084, 79.94101);
		shape3.lineTo(80.0002, 79.94101);
		shape3.moveTo(107.0559, 31.13121);
		shape3.lineTo(108.479904, 28.56236);
		shape3.moveTo(80.00011, 79.94096);
		shape3.lineTo(80.00022, 79.94096);
		shape3.moveTo(106.335724, 30.411366);
		shape3.lineTo(107.721794, 27.804466);
		shape3.moveTo(80.00009, 79.94096);
		shape3.lineTo(80.000206, 79.94096);
		shape3.moveTo(105.6078, 29.683365);
		shape3.lineTo(106.95552, 27.038176);
		shape3.moveTo(80.00002, 79.94098);
		shape3.lineTo(80.00002, 79.94098);
		shape3.moveTo(104.87152, 28.947079);
		shape3.lineTo(106.18049, 26.262959);
		shape3.moveTo(79.999985, 79.94096);
		shape3.lineTo(79.999985, 79.94096);
		shape3.moveTo(79.999985, 79.94096);
		shape3.lineTo(79.999985, 79.94096);
		shape3.moveTo(103.37189, 27.447365);
		shape3.lineTo(104.60194, 24.684505);
		shape3.moveTo(79.99994, 79.94101);
		shape3.lineTo(79.99994, 79.94101);
		shape3.moveTo(102.60704, 26.68251);
		shape3.lineTo(103.796814, 23.87943);
		shape3.moveTo(79.999916, 79.94103);
		shape3.lineTo(79.999916, 79.94103);
		shape3.moveTo(101.831314, 25.906834);
		shape3.lineTo(102.98027, 23.062883);
		shape3.moveTo(79.99997, 79.94099);
		shape3.lineTo(79.99997, 79.94099);
		shape3.moveTo(101.044174, 25.119488);
		shape3.lineTo(102.15167, 22.234318);
		shape3.moveTo(79.99997, 79.94102);
		shape3.lineTo(79.99997, 79.94102);
		shape3.moveTo(79.99997, 79.94102);
		shape3.lineTo(79.99997, 79.94102);
		shape3.moveTo(99.43177, 23.507217);
		shape3.lineTo(100.45442, 20.537197);
		shape3.moveTo(79.99992, 79.940994);
		shape3.lineTo(79.99992, 79.940994);
		shape3.moveTo(98.605125, 22.680492);
		shape3.lineTo(99.58427, 19.666843);
		shape3.moveTo(79.99997, 79.94095);
		shape3.lineTo(79.99997, 79.94095);
		shape3.moveTo(97.763565, 21.83915);
		shape3.lineTo(98.698494, 18.780989);
		shape3.moveTo(79.99999, 79.94099);
		shape3.lineTo(79.99999, 79.94099);
		shape3.moveTo(96.906395, 20.981888);
		shape3.lineTo(97.79623, 17.878807);
		shape3.moveTo(80.00003, 79.94101);
		shape3.lineTo(80.00003, 79.94101);
		shape3.moveTo(80.00003, 79.94101);
		shape3.lineTo(80.00003, 79.94101);
		shape3.moveTo(95.14053, 19.215908);
		shape3.lineTo(95.937416, 16.019999);
		shape3.moveTo(80.000015, 79.941);
		shape3.lineTo(80.000015, 79.941);
		shape3.moveTo(94.23001, 18.305302);
		shape3.lineTo(94.97886, 15.061592);
		shape3.moveTo(80.00006, 79.940994);
		shape3.lineTo(80.00006, 79.940994);
		shape3.moveTo(93.29906, 17.374493);
		shape3.lineTo(93.99899, 14.081512);
		shape3.moveTo(79.99999, 79.94101);
		shape3.lineTo(79.99999, 79.94101);
		shape3.moveTo(92.34689, 16.422611);
		shape3.lineTo(92.99674, 13.079552);
		shape3.moveTo(80.000046, 79.94095);
		shape3.lineTo(80.000046, 79.94095);
		shape3.moveTo(80.000046, 79.94095);
		shape3.lineTo(80.000046, 79.94095);
		shape3.moveTo(90.373146, 14.448448);
		shape3.lineTo(90.91906, 11.001748);
		shape3.moveTo(80.00006, 79.94095);
		shape3.lineTo(80.00006, 79.94095);
		shape3.moveTo(89.348465, 13.42395);
		shape3.lineTo(89.84053, 9.922871);
		shape3.moveTo(80.00006, 79.94096);
		shape3.lineTo(80.00006, 79.94096);
		shape3.moveTo(88.296684, 12.372261);
		shape3.lineTo(88.73326, 8.815871);
		shape3.moveTo(80.00006, 79.94097);
		shape3.lineTo(80.00006, 79.94097);
		shape3.moveTo(87.21566, 11.29097);
		shape3.lineTo(87.5954, 7.67797);
		shape3.moveTo(80.00006, 79.94097);
		shape3.lineTo(80.00006, 79.94097);
		shape3.moveTo(80.00006, 79.94097);
		shape3.lineTo(80.00006, 79.94097);
		shape3.moveTo(84.95849, 9.0338745);
		shape3.lineTo(85.21944, 5.3022046);
		shape3.moveTo(80.00005, 79.94101);
		shape3.lineTo(80.00005, 79.94101);
		shape3.moveTo(83.77816, 7.8537064);
		shape3.lineTo(83.97698, 4.0597363);
		shape3.moveTo(80.000046, 79.94104);
		shape3.lineTo(80.000046, 79.94104);
		shape3.moveTo(82.56003, 6.635742);
		shape3.lineTo(82.694786, 2.7774723);
		shape3.moveTo(80.000046, 79.94107);
		shape3.lineTo(80.000046, 79.94107);
		shape3.moveTo(81.301735, 5.3772736);
		shape3.lineTo(81.37026, 1.4530036);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(80.00005, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99999, 79.941);
		shape3.moveTo(80.00005, 79.941);
		shape3.lineTo(79.99988, 79.941);
		shape3.moveTo(79.99999, 79.941475);
		shape3.lineTo(79.999756, 79.941);
		shape3.moveTo(79.99999, 79.941475);
		shape3.lineTo(79.999756, 79.941);
		shape3.moveTo(79.99999, 79.941475);
		shape3.lineTo(79.9997, 79.941);
		shape3.moveTo(80.00005, 79.941475);
		shape3.lineTo(79.99976, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(80.0, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(80.0, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(80.0, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(80.0, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(80.0, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(80.0, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(80.0, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(80.0, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99988, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99988, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99988, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99988, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99988, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99988, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99988, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99976, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99976, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99976, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.999886, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.999886, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.99976, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.99976, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99929, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.999176, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.999176, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.999054, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.999054, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.999, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99888, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.998764, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.998764, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.9987, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99859, 79.941);
		shape3.moveTo(80.00029, 79.941475);
		shape3.lineTo(79.99841, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.99841, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.998184, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.998, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.99777, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.9976, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.997185, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.99677, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.9963, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.9956, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.99471, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.99331, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.9909, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.986084, 79.941);
		shape3.moveTo(80.00041, 79.941475);
		shape3.lineTo(79.97182, 79.941);
		shape3.moveTo(131.12592, 70.92618);
		shape3.lineTo(147.88802, 67.97055);
		shape3.moveTo(128.78972, 62.18312);
		shape3.lineTo(138.54857, 58.63113);
		shape3.moveTo(124.95967, 53.98362);
		shape3.lineTo(130.62819, 50.71078);
		shape3.moveTo(119.76189, 46.57683);
		shape3.lineTo(123.42257, 43.505234);
		shape3.moveTo(113.36427, 40.179195);
		shape3.lineTo(116.435936, 36.518517);
		shape3.moveTo(105.957535, 34.981537);
		shape3.lineTo(109.230316, 29.312897);
		shape3.moveTo(97.75802, 31.151516);
		shape3.lineTo(101.310005, 21.392555);
		shape3.moveTo(89.01501, 28.815376);
		shape3.lineTo(91.970634, 12.053177);
		shape3.moveTo(80.00024, 26.783478);
		shape3.lineTo(80.00024, 21.683178);
		shape3.moveTo(80.00024, 15.811248);
		shape3.lineTo(80.00024, 0.083148);
		shape3.moveTo(78.69867, 5.377338);
		shape3.lineTo(78.6302, 1.4525979);
		shape3.moveTo(77.44038, 6.635808);
		shape3.lineTo(77.30562, 2.777538);
		shape3.moveTo(76.22225, 7.853818);
		shape3.lineTo(76.02343, 4.0593777);
		shape3.moveTo(75.04194, 9.034018);
		shape3.lineTo(74.78105, 5.3018775);
		shape3.moveTo(72.784775, 11.291128);
		shape3.lineTo(72.40504, 7.6781282);
		shape3.moveTo(71.70381, 12.372378);
		shape3.lineTo(71.26717, 8.815639);
		shape3.moveTo(70.651855, 13.424099);
		shape3.lineTo(70.15984, 9.923019);
		shape3.moveTo(69.62726, 14.448629);
		shape3.lineTo(69.081345, 11.001519);
		shape3.moveTo(67.653465, 16.422718);
		shape3.lineTo(67.00362, 13.079128);
		shape3.moveTo(66.70127, 17.374619);
		shape3.lineTo(66.001335, 14.081638);
		shape3.moveTo(65.77051, 18.305428);
		shape3.lineTo(65.021545, 15.061188);
		shape3.moveTo(64.85983, 19.216047);
		shape3.lineTo(64.06295, 16.020138);
		shape3.moveTo(63.093845, 20.981977);
		shape3.lineTo(62.20407, 17.878897);
		shape3.moveTo(62.23666, 21.839277);
		shape3.lineTo(61.30173, 18.781116);
		shape3.moveTo(61.395153, 22.680607);
		shape3.lineTo(60.415894, 19.666958);
		shape3.moveTo(60.568565, 23.507318);
		shape3.lineTo(59.545795, 20.537298);
		shape3.moveTo(58.956196, 25.120037);
		shape3.lineTo(57.848576, 22.234398);
		shape3.moveTo(58.169006, 25.906939);
		shape3.lineTo(57.019928, 23.062988);
		shape3.moveTo(57.393265, 26.682558);
		shape3.lineTo(56.203373, 23.879478);
		shape3.moveTo(56.62844, 27.447498);
		shape3.lineTo(55.39833, 24.684639);
		shape3.moveTo(55.12881, 28.947248);
		shape3.lineTo(53.81972, 26.263128);
		shape3.moveTo(54.392586, 29.683529);
		shape3.lineTo(53.044807, 27.03834);
		shape3.moveTo(53.664707, 30.41147);
		shape3.lineTo(52.278576, 27.80422);
		shape3.moveTo(52.94457, 31.13131);
		shape3.lineTo(51.520508, 28.56246);
		shape3.moveTo(51.525204, 32.55097);
		shape3.lineTo(50.026455, 30.056688);
		shape3.moveTo(50.82492, 33.25119);
		shape3.lineTo(49.289352, 30.79338);
		shape3.moveTo(50.130272, 33.94578);
		shape3.lineTo(48.55818, 31.524612);
		shape3.moveTo(49.44091, 34.63515);
		shape3.lineTo(47.83247, 32.2505);
		shape3.moveTo(48.07551, 36.00043);
		shape3.lineTo(46.3952, 33.687542);
		shape3.moveTo(47.39871, 36.67723);
		shape3.lineTo(45.68276, 34.40039);
		shape3.moveTo(46.72503, 37.351273);
		shape3.lineTo(44.973667, 35.109543);
		shape3.moveTo(46.054337, 38.021843);
		shape3.lineTo(44.267685, 35.815353);
		shape3.moveTo(44.719997, 39.356182);
		shape3.lineTo(42.863056, 37.21997);
		shape3.moveTo(44.055588, 40.02029);
		shape3.lineTo(42.163708, 37.919262);
		shape3.moveTo(43.392117, 40.684002);
		shape3.lineTo(41.465355, 38.61761);
		shape3.moveTo(42.729694, 41.34624);
		shape3.lineTo(40.768005, 39.31496);
		shape3.moveTo(41.40546, 42.67071);
		shape3.lineTo(39.37407, 40.708782);
		shape3.moveTo(40.742928, 43.332943);
		shape3.lineTo(38.676716, 41.406242);
		shape3.moveTo(40.079636, 43.996643);
		shape3.lineTo(37.978424, 42.10459);
		shape3.moveTo(39.414993, 44.660763);
		shape3.lineTo(37.278904, 42.803883);
		shape3.moveTo(38.080658, 45.99557);
		shape3.lineTo(35.87428, 44.20845);
		shape3.moveTo(37.41002, 46.6662);
		shape3.lineTo(35.16835, 44.914837);
		shape3.moveTo(36.73645, 47.339706);
		shape3.lineTo(34.45926, 45.623867);
		shape3.moveTo(36.05965, 48.0165);
		shape3.lineTo(33.746872, 46.336308);
		shape3.moveTo(34.69419, 49.38178);
		shape3.lineTo(32.309483, 47.77322);
		shape3.moveTo(34.00471, 50.07114);
		shape3.lineTo(31.58377, 48.49923);
		shape3.moveTo(33.31, 50.76579);
		shape3.lineTo(30.852602, 49.23034);
		shape3.moveTo(32.609894, 51.465958);
		shape3.lineTo(30.115503, 49.967617);
		shape3.moveTo(31.190594, 52.885666);
		shape3.lineTo(28.621563, 51.461254);
		shape3.moveTo(30.470343, 53.605446);
		shape3.lineTo(27.863382, 52.219616);
		shape3.moveTo(29.742462, 54.333397);
		shape3.lineTo(27.097273, 52.985497);
		shape3.moveTo(29.006172, 55.069675);
		shape3.lineTo(26.322113, 53.760586);
		shape3.moveTo(27.506424, 56.569427);
		shape3.lineTo(24.743504, 55.339607);
		shape3.moveTo(26.741663, 57.33436);
		shape3.lineTo(23.938463, 56.14436);
		shape3.moveTo(25.965982, 58.10999);
		shape3.lineTo(23.121912, 56.9608);
		shape3.moveTo(25.178732, 58.89736);
		shape3.lineTo(22.293272, 57.78945);
		shape3.moveTo(23.566422, 60.50962);
		shape3.lineTo(20.596111, 59.48697);
		shape3.moveTo(22.73977, 61.33633);
		shape3.lineTo(19.72595, 60.35689);
		shape3.moveTo(21.89821, 62.17766);
		shape3.lineTo(18.84011, 61.24262);
		shape3.moveTo(21.04091, 63.03497);
		shape3.lineTo(17.93771, 62.145313);
		shape3.moveTo(19.27516, 64.800896);
		shape3.lineTo(16.07889, 64.004135);
		shape3.moveTo(18.36448, 65.71152);
		shape3.lineTo(15.120299, 64.962616);
		shape3.moveTo(17.43372, 66.642334);
		shape3.lineTo(14.14057, 65.94257);
		shape3.moveTo(16.48153, 67.59476);
		shape3.lineTo(13.138229, 66.94462);
		shape3.moveTo(14.507679, 69.56837);
		shape3.lineTo(11.060619, 69.02234);
		shape3.moveTo(13.483139, 70.59285);
		shape3.lineTo(9.9820595, 70.10072);
		shape3.moveTo(12.431059, 71.64504);
		shape3.lineTo(8.874669, 71.20823);
		shape3.moveTo(11.3501, 72.72583);
		shape3.lineTo(7.7368097, 72.34621);
		shape3.moveTo(9.09299, 74.983055);
		shape3.lineTo(5.36091, 74.72187);
		shape3.moveTo(7.91261, 76.16366);
		shape3.lineTo(4.11835, 75.96436);
		shape3.moveTo(6.69454, 77.381676);
		shape3.lineTo(2.83621, 77.24674);
		shape3.moveTo(5.4363003, 78.639626);
		shape3.lineTo(1.5116804, 78.57116);
		shape3.moveTo(70.98528, 28.81556);
		shape3.lineTo(68.0297, 12.05336);
		shape3.moveTo(62.242092, 31.15176);
		shape3.lineTo(58.690224, 21.39291);
		shape3.moveTo(54.042652, 34.981712);
		shape3.lineTo(50.76993, 29.313192);
		shape3.moveTo(46.635803, 40.179493);
		shape3.lineTo(43.564255, 36.518932);
		shape3.moveTo(40.238163, 46.577133);
		shape3.lineTo(36.57754, 43.505585);
		shape3.moveTo(35.04039, 53.983887);
		shape3.lineTo(29.37193, 50.711166);
		shape3.moveTo(31.21037, 62.183365);
		shape3.lineTo(21.45164, 58.631496);
		shape3.moveTo(28.874218, 70.9265);
		shape3.lineTo(12.112219, 67.97087);
		g.draw(shape3);
	}

	private void paintShapeNode_0_0_0_0_4(Graphics2D g) {
		GeneralPath shape4 = new GeneralPath();
		shape4.moveTo(11.7066, 73.9662);
		shape4.lineTo(6.56614, 73.51653);
		shape4.moveTo(21.42634, 64.246346);
		shape4.lineTo(17.01758, 63.065086);
		shape4.moveTo(29.35008, 56.322647);
		shape4.lineTo(25.53779, 54.544918);
		shape4.moveTo(36.318092, 49.354717);
		shape4.lineTo(33.030224, 47.052505);
		shape4.moveTo(42.865814, 42.806927);
		shape4.lineTo(40.070774, 40.011887);
		shape4.moveTo(49.413593, 36.259136);
		shape4.lineTo(47.11138, 32.971268);
		shape4.moveTo(56.38151, 29.291218);
		shape4.lineTo(54.603783, 25.478868);
		shape4.moveTo(64.30526, 21.36752);
		shape4.lineTo(63.12395, 16.958698);
		shape4.moveTo(74.025055, 11.6477785);
		shape4.lineTo(73.575325, 6.5073786);
		shape4.moveTo(85.97502, 11.6477785);
		shape4.lineTo(86.42475, 6.5073786);
		shape4.moveTo(95.69476, 21.367477);
		shape4.lineTo(96.876076, 16.958656);
		shape4.moveTo(103.61839, 29.291058);
		shape4.lineTo(105.39617, 25.478767);
		shape4.moveTo(110.58642, 36.259068);
		shape4.lineTo(112.888626, 32.9712);
		shape4.moveTo(117.13421, 42.80673);
		shape4.lineTo(119.929245, 40.01169);
		shape4.moveTo(123.68199, 49.35463);
		shape4.lineTo(126.969864, 47.052418);
		shape4.moveTo(130.64992, 56.322548);
		shape4.lineTo(134.46227, 54.54482);
		shape4.moveTo(138.57355, 64.24618);
		shape4.lineTo(142.9823, 63.06487);
		shape4.moveTo(148.29327, 73.96597);
		shape4.lineTo(153.43361, 73.51624);
		g.draw(shape4);
	}

	private void paintShapeNode_0_0_0_0_5_0(Graphics2D g) {
		GeneralPath shape5 = new GeneralPath();
		shape5.moveTo(80.6291, 75.5662);
		shape5.curveTo(80.6291, 75.80478, 80.60467, 75.99685, 80.55534, 76.141884);
		shape5.curveTo(80.506546, 76.28739, 80.433266, 76.399605, 80.336555, 76.47858);
		shape5.curveTo(80.239784, 76.557434, 80.11824, 76.59684, 79.971794, 76.59684);
		shape5.curveTo(79.86328, 76.59684, 79.76845, 76.57482, 79.68677, 76.531075);
		shape5.curveTo(79.605034, 76.487976, 79.53786, 76.424965, 79.48436, 76.34282);
		shape5.curveTo(79.431335, 76.26067, 79.38953, 76.160614, 79.35947, 76.04282);
		shape5.curveTo(79.32893, 75.92497, 79.31396, 75.76626, 79.31396, 75.5662);
		shape5.curveTo(79.31396, 75.32962, 79.338326, 75.13896, 79.38671, 74.99339);
		shape5.curveTo(79.43551, 74.847885, 79.508316, 74.73567, 79.60503, 74.65629);
		shape5.curveTo(79.70174, 74.57696, 79.823814, 74.5375, 79.97179, 74.5375);
		shape5.curveTo(80.166145, 74.5375, 80.318756, 74.606964, 80.42956, 74.74642);
		shape5.curveTo(80.56244, 74.91406, 80.62909, 75.18776, 80.62909, 75.5662);
		shape5.closePath();
		shape5.moveTo(80.37455, 75.5662);
		shape5.curveTo(80.37455, 75.2352, 80.33562, 75.014946, 80.25817, 74.905136);
		shape5.curveTo(80.18072, 74.795746, 80.08536, 74.74072, 79.971794, 74.74072);
		shape5.curveTo(79.857704, 74.74072, 79.762344, 74.79615, 79.68489, 74.906075);
		shape5.curveTo(79.60744, 75.01594, 79.56839, 75.23619, 79.56839, 75.5662);
		shape5.curveTo(79.56839, 75.8982, 79.60744, 76.11893, 79.68489, 76.22779);
		shape5.curveTo(79.762344, 76.33724, 79.85858, 76.39162, 79.97408, 76.39162);
		shape5.curveTo(80.08817, 76.39162, 80.17884, 76.34329, 80.246895, 76.24705);
		shape5.curveTo(80.33186, 76.124504, 80.37455, 75.8972, 80.37455, 75.5662);
		shape5.closePath();
		g.fill(shape5);
	}

	private void paintShapeNode_0_0_0_0_5_1(Graphics2D g) {
		GeneralPath shape6 = new GeneralPath();
		shape6.moveTo(89.5825, 74.5546);
		shape6.lineTo(89.82994, 74.5546);
		shape6.lineTo(89.82994, 76.13222);
		shape6.curveTo(89.889656, 76.07538, 89.96752, 76.01859, 90.06423, 75.961815);
		shape6.curveTo(90.16147, 75.90504, 90.248314, 75.86229, 90.325356, 75.834045);
		shape6.lineTo(90.325356, 76.07309);
		shape6.curveTo(90.186775, 76.13827, 90.06564, 76.21725, 89.96188, 76.30967);
		shape6.curveTo(89.8586, 76.40274, 89.784904, 76.49235, 89.741684, 76.57972);
		shape6.lineTo(89.5825, 76.57972);
		shape6.lineTo(89.5825, 74.55461);
		shape6.closePath();
		g.fill(shape6);
	}

	private void paintShapeNode_0_0_0_0_5_2(Graphics2D g) {
		GeneralPath shape7 = new GeneralPath();
		shape7.moveTo(69.6179, 74.5546);
		shape7.lineTo(69.86534, 74.5546);
		shape7.lineTo(69.86534, 76.13222);
		shape7.curveTo(69.92506, 76.07538, 70.00292, 76.01859, 70.09963, 75.961815);
		shape7.curveTo(70.19687, 75.90504, 70.283714, 75.86229, 70.360756, 75.834045);
		shape7.lineTo(70.360756, 76.07309);
		shape7.curveTo(70.222176, 76.13827, 70.10104, 76.21725, 69.997284, 76.30967);
		shape7.curveTo(69.894, 76.40274, 69.820305, 76.49235, 69.777084, 76.57972);
		shape7.lineTo(69.6179, 76.57972);
		shape7.lineTo(69.6179, 74.55461);
		shape7.closePath();
		g.fill(shape7);
	}

	private void paintShapeNode_0_0_0_0_5_3(Graphics2D g) {
		GeneralPath shape8 = new GeneralPath();
		shape8.moveTo(99.3524, 74.8575);
		shape8.lineTo(99.3524, 74.61945);
		shape8.lineTo(100.68544, 74.61945);
		shape8.curveTo(100.68732, 74.679054, 100.677925, 74.73637, 100.656784, 74.79132);
		shape8.curveTo(100.6229, 74.88239, 100.56853, 74.97165, 100.493835, 75.059845);
		shape8.curveTo(100.41868, 75.14769, 100.31069, 75.24951, 100.16982, 75.365005);
		shape8.curveTo(99.95062, 75.54492, 99.802185, 75.68714, 99.725204, 75.792366);
		shape8.curveTo(99.648285, 75.897064, 99.6097, 75.99659, 99.6097, 76.09001);
		shape8.curveTo(99.6097, 76.18813, 99.644936, 76.270805, 99.71481, 76.338394);
		shape8.curveTo(99.78533, 76.40557, 99.87688, 76.43933, 99.98956, 76.43933);
		shape8.curveTo(100.108826, 76.43933, 100.204185, 76.40369, 100.27553, 76.33182);
		shape8.curveTo(100.34734, 76.260414, 100.38345, 76.161415, 100.38445, 76.03511);
		shape8.lineTo(100.638885, 76.06089);
		shape8.curveTo(100.62157, 76.25102, 100.555855, 76.395706, 100.44218, 76.49524);
		shape8.curveTo(100.32855, 76.594765, 100.17553, 76.64456, 99.98399, 76.64456);
		shape8.curveTo(99.79051, 76.64456, 99.637375, 76.59054, 99.52469, 76.48355);
		shape8.curveTo(99.4116, 76.375916, 99.35516, 76.243034, 99.35516, 76.08432);
		shape8.curveTo(99.35516, 76.00358, 99.371666, 75.92425, 99.405014, 75.84639);
		shape8.curveTo(99.437836, 75.76841, 99.49286, 75.686264, 99.56931, 75.600296);
		shape8.curveTo(99.64588, 75.51386, 99.773125, 75.39554, 99.95105, 75.24534);
		shape8.curveTo(100.099495, 75.120445, 100.19479, 75.03589, 100.237015, 74.99126);
		shape8.curveTo(100.279236, 74.94716, 100.313995, 74.902534, 100.34171, 74.8575);
		shape8.lineTo(99.35241, 74.8575);
		shape8.closePath();
		g.fill(shape8);
	}

	private void paintShapeNode_0_0_0_0_5_4(Graphics2D g) {
		GeneralPath shape9 = new GeneralPath();
		shape9.moveTo(59.4233, 74.8575);
		shape9.lineTo(59.4233, 74.61945);
		shape9.lineTo(60.75634, 74.61945);
		shape9.curveTo(60.75822, 74.679054, 60.748825, 74.73637, 60.727684, 74.79132);
		shape9.curveTo(60.6938, 74.88239, 60.639427, 74.97165, 60.56474, 75.059845);
		shape9.curveTo(60.48958, 75.14769, 60.381596, 75.24951, 60.240726, 75.365005);
		shape9.curveTo(60.021526, 75.54492, 59.873085, 75.68714, 59.796104, 75.792366);
		shape9.curveTo(59.71918, 75.897064, 59.680603, 75.99659, 59.680603, 76.09001);
		shape9.curveTo(59.680603, 76.18813, 59.715836, 76.270805, 59.78571, 76.338394);
		shape9.curveTo(59.85623, 76.40557, 59.947773, 76.43933, 60.060455, 76.43933);
		shape9.curveTo(60.179714, 76.43933, 60.275074, 76.40369, 60.346416, 76.33182);
		shape9.curveTo(60.418232, 76.260414, 60.45434, 76.161415, 60.45534, 76.03511);
		shape9.lineTo(60.709774, 76.06089);
		shape9.curveTo(60.69245, 76.25102, 60.626743, 76.395706, 60.513065, 76.49524);
		shape9.curveTo(60.399445, 76.594765, 60.24642, 76.64456, 60.05488, 76.64456);
		shape9.curveTo(59.861397, 76.64456, 59.70826, 76.59054, 59.595577, 76.48355);
		shape9.curveTo(59.482483, 76.375916, 59.426056, 76.243034, 59.426056, 76.08432);
		shape9.curveTo(59.426056, 76.00358, 59.442554, 75.92425, 59.47591, 75.84639);
		shape9.curveTo(59.508736, 75.76841, 59.563755, 75.686264, 59.640205, 75.600296);
		shape9.curveTo(59.716774, 75.51386, 59.84402, 75.39554, 60.02194, 75.24534);
		shape9.curveTo(60.17038, 75.120445, 60.265682, 75.03589, 60.3079, 74.99126);
		shape9.curveTo(60.350117, 74.94716, 60.38488, 74.902534, 60.412594, 74.8575);
		shape9.lineTo(59.42329, 74.8575);
		shape9.closePath();
		g.fill(shape9);
	}

	private void paintShapeNode_0_0_0_0_5_5(Graphics2D g) {
		GeneralPath shape10 = new GeneralPath();
		shape10.moveTo(110.735, 75.1439);
		shape10.lineTo(110.48756, 75.17672);
		shape10.curveTo(110.4589, 75.03638, 110.410515, 74.93544, 110.3424, 74.8735);
		shape10.curveTo(110.27393, 74.81149, 110.19079, 74.780426, 110.09267, 74.780426);
		shape10.curveTo(109.97629, 74.780426, 109.87764, 74.820885, 109.79725, 74.90163);
		shape10.curveTo(109.71751, 74.98237, 109.67711, 75.08242, 109.67711, 75.20168);
		shape10.curveTo(109.67711, 75.31531, 109.71422, 75.409256, 109.78838, 75.48289);
		shape10.curveTo(109.863075, 75.55658, 109.957375, 75.59369, 110.072, 75.59369);
		shape10.curveTo(110.11891, 75.59369, 110.17722, 75.5843, 110.24669, 75.56603);
		shape10.lineTo(110.21892, 75.783356);
		shape10.curveTo(110.20254, 75.78159, 110.189384, 75.780594, 110.17911, 75.780594);
		shape10.curveTo(110.07388, 75.780594, 109.97905, 75.80831, 109.89449, 75.86321);
		shape10.curveTo(109.80994, 75.91811, 109.767715, 76.0032, 109.767715, 76.1177);
		shape10.curveTo(109.767715, 76.208305, 109.798775, 76.283875, 109.86026, 76.34306);
		shape10.curveTo(109.92174, 76.402664, 110.00113, 76.432785, 110.098366, 76.432785);
		shape10.curveTo(110.19455, 76.432785, 110.27482, 76.40225, 110.33871, 76.34218);
		shape10.curveTo(110.40301, 76.28158, 110.44434, 76.190445, 110.46272, 76.06937);
		shape10.lineTo(110.71005, 76.113464);
		shape10.curveTo(110.680046, 76.2797, 110.61105, 76.408356, 110.503944, 76.49937);
		shape10.curveTo(110.39643, 76.5905, 110.26308, 76.636536, 110.103424, 76.636536);
		shape10.curveTo(109.99362, 76.636536, 109.89221, 76.61258, 109.79967, 76.56566);
		shape10.curveTo(109.70712, 76.51827, 109.63625, 76.45386, 109.58698, 76.37224);
		shape10.curveTo(109.53813, 76.29044, 109.51329, 76.204124, 109.51329, 76.11205);
		shape10.curveTo(109.51329, 76.02521, 109.53672, 75.94588, 109.58363, 75.87401);
		shape10.curveTo(109.63067, 75.802605, 109.69966, 75.74583, 109.79121, 75.703606);
		shape10.curveTo(109.672005, 75.67636, 109.57947, 75.61905, 109.51329, 75.5322);
		shape10.curveTo(109.447525, 75.44577, 109.41464, 75.33737, 109.41464, 75.207306);
		shape10.curveTo(109.41464, 75.031204, 109.47853, 74.88188, 109.607124, 74.75939);
		shape10.curveTo(109.73537, 74.63679, 109.89778, 74.575775, 110.094086, 74.575775);
		shape10.curveTo(110.271065, 74.575775, 110.4181, 74.62833, 110.53495, 74.734024);
		shape10.curveTo(110.65186, 74.839134, 110.71857, 74.97577, 110.73501, 75.14388);
		shape10.closePath();
		g.fill(shape10);
	}

	private void paintShapeNode_0_0_0_0_5_6(Graphics2D g) {
		GeneralPath shape11 = new GeneralPath();
		shape11.moveTo(50.6878, 75.1174);
		shape11.lineTo(50.4403, 75.15022);
		shape11.curveTo(50.411705, 75.00988, 50.363262, 74.90894, 50.295204, 74.84694);
		shape11.curveTo(50.226677, 74.78499, 50.14353, 74.75393, 50.045414, 74.75393);
		shape11.curveTo(49.92903, 74.75393, 49.830383, 74.79433, 49.750057, 74.87513);
		shape11.curveTo(49.670315, 74.95587, 49.62986, 75.05587, 49.62986, 75.175186);
		shape11.curveTo(49.62986, 75.28875, 49.66703, 75.3827, 49.74119, 75.45639);
		shape11.curveTo(49.815823, 75.53002, 49.910187, 75.56719, 50.024746, 75.56719);
		shape11.curveTo(50.071663, 75.56719, 50.12997, 75.5578, 50.199436, 75.539536);
		shape11.lineTo(50.171722, 75.75686);
		shape11.curveTo(50.15534, 75.755035, 50.142185, 75.75404, 50.13185, 75.75404);
		shape11.curveTo(50.026684, 75.75404, 49.931854, 75.781815, 49.847298, 75.83672);
		shape11.curveTo(49.76274, 75.891624, 49.720524, 75.976654, 49.720524, 76.091156);
		shape11.curveTo(49.720524, 76.181816, 49.751526, 76.25739, 49.813065, 76.31658);
		shape11.curveTo(49.874542, 76.37618, 49.953873, 76.4063, 50.051113, 76.4063);
		shape11.curveTo(50.147354, 76.4063, 50.227623, 76.37577, 50.29151, 76.31564);
		shape11.curveTo(50.35581, 76.25504, 50.397087, 76.16391, 50.415466, 76.04283);
		shape11.lineTo(50.66285, 76.08699);
		shape11.curveTo(50.632786, 76.25317, 50.56379, 76.38182, 50.456688, 76.47289);
		shape11.curveTo(50.349174, 76.564026, 50.21582, 76.61006, 50.05622, 76.61006);
		shape11.curveTo(49.946415, 76.61006, 49.84501, 76.586105, 49.752464, 76.539185);
		shape11.curveTo(49.659924, 76.49174, 49.589046, 76.42738, 49.539722, 76.3457);
		shape11.curveTo(49.49093, 76.26397, 49.46603, 76.17759, 49.46603, 76.08558);
		shape11.curveTo(49.46603, 75.99873, 49.489517, 75.91934, 49.536434, 75.84747);
		shape11.curveTo(49.58347, 75.77613, 49.652405, 75.719345, 49.74395, 75.67707);
		shape11.curveTo(49.62475, 75.649826, 49.53227, 75.592575, 49.46603, 75.50573);
		shape11.curveTo(49.400265, 75.419235, 49.36744, 75.310844, 49.36744, 75.18078);
		shape11.curveTo(49.36744, 75.00468, 49.431324, 74.855354, 49.559864, 74.732864);
		shape11.curveTo(49.688168, 74.61032, 49.850582, 74.54931, 50.04682, 74.54931);
		shape11.curveTo(50.2238, 74.54931, 50.370834, 74.60181, 50.487743, 74.7075);
		shape11.curveTo(50.604652, 74.81261, 50.6713, 74.9493, 50.6878, 75.11742);
		shape11.closePath();
		g.fill(shape11);
	}

	private void paintShapeNode_0_0_0_0_5_7(Graphics2D g) {
		GeneralPath shape12 = new GeneralPath();
		shape12.moveTo(119.724, 74.5589);
		shape12.lineTo(119.724, 75.04157);
		shape12.lineTo(120.59874, 75.04157);
		shape12.lineTo(120.59874, 75.268814);
		shape12.lineTo(119.678375, 76.57549);
		shape12.lineTo(119.47603, 76.57549);
		shape12.lineTo(119.47603, 75.268814);
		shape12.lineTo(119.20369, 75.268814);
		shape12.lineTo(119.20369, 75.04157);
		shape12.lineTo(119.47603, 75.04157);
		shape12.lineTo(119.47603, 74.5589);
		shape12.lineTo(119.724, 74.5589);
		shape12.closePath();
		shape12.moveTo(119.724, 75.268814);
		shape12.lineTo(119.724, 76.17785);
		shape12.lineTo(120.354996, 75.268814);
		shape12.lineTo(119.724, 75.268814);
		shape12.closePath();
		g.fill(shape12);
	}

	private void paintShapeNode_0_0_0_0_5_8(Graphics2D g) {
		GeneralPath shape13 = new GeneralPath();
		shape13.moveTo(39.8652, 74.5589);
		shape13.lineTo(39.8652, 75.04157);
		shape13.lineTo(40.73994, 75.04157);
		shape13.lineTo(40.73994, 75.268814);
		shape13.lineTo(39.819576, 76.57549);
		shape13.lineTo(39.61723, 76.57549);
		shape13.lineTo(39.61723, 75.268814);
		shape13.lineTo(39.344948, 75.268814);
		shape13.lineTo(39.344948, 75.04157);
		shape13.lineTo(39.61723, 75.04157);
		shape13.lineTo(39.61723, 74.5589);
		shape13.lineTo(39.8652, 74.5589);
		shape13.closePath();
		shape13.moveTo(39.8652, 75.268814);
		shape13.lineTo(39.8652, 76.17785);
		shape13.lineTo(40.496197, 75.268814);
		shape13.lineTo(39.8652, 75.268814);
		shape13.closePath();
		g.fill(shape13);
	}

	private void paintShapeNode_0_0_0_0_5_9(Graphics2D g) {
		GeneralPath shape14 = new GeneralPath();
		shape14.moveTo(130.552, 75.1181);
		shape14.lineTo(130.29187, 75.14018);
		shape14.curveTo(130.27261, 75.013405, 130.22798, 74.91858, 130.15799, 74.85469);
		shape14.curveTo(130.08765, 74.7908, 130.00308, 74.75886, 129.90402, 74.75886);
		shape14.curveTo(129.78476, 74.75886, 129.68382, 74.804016, 129.60168, 74.893616);
		shape14.curveTo(129.51901, 74.98375, 129.47772, 75.103065, 129.47772, 75.25139);
		shape14.curveTo(129.47772, 75.39272, 129.51712, 75.50406, 129.59651, 75.58568);
		shape14.curveTo(129.67584, 75.66747, 129.77954, 75.70828, 129.90825, 75.70828);
		shape14.curveTo(129.98811, 75.70828, 130.05986, 75.69002, 130.12428, 75.65385);
		shape14.curveTo(130.18852, 75.617615, 130.23872, 75.5707, 130.27538, 75.512924);
		shape14.lineTo(130.5079, 75.54299);
		shape14.lineTo(130.31255, 76.578865);
		shape14.lineTo(129.30963, 76.578865);
		shape14.lineTo(129.30963, 76.34217);
		shape14.lineTo(130.11438, 76.34217);
		shape14.lineTo(130.2233, 75.80031);
		shape14.curveTo(130.10222, 75.884865, 129.97491, 75.927086, 129.84209, 75.927086);
		shape14.curveTo(129.66599, 75.927086, 129.51767, 75.866135, 129.39647, 75.743996);
		shape14.curveTo(129.27539, 75.62192, 129.2148, 75.46514, 129.2148, 75.27354);
		shape14.curveTo(129.2148, 75.09086, 129.2683, 74.93308, 129.37439, 74.8002);
		shape14.curveTo(129.50398, 74.637314, 129.68056, 74.55563, 129.90404, 74.55563);
		shape14.curveTo(130.08765, 74.55563, 130.23738, 74.60672, 130.35336, 74.70953);
		shape14.curveTo(130.46933, 74.81247, 130.53557, 74.94864, 130.55202, 75.1181);
		shape14.closePath();
		g.fill(shape14);
	}

	private void paintShapeNode_0_0_0_0_5_10(Graphics2D g) {
		GeneralPath shape15 = new GeneralPath();
		shape15.moveTo(30.7288, 75.1181);
		shape15.lineTo(30.468674, 75.14018);
		shape15.curveTo(30.449413, 75.013405, 30.404787, 74.91858, 30.334793, 74.85469);
		shape15.curveTo(30.264389, 74.7908, 30.179892, 74.75886, 30.080832, 74.75886);
		shape15.curveTo(29.961573, 74.75886, 29.860634, 74.804016, 29.778486, 74.893616);
		shape15.curveTo(29.69581, 74.98375, 29.65447, 75.103065, 29.65447, 75.25139);
		shape15.curveTo(29.65447, 75.39272, 29.69393, 75.50406, 29.77326, 75.58568);
		shape15.curveTo(29.852648, 75.66747, 29.956347, 75.70828, 30.085058, 75.70828);
		shape15.curveTo(30.164917, 75.70828, 30.236671, 75.69002, 30.301027, 75.65385);
		shape15.curveTo(30.365324, 75.617615, 30.415531, 75.5707, 30.45217, 75.512924);
		shape15.lineTo(30.684639, 75.54299);
		shape15.lineTo(30.48934, 76.578865);
		shape15.lineTo(29.48642, 76.578865);
		shape15.lineTo(29.48642, 76.34217);
		shape15.lineTo(30.291166, 76.34217);
		shape15.lineTo(30.400091, 75.80031);
		shape15.curveTo(30.279013, 75.884865, 30.151709, 75.927086, 30.018827, 75.927086);
		shape15.curveTo(29.842787, 75.927086, 29.694403, 75.866135, 29.573265, 75.743996);
		shape15.curveTo(29.452187, 75.62192, 29.391588, 75.46514, 29.391588, 75.27354);
		shape15.curveTo(29.391588, 75.09086, 29.445082, 74.93308, 29.551188, 74.8002);
		shape15.curveTo(29.68078, 74.637314, 29.85735, 74.55563, 30.080835, 74.55563);
		shape15.curveTo(30.26439, 74.55563, 30.414125, 74.60672, 30.530155, 74.70953);
		shape15.curveTo(30.646126, 74.81247, 30.712301, 74.94864, 30.728802, 75.1181);
		shape15.closePath();
		g.fill(shape15);
	}

	private void paintShapeNode_0_0_0_0_5_11(Graphics2D g) {
		GeneralPath shape16 = new GeneralPath();
		shape16.moveTo(139.236, 76.0945);
		shape16.lineTo(139.4819, 76.07524);
		shape16.curveTo(139.50398, 76.17248, 139.53505, 76.243355, 139.57538, 76.28698);
		shape16.curveTo(139.64255, 76.35791, 139.72517, 76.3932, 139.82329, 76.3932);
		shape16.curveTo(139.90215, 76.3932, 139.97115, 76.37101, 140.03087, 76.32691);
		shape16.curveTo(140.1088, 76.27007, 140.17033, 76.18704, 140.2153, 76.07806);
		shape16.curveTo(140.25993, 75.96913, 140.28342, 75.813705, 140.2853, 75.611885);
		shape16.curveTo(140.2257, 75.70243, 140.15294, 75.77007, 140.06651, 75.81423);
		shape16.curveTo(139.98067, 75.8578, 139.89, 75.87988, 139.79558, 75.87988);
		shape16.curveTo(139.63034, 75.87988, 139.48988, 75.819405, 139.37392, 75.69774);
		shape16.curveTo(139.25801, 75.57613, 139.19969, 75.41929, 139.19969, 75.22687);
		shape16.curveTo(139.19969, 75.10009, 139.22746, 74.98271, 139.28195, 74.87367);
		shape16.curveTo(139.33638, 74.765274, 139.41154, 74.68172, 139.50685, 74.62394);
		shape16.curveTo(139.6021, 74.566216, 139.71014, 74.53757, 139.83127, 74.53757);
		shape16.curveTo(140.03784, 74.53757, 140.2059, 74.61314, 140.33601, 74.76528);
		shape16.curveTo(140.46655, 74.91689, 140.53131, 75.16716, 140.53131, 75.51554);
		shape16.curveTo(140.53131, 75.90526, 140.45944, 76.18846, 140.31534, 76.36544);
		shape16.curveTo(140.18991, 76.51946, 140.02092, 76.59691, 139.80783, 76.59691);
		shape16.curveTo(139.64952, 76.59691, 139.51952, 76.552284, 139.4181, 76.46309);
		shape16.curveTo(139.3167, 76.374306, 139.2561, 76.25135, 139.23596, 76.094505);
		shape16.closePath();
		shape16.moveTo(140.24684, 75.22546);
		shape16.curveTo(140.24684, 75.1399, 140.22852, 75.05823, 140.19235, 74.9803);
		shape16.curveTo(140.15623, 74.90232, 140.1055, 74.84325, 140.04074, 74.80238);
		shape16.curveTo(139.97545, 74.76146, 139.90686, 74.74078, 139.83551, 74.74078);
		shape16.curveTo(139.73076, 74.74078, 139.64114, 74.78312, 139.566, 74.86756);
		shape16.curveTo(139.49089, 74.95212, 139.45325, 75.066734, 139.45325, 75.2113);
		shape16.curveTo(139.45325, 75.3507, 139.49042, 75.460625, 139.56458, 75.54089);
		shape16.curveTo(139.63873, 75.62122, 139.73216, 75.66115, 139.8449, 75.66115);
		shape16.curveTo(139.95705, 75.66115, 140.052, 75.621216, 140.12985, 75.54089);
		shape16.curveTo(140.20782, 75.460625, 140.24683, 75.3554, 140.24683, 75.22546);
		shape16.closePath();
		g.fill(shape16);
	}

	private void paintShapeNode_0_0_0_0_5_12(Graphics2D g) {
		GeneralPath shape17 = new GeneralPath();
		shape17.moveTo(19.4483, 76.0945);
		shape17.lineTo(19.694275, 76.07524);
		shape17.curveTo(19.716352, 76.17248, 19.747416, 76.243355, 19.787697, 76.28698);
		shape17.curveTo(19.854872, 76.35791, 19.93755, 76.3932, 20.035667, 76.3932);
		shape17.curveTo(20.114527, 76.3932, 20.183523, 76.37101, 20.243181, 76.32691);
		shape17.curveTo(20.321161, 76.27007, 20.382639, 76.18704, 20.427677, 76.07806);
		shape17.curveTo(20.472303, 75.96913, 20.495733, 75.813705, 20.497671, 75.611885);
		shape17.curveTo(20.43807, 75.70243, 20.365318, 75.77007, 20.278824, 75.81423);
		shape17.curveTo(20.192976, 75.8578, 20.102373, 75.87988, 20.007952, 75.87988);
		shape17.curveTo(19.842716, 75.87988, 19.70226, 75.819405, 19.586288, 75.69774);
		shape17.curveTo(19.470377, 75.57613, 19.41207, 75.41929, 19.41207, 75.22687);
		shape17.curveTo(19.41207, 75.10009, 19.439844, 74.98271, 19.494335, 74.87367);
		shape17.curveTo(19.548767, 74.765274, 19.623928, 74.68172, 19.71923, 74.62394);
		shape17.curveTo(19.814474, 74.566216, 19.922457, 74.53757, 20.043655, 74.53757);
		shape17.curveTo(20.250172, 74.53757, 20.418283, 74.61314, 20.548347, 74.76528);
		shape17.curveTo(20.67894, 74.91689, 20.743706, 75.16716, 20.743706, 75.51554);
		shape17.curveTo(20.743706, 75.90526, 20.671833, 76.18846, 20.527678, 76.36544);
		shape17.curveTo(20.402311, 76.51946, 20.233318, 76.59691, 20.020226, 76.59691);
		shape17.curveTo(19.86186, 76.59691, 19.731913, 76.552284, 19.630505, 76.46309);
		shape17.curveTo(19.529097, 76.374306, 19.468498, 76.25135, 19.4483, 76.094505);
		shape17.closePath();
		shape17.moveTo(20.45921, 75.22546);
		shape17.curveTo(20.45921, 75.1399, 20.44083, 75.05823, 20.404718, 74.9803);
		shape17.curveTo(20.368607, 74.90232, 20.317873, 74.84325, 20.253105, 74.80238);
		shape17.curveTo(20.187809, 74.76146, 20.119225, 74.74078, 20.047882, 74.74078);
		shape17.curveTo(19.943127, 74.74078, 19.853521, 74.78312, 19.77836, 74.86756);
		shape17.curveTo(19.703259, 74.95212, 19.665619, 75.066734, 19.665619, 75.2113);
		shape17.curveTo(19.665619, 75.3507, 19.702787, 75.460625, 19.77695, 75.54089);
		shape17.curveTo(19.851112, 75.62122, 19.944536, 75.66115, 20.057219, 75.66115);
		shape17.curveTo(20.169432, 75.66115, 20.264381, 75.621216, 20.342241, 75.54089);
		shape17.curveTo(20.420221, 75.460625, 20.45921, 75.3554, 20.45921, 75.22546);
		shape17.closePath();
		g.fill(shape17);
	}

	private void paintShapeNode_0_0_0_0_5_13(Graphics2D g) {
		GeneralPath shape18 = new GeneralPath();
		shape18.moveTo(150.5, 76.3243);
		shape18.lineTo(150.5, 76.56235);
		shape18.lineTo(149.19473, 76.56235);
		shape18.lineTo(149.19473, 76.369804);
		shape18.curveTo(149.32292, 76.23322, 149.45016, 76.05143, 149.57646, 75.82518);
		shape18.curveTo(149.7023, 75.59888, 149.79994, 75.36546, 149.86853, 75.12648);
		shape18.curveTo(149.91821, 74.95749, 149.94968, 74.772934, 149.96336, 74.57199);
		shape18.lineTo(150.21779, 74.57199);
		shape18.curveTo(150.21503, 74.730705, 150.18408, 74.92225, 150.12436, 75.14715);
		shape18.curveTo(150.06476, 75.371574, 149.97932, 75.588486, 149.868, 75.797);
		shape18.curveTo(149.75632, 76.00545, 149.63794, 76.18155, 149.51216, 76.324295);
		shape18.lineTo(150.5, 76.324295);
		shape18.closePath();
		g.fill(shape18);
	}

	private void paintShapeNode_0_0_0_0_5_14(Graphics2D g) {
		GeneralPath shape19 = new GeneralPath();
		shape19.moveTo(10.7483, 76.3243);
		shape19.lineTo(10.7483, 76.56235);
		shape19.lineTo(9.443029, 76.56235);
		shape19.lineTo(9.443029, 76.369804);
		shape19.curveTo(9.571155, 76.23322, 9.698459, 76.05143, 9.824704, 75.82518);
		shape19.curveTo(9.950598, 75.59888, 10.048248, 75.36546, 10.116832, 75.12648);
		shape19.curveTo(10.166509, 74.95749, 10.197982, 74.772934, 10.211663, 74.57199);
		shape19.lineTo(10.466094, 74.57199);
		shape19.curveTo(10.463276, 74.730705, 10.43233, 74.92225, 10.372672, 75.14715);
		shape19.curveTo(10.313072, 75.371574, 10.227635, 75.588486, 10.116304, 75.797);
		shape19.curveTo(10.004621, 76.00545, 9.886243, 76.18155, 9.760466, 76.324295);
		shape19.lineTo(10.7483, 76.324295);
		shape19.closePath();
		g.fill(shape19);
	}

	private void paintCompositeGraphicsNode_0_0_0_0_5(Graphics2D g) {
		// _0_0_0_0_5_0
		AffineTransform trans_0_0_0_0_5_0 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_0(g);
		g.setTransform(trans_0_0_0_0_5_0);
		// _0_0_0_0_5_1
		AffineTransform trans_0_0_0_0_5_1 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_1(g);
		g.setTransform(trans_0_0_0_0_5_1);
		// _0_0_0_0_5_2
		AffineTransform trans_0_0_0_0_5_2 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_2(g);
		g.setTransform(trans_0_0_0_0_5_2);
		// _0_0_0_0_5_3
		AffineTransform trans_0_0_0_0_5_3 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_3(g);
		g.setTransform(trans_0_0_0_0_5_3);
		// _0_0_0_0_5_4
		AffineTransform trans_0_0_0_0_5_4 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_4(g);
		g.setTransform(trans_0_0_0_0_5_4);
		// _0_0_0_0_5_5
		AffineTransform trans_0_0_0_0_5_5 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_5(g);
		g.setTransform(trans_0_0_0_0_5_5);
		// _0_0_0_0_5_6
		AffineTransform trans_0_0_0_0_5_6 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_6(g);
		g.setTransform(trans_0_0_0_0_5_6);
		// _0_0_0_0_5_7
		AffineTransform trans_0_0_0_0_5_7 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_7(g);
		g.setTransform(trans_0_0_0_0_5_7);
		// _0_0_0_0_5_8
		AffineTransform trans_0_0_0_0_5_8 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_8(g);
		g.setTransform(trans_0_0_0_0_5_8);
		// _0_0_0_0_5_9
		AffineTransform trans_0_0_0_0_5_9 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_9(g);
		g.setTransform(trans_0_0_0_0_5_9);
		// _0_0_0_0_5_10
		AffineTransform trans_0_0_0_0_5_10 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_10(g);
		g.setTransform(trans_0_0_0_0_5_10);
		// _0_0_0_0_5_11
		AffineTransform trans_0_0_0_0_5_11 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_11(g);
		g.setTransform(trans_0_0_0_0_5_11);
		// _0_0_0_0_5_12
		AffineTransform trans_0_0_0_0_5_12 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_12(g);
		g.setTransform(trans_0_0_0_0_5_12);
		// _0_0_0_0_5_13
		AffineTransform trans_0_0_0_0_5_13 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_13(g);
		g.setTransform(trans_0_0_0_0_5_13);
		// _0_0_0_0_5_14
		AffineTransform trans_0_0_0_0_5_14 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_5_14(g);
		g.setTransform(trans_0_0_0_0_5_14);
	}

	private void paintShapeNode_0_0_0_0_6_0(Graphics2D g) {
		GeneralPath shape20 = new GeneralPath();
		shape20.moveTo(103.929, 73.9517);
		shape20.lineTo(105.92545, 73.9517);
		shape20.moveTo(103.929, 72.95347);
		shape20.lineTo(105.92545, 72.95347);
		shape20.moveTo(103.929, 71.95524);
		shape20.lineTo(105.92545, 71.95524);
		shape20.moveTo(103.929, 70.95701);
		shape20.lineTo(105.92545, 70.95701);
		shape20.moveTo(103.929, 68.960556);
		shape20.lineTo(105.92545, 68.960556);
		shape20.moveTo(103.929, 67.962326);
		shape20.lineTo(105.92545, 67.962326);
		shape20.moveTo(103.929, 66.964096);
		shape20.lineTo(105.92545, 66.964096);
		shape20.moveTo(103.929, 65.965866);
		shape20.lineTo(105.92545, 65.965866);
		shape20.moveTo(103.929, 63.969418);
		shape20.lineTo(105.92545, 63.969418);
		shape20.moveTo(103.929, 62.97119);
		shape20.lineTo(105.92545, 62.97119);
		shape20.moveTo(103.929, 61.972965);
		shape20.lineTo(105.92545, 61.972965);
		shape20.moveTo(103.929, 60.97474);
		shape20.lineTo(105.92545, 60.97474);
		shape20.moveTo(103.929, 58.97829);
		shape20.lineTo(105.92545, 58.97829);
		shape20.moveTo(103.929, 57.980064);
		shape20.lineTo(105.92545, 57.980064);
		shape20.moveTo(103.929, 56.98184);
		shape20.lineTo(105.92545, 56.98184);
		shape20.moveTo(103.929, 55.983612);
		shape20.lineTo(105.92545, 55.983612);
		shape20.moveTo(103.929, 53.987164);
		shape20.lineTo(105.92545, 53.987164);
		shape20.moveTo(103.929, 52.988937);
		shape20.lineTo(105.92545, 52.988937);
		shape20.moveTo(103.929, 51.99071);
		shape20.lineTo(105.92545, 51.99071);
		shape20.moveTo(103.929, 50.992485);
		shape20.lineTo(105.92545, 50.992485);
		shape20.moveTo(103.929, 48.996037);
		shape20.lineTo(105.92545, 48.996037);
		shape20.moveTo(103.929, 47.99781);
		shape20.lineTo(105.92545, 47.99781);
		shape20.moveTo(103.929, 46.999584);
		shape20.lineTo(105.92545, 46.999584);
		shape20.moveTo(103.929, 46.001358);
		shape20.lineTo(105.92545, 46.001358);
		shape20.moveTo(102.93077, 74.94996);
		shape20.lineTo(105.92545, 74.94996);
		shape20.moveTo(102.93077, 64.96769);
		shape20.lineTo(105.92545, 64.96769);
		shape20.moveTo(102.93077, 54.98542);
		shape20.lineTo(105.92545, 54.98542);
		shape20.moveTo(102.731125, 69.95882);
		shape20.lineTo(105.92545, 69.95882);
		shape20.moveTo(102.731125, 59.976547);
		shape20.lineTo(105.92545, 59.976547);
		shape20.moveTo(102.731125, 49.994278);
		shape20.lineTo(105.92545, 49.994278);
		shape20.moveTo(56.014053, 73.95178);
		shape20.lineTo(54.017605, 73.95178);
		shape20.moveTo(56.014053, 72.95355);
		shape20.lineTo(54.017605, 72.95355);
		shape20.moveTo(56.014053, 71.95532);
		shape20.lineTo(54.017605, 71.95532);
		shape20.moveTo(56.014053, 70.95709);
		shape20.lineTo(54.017605, 70.95709);
		shape20.moveTo(56.014053, 68.96064);
		shape20.lineTo(54.017605, 68.96064);
		shape20.moveTo(56.014053, 67.96241);
		shape20.lineTo(54.017605, 67.96241);
		shape20.moveTo(56.014053, 66.96418);
		shape20.lineTo(54.017605, 66.96418);
		shape20.moveTo(56.014053, 65.96595);
		shape20.lineTo(54.017605, 65.96595);
		shape20.moveTo(56.014053, 63.9695);
		shape20.lineTo(54.017605, 63.9695);
		shape20.moveTo(56.014053, 62.971275);
		shape20.lineTo(54.017605, 62.971275);
		shape20.moveTo(56.014053, 61.97305);
		shape20.lineTo(54.017605, 61.97305);
		shape20.moveTo(56.014053, 60.974823);
		shape20.lineTo(54.017605, 60.974823);
		shape20.moveTo(56.014053, 58.978374);
		shape20.lineTo(54.017605, 58.978374);
		shape20.moveTo(56.014053, 57.98015);
		shape20.lineTo(54.017605, 57.98015);
		shape20.moveTo(56.014053, 56.981922);
		shape20.lineTo(54.017605, 56.981922);
		shape20.moveTo(56.014053, 55.983696);
		shape20.lineTo(54.017605, 55.983696);
		shape20.moveTo(56.014053, 53.987247);
		shape20.lineTo(54.017605, 53.987247);
		shape20.moveTo(56.014053, 52.98902);
		shape20.lineTo(54.017605, 52.98902);
		shape20.moveTo(56.014053, 51.990795);
		shape20.lineTo(54.017605, 51.990795);
		shape20.moveTo(56.014053, 50.99257);
		shape20.lineTo(54.017605, 50.99257);
		shape20.moveTo(56.014053, 48.99612);
		shape20.lineTo(54.017605, 48.99612);
		shape20.moveTo(56.014053, 47.997894);
		shape20.lineTo(54.017605, 47.997894);
		shape20.moveTo(56.014053, 46.99967);
		shape20.lineTo(54.017605, 46.99967);
		shape20.moveTo(56.014053, 46.001442);
		shape20.lineTo(54.017605, 46.001442);
		shape20.moveTo(57.012283, 74.95004);
		shape20.lineTo(54.017605, 74.95004);
		shape20.moveTo(57.012283, 64.96777);
		shape20.lineTo(54.017605, 64.96777);
		shape20.moveTo(57.012283, 54.985504);
		shape20.lineTo(54.017605, 54.985504);
		shape20.moveTo(57.211933, 69.95891);
		shape20.lineTo(54.017605, 69.95891);
		shape20.moveTo(57.211933, 59.97664);
		shape20.lineTo(54.017605, 59.97664);
		shape20.moveTo(57.211933, 49.99437);
		shape20.lineTo(54.017605, 49.99437);
		g.draw(shape20);
	}

	private void paintShapeNode_0_0_0_0_6_1(Graphics2D g) {
		GeneralPath shape21 = new GeneralPath();
		shape21.moveTo(82.8201, 29.1138);
		shape21.curveTo(107.5399, 30.47767, 127.552795, 49.4757, 130.4999, 73.740906);
		shape21.lineTo(127.52782, 74.10584);
		shape21.curveTo(124.75721, 51.293743, 105.96101, 33.42824, 82.73161, 32.108543);
		shape21.curveTo(82.18576, 33.0504, 81.166985, 33.683983, 80.000175, 33.683983);
		shape21.curveTo(78.83183, 33.683983, 77.81171, 33.048817, 77.26662, 32.10496);
		shape21.curveTo(54.014217, 33.399483, 35.191917, 51.272263, 32.416317, 74.09896);
		shape21.lineTo(29.444126, 73.734024);
		shape21.curveTo(32.396645, 49.452724, 52.437626, 30.446724, 77.18222, 29.110123);
		shape21.curveTo(77.70165, 28.080652, 78.7684, 27.374554, 80.00022, 27.374554);
		shape21.curveTo(81.2335, 27.374554, 82.30131, 28.082296, 82.8201, 29.113813);
		shape21.closePath();
		g.setPaint(new Color(255, 237, 0, 255));
		g.fill(shape21);
	}

	private void paintShapeNode_0_0_0_0_6_2_0(Graphics2D g) {
		GeneralPath shape22 = new GeneralPath();
		shape22.moveTo(100.986, 68.9463);
		shape22.lineTo(101.233505, 68.9463);
		shape22.lineTo(101.233505, 70.52391);
		shape22.curveTo(101.29317, 70.46707, 101.371025, 70.410286, 101.467735, 70.35351);
		shape22.curveTo(101.56497, 70.29667, 101.65182, 70.25398, 101.72892, 70.22574);
		shape22.lineTo(101.72892, 70.46478);
		shape22.curveTo(101.59034, 70.52996, 101.46915, 70.60894, 101.36545, 70.70136);
		shape22.curveTo(101.2621, 70.79443, 101.18847, 70.88404, 101.14525, 70.97141);
		shape22.lineTo(100.986, 70.97141);
		shape22.lineTo(100.986, 68.946304);
		shape22.closePath();
		g.setPaint(new Color(43, 42, 41, 255));
		g.fill(shape22);
	}

	private void paintShapeNode_0_0_0_0_6_2_1(Graphics2D g) {
		GeneralPath shape23 = new GeneralPath();
		shape23.moveTo(58.2291, 68.9463);
		shape23.lineTo(58.476543, 68.9463);
		shape23.lineTo(58.476543, 70.52391);
		shape23.curveTo(58.536263, 70.46707, 58.614124, 70.410286, 58.710835, 70.35351);
		shape23.curveTo(58.808075, 70.29667, 58.89492, 70.25398, 58.97196, 70.22574);
		shape23.lineTo(58.97196, 70.46478);
		shape23.curveTo(58.83338, 70.52996, 58.712242, 70.60894, 58.608486, 70.70136);
		shape23.curveTo(58.5052, 70.79443, 58.431507, 70.88404, 58.38829, 70.97141);
		shape23.lineTo(58.229103, 70.97141);
		shape23.lineTo(58.229103, 68.946304);
		shape23.closePath();
		g.fill(shape23);
	}

	private void paintShapeNode_0_0_0_0_6_2_2(Graphics2D g) {
		GeneralPath shape24 = new GeneralPath();
		shape24.moveTo(100.774, 59.2669);
		shape24.lineTo(100.774, 59.028793);
		shape24.lineTo(102.10704, 59.028793);
		shape24.curveTo(102.10892, 59.088394, 102.09959, 59.145702, 102.078384, 59.20072);
		shape24.curveTo(102.04456, 59.291737, 101.99013, 59.38099, 101.9155, 59.469246);
		shape24.curveTo(101.84034, 59.55709, 101.73229, 59.65891, 101.591484, 59.77441);
		shape24.curveTo(101.37222, 59.954266, 101.22385, 60.096485, 101.146866, 60.20171);
		shape24.curveTo(101.069885, 60.306465, 101.031364, 60.405937, 101.031364, 60.499416);
		shape24.curveTo(101.031364, 60.597538, 101.0666, 60.680153, 101.136475, 60.74774);
		shape24.curveTo(101.20694, 60.814915, 101.29848, 60.84868, 101.41116, 60.84868);
		shape24.curveTo(101.53048, 60.84868, 101.625786, 60.813038, 101.69718, 60.741165);
		shape24.curveTo(101.768936, 60.669823, 101.80511, 60.570763, 101.80611, 60.444515);
		shape24.lineTo(102.06054, 60.470234);
		shape24.curveTo(102.04316, 60.660427, 101.97751, 60.80505, 101.86377, 60.90458);
		shape24.curveTo(101.750206, 61.00411, 101.59718, 61.0539, 101.40559, 61.0539);
		shape24.curveTo(101.212166, 61.0539, 101.05903, 60.99988, 100.94634, 60.892895);
		shape24.curveTo(100.83319, 60.785263, 100.77682, 60.65244, 100.77682, 60.49372);
		shape24.curveTo(100.77682, 60.41292, 100.79326, 60.333595, 100.826614, 60.255733);
		shape24.curveTo(100.8595, 60.177753, 100.91446, 60.095608, 100.99091, 60.00964);
		shape24.curveTo(101.06748, 59.923206, 101.194786, 59.804943, 101.37265, 59.654682);
		shape24.curveTo(101.521095, 59.529846, 101.616455, 59.44529, 101.65867, 59.40066);
		shape24.curveTo(101.70089, 59.356506, 101.73559, 59.31188, 101.76337, 59.2669);
		shape24.lineTo(100.774, 59.2669);
		shape24.closePath();
		g.fill(shape24);
	}

	private void paintShapeNode_0_0_0_0_6_2_3(Graphics2D g) {
		GeneralPath shape25 = new GeneralPath();
		shape25.moveTo(58.0168, 59.2669);
		shape25.lineTo(58.0168, 59.028793);
		shape25.lineTo(59.34984, 59.028793);
		shape25.curveTo(59.35172, 59.088394, 59.342323, 59.145702, 59.321182, 59.20072);
		shape25.curveTo(59.2873, 59.291737, 59.232925, 59.38099, 59.158237, 59.469246);
		shape25.curveTo(59.083076, 59.55709, 58.975094, 59.65891, 58.834225, 59.77441);
		shape25.curveTo(58.615025, 59.954266, 58.466583, 60.096485, 58.389603, 60.20171);
		shape25.curveTo(58.31268, 60.306465, 58.2741, 60.405937, 58.2741, 60.499416);
		shape25.curveTo(58.2741, 60.597538, 58.309334, 60.680153, 58.379208, 60.74774);
		shape25.curveTo(58.44973, 60.814915, 58.54127, 60.84868, 58.653954, 60.84868);
		shape25.curveTo(58.773212, 60.84868, 58.868572, 60.813038, 58.939915, 60.741165);
		shape25.curveTo(59.01173, 60.669823, 59.04784, 60.570763, 59.04884, 60.444515);
		shape25.lineTo(59.303272, 60.470234);
		shape25.curveTo(59.28595, 60.660427, 59.22024, 60.80505, 59.106564, 60.90458);
		shape25.curveTo(58.992943, 61.00411, 58.83992, 61.0539, 58.648376, 61.0539);
		shape25.curveTo(58.454895, 61.0539, 58.301758, 60.99988, 58.189075, 60.892895);
		shape25.curveTo(58.07598, 60.785263, 58.019554, 60.65244, 58.019554, 60.49372);
		shape25.curveTo(58.019554, 60.41292, 58.036053, 60.333595, 58.06941, 60.255733);
		shape25.curveTo(58.102234, 60.177753, 58.157253, 60.095608, 58.233704, 60.00964);
		shape25.curveTo(58.310272, 59.923206, 58.43752, 59.804943, 58.615437, 59.654682);
		shape25.curveTo(58.763878, 59.529846, 58.85918, 59.44529, 58.901398, 59.40066);
		shape25.curveTo(58.943615, 59.356506, 58.97838, 59.31188, 59.006092, 59.2669);
		shape25.lineTo(58.01679, 59.2669);
		shape25.closePath();
		g.fill(shape25);
	}

	private void paintShapeNode_0_0_0_0_6_2_4(Graphics2D g) {
		GeneralPath shape26 = new GeneralPath();
		shape26.moveTo(101.913, 49.5444);
		shape26.lineTo(101.66556, 49.577225);
		shape26.curveTo(101.6369, 49.436886, 101.58852, 49.33595, 101.5204, 49.274);
		shape26.curveTo(101.451935, 49.21199, 101.36879, 49.180927, 101.27067, 49.180927);
		shape26.curveTo(101.15429, 49.180927, 101.05564, 49.221386, 100.97525, 49.302124);
		shape26.curveTo(100.89551, 49.382862, 100.85511, 49.48292, 100.85511, 49.60218);
		shape26.curveTo(100.85511, 49.7158, 100.89222, 49.809696, 100.966385, 49.88339);
		shape26.curveTo(101.04108, 49.95708, 101.135376, 49.99419, 101.25, 49.99419);
		shape26.curveTo(101.29691, 49.99419, 101.355225, 49.984795, 101.42469, 49.966534);
		shape26.lineTo(101.39692, 50.183853);
		shape26.curveTo(101.38054, 50.182034, 101.367386, 50.181034, 101.35711, 50.181034);
		shape26.curveTo(101.251884, 50.181034, 101.15705, 50.20881, 101.072495, 50.26371);
		shape26.curveTo(100.98794, 50.31861, 100.94572, 50.403637, 100.94572, 50.518143);
		shape26.curveTo(100.94572, 50.608807, 100.976776, 50.684376, 101.03826, 50.743565);
		shape26.curveTo(101.09974, 50.803165, 101.17913, 50.833286, 101.27637, 50.833286);
		shape26.curveTo(101.37255, 50.833286, 101.45282, 50.802753, 101.51671, 50.742683);
		shape26.curveTo(101.58101, 50.682087, 101.622345, 50.590954, 101.640724, 50.46987);
		shape26.lineTo(101.888054, 50.51397);
		shape26.curveTo(101.85805, 50.680145, 101.789055, 50.808857, 101.681946, 50.89987);
		shape26.curveTo(101.57443, 50.991005, 101.44108, 51.03704, 101.281425, 51.03704);
		shape26.curveTo(101.17162, 51.03704, 101.07021, 51.013084, 100.97767, 50.966167);
		shape26.curveTo(100.885124, 50.918724, 100.814255, 50.854366, 100.764984, 50.772686);
		shape26.curveTo(100.71613, 50.69095, 100.69129, 50.60463, 100.69129, 50.51256);
		shape26.curveTo(100.69129, 50.425716, 100.71472, 50.346386, 100.761635, 50.274513);
		shape26.curveTo(100.80867, 50.20311, 100.87766, 50.146328, 100.96921, 50.10411);
		shape26.curveTo(100.850006, 50.076866, 100.75747, 50.019554, 100.69129, 49.93271);
		shape26.curveTo(100.62553, 49.846275, 100.59264, 49.73782, 100.59264, 49.607758);
		shape26.curveTo(100.59264, 49.431717, 100.65653, 49.282337, 100.785126, 49.159847);
		shape26.curveTo(100.91337, 49.0373, 101.07578, 48.97629, 101.27209, 48.97629);
		shape26.curveTo(101.449066, 48.97629, 101.5961, 49.028786, 101.71295, 49.13454);
		shape26.curveTo(101.829865, 49.239647, 101.89657, 49.37629, 101.91301, 49.5444);
		shape26.closePath();
		g.fill(shape26);
	}

	private void paintShapeNode_0_0_0_0_6_2_5(Graphics2D g) {
		GeneralPath shape27 = new GeneralPath();
		shape27.moveTo(59.1565, 49.5444);
		shape27.lineTo(58.909, 49.577225);
		shape27.curveTo(58.880344, 49.436886, 58.831963, 49.33595, 58.763905, 49.274);
		shape27.curveTo(58.695377, 49.21199, 58.612232, 49.180927, 58.514114, 49.180927);
		shape27.curveTo(58.39773, 49.180927, 58.299084, 49.221386, 58.218758, 49.302124);
		shape27.curveTo(58.139015, 49.382862, 58.09856, 49.48292, 58.09856, 49.60218);
		shape27.curveTo(58.09856, 49.7158, 58.13567, 49.809696, 58.209892, 49.88339);
		shape27.curveTo(58.284523, 49.95708, 58.378887, 49.99419, 58.493446, 49.99419);
		shape27.curveTo(58.540363, 49.99419, 58.59867, 49.984795, 58.668137, 49.966534);
		shape27.lineTo(58.640423, 50.183853);
		shape27.curveTo(58.62404, 50.182034, 58.61083, 50.181034, 58.60055, 50.181034);
		shape27.curveTo(58.495384, 50.181034, 58.400555, 50.20881, 58.315998, 50.26371);
		shape27.curveTo(58.23144, 50.31861, 58.189224, 50.403637, 58.189224, 50.518143);
		shape27.curveTo(58.189224, 50.608807, 58.220226, 50.684376, 58.281765, 50.743565);
		shape27.curveTo(58.343243, 50.803165, 58.422573, 50.833286, 58.519814, 50.833286);
		shape27.curveTo(58.616055, 50.833286, 58.696323, 50.802753, 58.760212, 50.742683);
		shape27.curveTo(58.82445, 50.682087, 58.865788, 50.590954, 58.884167, 50.46987);
		shape27.lineTo(59.13155, 50.51397);
		shape27.curveTo(59.101486, 50.680145, 59.03249, 50.808857, 58.92539, 50.89987);
		shape27.curveTo(58.817875, 50.991005, 58.68452, 51.03704, 58.524925, 51.03704);
		shape27.curveTo(58.41512, 51.03704, 58.313713, 51.013084, 58.22117, 50.966167);
		shape27.curveTo(58.128628, 50.918724, 58.05775, 50.854366, 58.008427, 50.772686);
		shape27.curveTo(57.959633, 50.69095, 57.934734, 50.60463, 57.934734, 50.51256);
		shape27.curveTo(57.934734, 50.425716, 57.95822, 50.346386, 58.00514, 50.274513);
		shape27.curveTo(58.052174, 50.20311, 58.12111, 50.146328, 58.212654, 50.10411);
		shape27.curveTo(58.093452, 50.076866, 58.00091, 50.019554, 57.934734, 49.93271);
		shape27.curveTo(57.86897, 49.846275, 57.836143, 49.73782, 57.836143, 49.607758);
		shape27.curveTo(57.836143, 49.431717, 57.90003, 49.282337, 58.02857, 49.159847);
		shape27.curveTo(58.15687, 49.0373, 58.319286, 48.97629, 58.515526, 48.97629);
		shape27.curveTo(58.692505, 48.97629, 58.83954, 49.028786, 58.956448, 49.13454);
		shape27.curveTo(59.073357, 49.239647, 59.140003, 49.37629, 59.156506, 49.5444);
		shape27.closePath();
		g.fill(shape27);
	}

	private void paintCompositeGraphicsNode_0_0_0_0_6_2(Graphics2D g) {
		// _0_0_0_0_6_2_0
		AffineTransform trans_0_0_0_0_6_2_0 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_2_0(g);
		g.setTransform(trans_0_0_0_0_6_2_0);
		// _0_0_0_0_6_2_1
		AffineTransform trans_0_0_0_0_6_2_1 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_2_1(g);
		g.setTransform(trans_0_0_0_0_6_2_1);
		// _0_0_0_0_6_2_2
		AffineTransform trans_0_0_0_0_6_2_2 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_2_2(g);
		g.setTransform(trans_0_0_0_0_6_2_2);
		// _0_0_0_0_6_2_3
		AffineTransform trans_0_0_0_0_6_2_3 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_2_3(g);
		g.setTransform(trans_0_0_0_0_6_2_3);
		// _0_0_0_0_6_2_4
		AffineTransform trans_0_0_0_0_6_2_4 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_2_4(g);
		g.setTransform(trans_0_0_0_0_6_2_4);
		// _0_0_0_0_6_2_5
		AffineTransform trans_0_0_0_0_6_2_5 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_2_5(g);
		g.setTransform(trans_0_0_0_0_6_2_5);
	}

	private void paintShapeNode_0_0_0_0_6_3(Graphics2D g) {
		GeneralPath shape28 = new GeneralPath();
		shape28.moveTo(67.1889, 40.0119);
		shape28.lineTo(78.973305, 40.0119);
		shape28.moveTo(80.96976, 40.0119);
		shape28.lineTo(92.81096, 40.0119);
		shape28.moveTo(60.006958, 45.00304);
		shape28.lineTo(78.97326, 45.00304);
		shape28.moveTo(80.96971, 45.00304);
		shape28.lineTo(99.93601, 45.00304);
		shape28.moveTo(50.662212, 49.99418);
		shape28.lineTo(53.01932, 49.99418);
		shape28.moveTo(60.006912, 49.99418);
		shape28.lineTo(78.97321, 49.99418);
		shape28.moveTo(80.969666, 49.99418);
		shape28.lineTo(99.93597, 49.99418);
		shape28.moveTo(106.92355, 49.99418);
		shape28.lineTo(109.33739, 49.99418);
		shape28.moveTo(46.30919, 54.985317);
		shape28.lineTo(53.019268, 54.985317);
		shape28.moveTo(60.00686, 54.985317);
		shape28.lineTo(78.97316, 54.985317);
		shape28.moveTo(80.96961, 54.985317);
		shape28.lineTo(99.93591, 54.985317);
		shape28.moveTo(106.9235, 54.985317);
		shape28.lineTo(113.69031, 54.985317);
		shape28.moveTo(43.12401, 59.976456);
		shape28.lineTo(53.0192, 59.976456);
		shape28.moveTo(60.00679, 59.976456);
		shape28.lineTo(78.97309, 59.976456);
		shape28.moveTo(80.96954, 59.976456);
		shape28.lineTo(99.935844, 59.976456);
		shape28.moveTo(106.92343, 59.976456);
		shape28.lineTo(116.87541, 59.976456);
		shape28.moveTo(40.830315, 64.9676);
		shape28.lineTo(53.019115, 64.9676);
		shape28.moveTo(60.006706, 64.9676);
		shape28.lineTo(78.97301, 64.9676);
		shape28.moveTo(80.96946, 64.9676);
		shape28.lineTo(99.93576, 64.9676);
		shape28.moveTo(106.92335, 64.9676);
		shape28.lineTo(119.168846, 64.9676);
		shape28.moveTo(39.27295, 69.95874);
		shape28.lineTo(53.01905, 69.95874);
		shape28.moveTo(60.00664, 69.95874);
		shape28.lineTo(78.97294, 69.95874);
		shape28.moveTo(80.96939, 69.95874);
		shape28.lineTo(99.93569, 69.95874);
		shape28.moveTo(106.92328, 69.95874);
		shape28.lineTo(120.72608, 69.95874);
		shape28.moveTo(38.37088, 74.94988);
		shape28.lineTo(39.043743, 74.94988);
		shape28.moveTo(41.04019, 74.94988);
		shape28.lineTo(49.026012, 74.94988);
		shape28.moveTo(51.02246, 74.94988);
		shape28.lineTo(53.01891, 74.94988);
		shape28.moveTo(61.00473, 74.94988);
		shape28.lineTo(68.99055, 74.94988);
		shape28.moveTo(70.987, 74.94988);
		shape28.lineTo(78.97282, 74.94988);
		shape28.moveTo(80.96927, 74.94988);
		shape28.lineTo(88.955086, 74.94988);
		shape28.moveTo(90.95154, 74.94988);
		shape28.lineTo(98.937355, 74.94988);
		shape28.moveTo(106.92317, 74.94988);
		shape28.lineTo(108.919624, 74.94988);
		shape28.moveTo(110.91608, 74.94988);
		shape28.lineTo(118.90189, 74.94988);
		shape28.moveTo(120.898346, 74.94988);
		shape28.lineTo(121.62799, 74.94988);
		shape28.moveTo(79.99949, 38.023884);
		shape28.lineTo(79.99949, 74.09749);
		shape28.moveTo(70.98696, 70.95695);
		shape28.lineTo(73.98164, 73.95163);
		shape28.moveTo(65.99583, 65.96581);
		shape28.lineTo(68.99051, 68.960495);
		shape28.moveTo(61.00469, 60.974674);
		shape28.lineTo(63.999367, 63.969353);
		shape28.moveTo(51.022366, 50.99235);
		shape28.lineTo(53.018814, 52.9888);
		shape28.moveTo(88.98392, 70.9564);
		shape28.lineTo(85.989235, 73.95108);
		shape28.moveTo(93.97505, 65.96526);
		shape28.lineTo(90.98037, 68.959946);
		shape28.moveTo(98.96619, 60.974186);
		shape28.lineTo(95.971504, 63.968807);
		shape28.moveTo(108.9485, 50.991905);
		shape28.lineTo(106.923164, 53.017197);
		g.draw(shape28);
	}

	private void paintShapeNode_0_0_0_0_6_4(Graphics2D g) {
		GeneralPath shape29 = new GeneralPath();
		shape29.moveTo(129.697, 71.7405);
		shape29.lineTo(129.73999, 71.984184);
		shape29.lineTo(128.18634, 72.25817);
		shape29.curveTo(128.25264, 72.30697, 128.3221, 72.37385, 128.39485, 72.45928);
		shape29.curveTo(128.46767, 72.54513, 128.52486, 72.62322, 128.56596, 72.69422);
		shape29.lineTo(128.33061, 72.73573);
		shape29.curveTo(128.2423, 72.6106, 128.14359, 72.50503, 128.03444, 72.41882);
		shape29.curveTo(127.92499, 72.33327, 127.823875, 72.27625, 127.73039, 72.248886);
		shape29.lineTo(127.702736, 72.09216);
		shape29.lineTo(129.69702, 71.74049);
		shape29.closePath();
		g.fill(shape29);
	}

	private void paintShapeNode_0_0_0_0_6_5(Graphics2D g) {
		GeneralPath shape30 = new GeneralPath();
		shape30.moveTo(128.608, 71.2885);
		shape30.curveTo(128.37312, 71.329895, 128.17976, 71.33917, 128.02832, 71.315865);
		shape30.curveTo(127.87647, 71.29302, 127.75328, 71.24041, 127.6588, 71.15879);
		shape30.curveTo(127.56426, 71.077286, 127.504364, 70.96437, 127.47894, 70.8201);
		shape30.curveTo(127.46009, 70.71329, 127.46532, 70.61605, 127.49415, 70.52803);
		shape30.curveTo(127.52251, 70.44001, 127.57278, 70.36303, 127.64441, 70.29603);
		shape30.curveTo(127.71616, 70.2295, 127.80736, 70.17096, 127.91822, 70.120926);
		shape30.curveTo(128.02896, 70.07043, 128.18263, 70.02809, 128.37964, 69.99333);
		shape30.curveTo(128.61264, 69.952225, 128.80464, 69.94313, 128.95638, 69.9655);
		shape30.curveTo(129.10817, 69.98834, 129.23135, 70.04054, 129.32631, 70.12199);
		shape30.curveTo(129.42126, 70.20349, 129.48132, 70.31687, 129.50699, 70.4625);
		shape30.curveTo(129.54076, 70.65392, 129.49876, 70.81622, 129.38068, 70.94964);
		shape30.curveTo(129.2387, 71.10953, 128.98068, 71.2228, 128.60799, 71.288506);
		shape30.closePath();
		shape30.moveTo(128.56378, 71.03789);
		shape30.curveTo(128.88979, 70.9804, 129.09988, 70.90377, 129.19466, 70.80841);
		shape30.curveTo(129.28896, 70.71311, 129.32642, 70.6097, 129.30669, 70.49779);
		shape30.curveTo(129.2869, 70.3854, 129.21579, 70.30119, 129.09418, 70.244);
		shape30.curveTo(128.9725, 70.18675, 128.74886, 70.18664, 128.42378, 70.24394);
		shape30.curveTo(128.0969, 70.301605, 127.88632, 70.378235, 127.79255, 70.47347);
		shape30.curveTo(127.69819, 70.56877, 127.66131, 70.673004, 127.681335, 70.78674);
		shape30.curveTo(127.70118, 70.89913, 127.76448, 70.97999, 127.87118, 71.03031);
		shape30.curveTo(128.00659, 71.09273, 128.23782, 71.095375, 128.56377, 71.03789);
		shape30.closePath();
		g.fill(shape30);
	}

	private void paintShapeNode_0_0_0_0_6_6(Graphics2D g) {
		GeneralPath shape31 = new GeneralPath();
		shape31.moveTo(33.8747, 71.4086);
		shape31.lineTo(33.917683, 71.16492);
		shape31.lineTo(35.47127, 71.4389);
		shape31.curveTo(35.425705, 71.37032, 35.38325, 71.28371, 35.344143, 71.17854);
		shape31.curveTo(35.305035, 71.07302, 35.278027, 70.98007, 35.2637, 70.89928);
		shape31.lineTo(35.499104, 70.94079);
		shape31.curveTo(35.53927, 71.088524, 35.59587, 71.22153, 35.66898, 71.33984);
		shape31.curveTo(35.742615, 71.457695, 35.818127, 71.54583, 35.896576, 71.60355);
		shape31.lineTo(35.868977, 71.76028);
		shape31.lineTo(33.874695, 71.40861);
		shape31.closePath();
		g.fill(shape31);
	}

	private void paintShapeNode_0_0_0_0_6_7(Graphics2D g) {
		GeneralPath shape32 = new GeneralPath();
		shape32.moveTo(34.7439, 72.206);
		shape32.curveTo(34.97878, 72.24746, 35.163624, 72.304825, 35.297974, 72.37858);
		shape32.curveTo(35.43285, 72.45198, 35.53062, 72.54352, 35.59151, 72.652565);
		shape32.curveTo(35.6524, 72.76149, 35.670135, 72.888084, 35.644653, 73.032364);
		shape32.curveTo(35.625862, 73.139175, 35.587696, 73.22872, 35.530502, 73.30159);
		shape32.curveTo(35.47372, 73.37458, 35.400146, 73.42977, 35.30995, 73.46823);
		shape32.curveTo(35.219757, 73.506165, 35.114063, 73.53001, 34.992752, 73.53911);
		shape32.curveTo(34.87138, 73.54868, 34.712486, 73.535934, 34.51554, 73.501175);
		shape32.curveTo(34.282486, 73.46007, 34.098927, 73.403, 33.96405, 73.33007);
		shape32.curveTo(33.829174, 73.25667, 33.73129, 73.16554, 33.669926, 73.05656);
		shape32.curveTo(33.608624, 72.94746, 33.59095, 72.82039, 33.616608, 72.67477);
		shape32.curveTo(33.65037, 72.483284, 33.74532, 72.345116, 33.901924, 72.26021);
		shape32.curveTo(34.090004, 72.15851, 34.37121, 72.140305, 34.7439, 72.20601);
		shape32.closePath();
		shape32.moveTo(34.699684, 72.45667);
		shape32.curveTo(34.373734, 72.399185, 34.150074, 72.39931, 34.028404, 72.4565);
		shape32.curveTo(33.907207, 72.51381, 33.836628, 72.59819, 33.8169, 72.710045);
		shape32.curveTo(33.79705, 72.82243, 33.83504, 72.925896, 33.929817, 73.021255);
		shape32.curveTo(34.024532, 73.11668, 34.234688, 73.193245, 34.5597, 73.25056);
		shape32.curveTo(34.88665, 73.30822, 35.11072, 73.30816, 35.23145, 73.25079);
		shape32.curveTo(35.352646, 73.19348, 35.422993, 73.10816, 35.443012, 72.99442);
		shape32.curveTo(35.46286, 72.882095, 35.430973, 72.784386, 35.348003, 72.70065);
		shape32.curveTo(35.242073, 72.595665, 35.025692, 72.51416, 34.699684, 72.45667);
		shape32.closePath();
		g.fill(shape32);
	}

	private void paintShapeNode_0_0_0_0_6_8(Graphics2D g) {
		GeneralPath shape33 = new GeneralPath();
		shape33.moveTo(127.123, 62.8875);
		shape33.lineTo(127.34672, 62.806057);
		shape33.lineTo(127.80261, 64.058716);
		shape33.curveTo(127.7473, 64.08091, 127.69022, 64.09166, 127.631325, 64.0906);
		shape33.curveTo(127.5342, 64.09001, 127.43168, 64.06934, 127.32328, 64.02941);
		shape33.curveTo(127.21494, 63.988777, 127.08235, 63.922073, 126.92564, 63.82924);
		shape33.curveTo(126.68172, 63.68479, 126.49722, 63.593952, 126.37203, 63.557606);
		shape33.curveTo(126.24725, 63.521084, 126.14068, 63.51891, 126.05283, 63.550854);
		shape33.curveTo(125.96059, 63.5845, 125.895, 63.645863, 125.85536, 63.734646);
		shape33.curveTo(125.816315, 63.82378, 125.81596, 63.92137, 125.854485, 64.027306);
		shape33.curveTo(125.895294, 64.139404, 125.96136, 64.21679, 126.05331, 64.25931);
		shape33.curveTo(126.144966, 64.30229, 126.25037, 64.30241, 126.36933, 64.26007);
		shape33.lineTo(126.43222, 64.50804);
		shape33.curveTo(126.24755, 64.55678, 126.08913, 64.54451, 125.95677, 64.471695);
		shape33.curveTo(125.82442, 64.39894, 125.7253, 64.272285, 125.65971, 64.09219);
		shape33.curveTo(125.59354, 63.9104, 125.59195, 63.7481, 125.6539, 63.60553);
		shape33.curveTo(125.71644, 63.46249, 125.82195, 63.364075, 125.97105, 63.30976);
		shape33.curveTo(126.04703, 63.282104, 126.12718, 63.270477, 126.21174, 63.275116);
		shape33.curveTo(126.296295, 63.279343, 126.3923, 63.30301, 126.49923, 63.3454);
		shape33.curveTo(126.60663, 63.387737, 126.76129, 63.466892, 126.96329, 63.582687);
		shape33.curveTo(127.1314, 63.679516, 127.24349, 63.740112, 127.299805, 63.7646);
		shape33.curveTo(127.355705, 63.789143, 127.40961, 63.806526, 127.46134, 63.817154);
		shape33.lineTo(127.123, 62.887512);
		shape33.closePath();
		shape33.moveTo(126.32154, 62.897015);
		shape33.curveTo(126.09735, 62.978577, 125.90845, 63.021263, 125.75537, 63.02455);
		shape33.curveTo(125.60194, 63.028484, 125.471344, 62.99801, 125.36413, 62.934124);
		shape33.curveTo(125.256966, 62.87018, 125.17828, 62.769417, 125.1282, 62.63178);
		shape33.curveTo(125.09115, 62.529842, 125.079346, 62.433132, 125.09255, 62.34147);
		shape33.curveTo(125.10512, 62.249985, 125.14135, 62.165195, 125.20024, 62.08686);
		shape33.curveTo(125.259315, 62.009, 125.339, 61.935425, 125.43941, 61.866955);
		shape33.curveTo(125.53976, 61.797844, 125.6838, 61.729553, 125.87176, 61.661083);
		shape33.curveTo(126.09407, 61.580227, 126.2815, 61.53789, 126.434875, 61.533604);
		shape33.curveTo(126.58825, 61.529728, 126.718666, 61.559673, 126.82624, 61.623444);
		shape33.curveTo(126.93399, 61.68727, 127.01273, 61.788445, 127.06335, 61.927433);
		shape33.curveTo(127.129814, 62.110107, 127.11672, 62.277283, 127.0236, 62.42913);
		shape33.curveTo(126.9115, 62.61128, 126.67709, 62.76753, 126.32155, 62.897007);
		shape33.closePath();
		shape33.moveTo(126.23452, 62.657852);
		shape33.curveTo(126.545555, 62.544525, 126.7391, 62.432663, 126.81584, 62.32233);
		shape33.curveTo(126.89212, 62.212116, 126.9112, 62.103718, 126.87239, 61.997025);
		shape33.curveTo(126.83334, 61.889805, 126.74867, 61.819107, 126.6189, 61.783875);
		shape33.curveTo(126.48913, 61.74876, 126.268814, 61.7874, 125.95877, 61.900257);
		shape33.curveTo(125.646736, 62.01376, 125.45267, 62.125973, 125.3768, 62.236015);
		shape33.curveTo(125.30052, 62.346172, 125.28238, 62.455215, 125.32184, 62.563786);
		shape33.curveTo(125.360825, 62.671005, 125.43728, 62.73959, 125.55102, 62.770714);
		shape33.curveTo(125.69524, 62.808586, 125.923416, 62.771008, 126.23451, 62.657856);
		shape33.closePath();
		g.fill(shape33);
	}

	private void paintShapeNode_0_0_0_0_6_9(Graphics2D g) {
		GeneralPath shape34 = new GeneralPath();
		shape34.moveTo(36.1457, 63.9556);
		shape34.lineTo(35.92192, 63.874214);
		shape34.lineTo(36.377934, 62.621555);
		shape34.curveTo(36.43454, 62.64011, 36.485214, 62.66859, 36.529606, 62.707287);
		shape34.curveTo(36.60365, 62.770176, 36.668888, 62.851913, 36.726257, 62.952145);
		shape34.curveTo(36.783096, 63.052906, 36.841816, 63.189194, 36.90218, 63.361065);
		shape34.curveTo(36.996246, 63.628532, 37.07916, 63.81667, 37.151676, 63.925003);
		shape34.curveTo(37.22378, 64.033165, 37.30405, 64.10339, 37.391838, 64.13539);
		shape34.curveTo(37.484146, 64.16892, 37.573868, 64.16405, 37.661243, 64.12154);
		shape34.curveTo(37.74844, 64.078316, 37.81151, 64.00381, 37.850086, 63.897873);
		shape34.curveTo(37.89084, 63.785778, 37.889957, 63.684017, 37.846855, 63.592358);
		shape34.curveTo(37.804344, 63.500523, 37.723663, 63.43264, 37.605286, 63.388603);
		shape34.lineTo(37.7165, 63.15825);
		shape34.curveTo(37.88925, 63.239635, 38.002754, 63.35085, 38.057365, 63.491714);
		shape34.curveTo(38.111973, 63.632523, 38.106453, 63.793297, 38.04098, 63.97339);
		shape34.curveTo(37.974804, 64.15518, 37.871693, 64.28055, 37.732586, 64.3499);
		shape34.curveTo(37.592773, 64.419304, 37.448677, 64.42688, 37.29953, 64.37263);
		shape34.curveTo(37.22355, 64.34497, 37.15467, 64.3024, 37.092896, 64.244446);
		shape34.curveTo(37.03083, 64.1869, 36.97246, 64.10704, 36.917854, 64.00581);
		shape34.curveTo(36.862774, 63.904346, 36.79519, 63.744335, 36.714863, 63.525726);
		shape34.curveTo(36.648335, 63.34352, 36.601418, 63.225025, 36.573994, 63.170124);
		shape34.curveTo(36.54698, 63.1154, 36.5168, 63.067425, 36.484093, 63.026028);
		shape34.lineTo(36.145695, 63.955612);
		shape34.closePath();
		shape34.moveTo(36.765717, 64.463524);
		shape34.curveTo(36.989906, 64.54514, 37.16207, 64.63381, 37.28145, 64.72976);
		shape34.curveTo(37.40153, 64.825356, 37.481976, 64.93263, 37.523018, 65.05048);
		shape34.curveTo(37.564003, 65.16839, 37.55954, 65.296104, 37.509453, 65.4338);
		shape34.curveTo(37.472282, 65.535675, 37.419144, 65.617355, 37.350147, 65.67907);
		shape34.curveTo(37.28168, 65.741135, 37.199417, 65.78271, 37.10394, 65.8049);
		shape34.curveTo(37.008636, 65.82657, 36.900303, 65.83173, 36.7794, 65.81964);
		shape34.curveTo(36.658146, 65.808075, 36.503887, 65.76779, 36.31587, 65.69939);
		shape34.curveTo(36.093616, 65.61847, 35.922802, 65.530396, 35.802605, 65.43509);
		shape34.curveTo(35.68252, 65.33949, 35.601902, 65.23274, 35.560505, 65.114716);
		shape34.curveTo(35.51899, 64.996574, 35.52369, 64.86845, 35.574245, 64.72946);
		shape34.curveTo(35.640774, 64.54678, 35.75827, 64.42712, 35.927208, 64.37069);
		shape34.curveTo(36.130142, 64.30316, 36.410114, 64.33417, 36.76572, 64.463524);
		shape34.closePath();
		shape34.moveTo(36.678696, 64.70269);
		shape34.curveTo(36.3676, 64.58954, 36.147346, 64.55084, 36.017693, 64.58601);
		shape34.curveTo(35.888393, 64.62142, 35.80413, 64.69218, 35.76526, 64.79893);
		shape34.curveTo(35.72627, 64.90615, 35.745647, 65.01466, 35.822453, 65.12505);
		shape34.curveTo(35.899258, 65.23539, 36.092854, 65.34742, 36.40295, 65.46022);
		shape34.curveTo(36.714924, 65.573845, 36.93571, 65.612656, 37.06454, 65.57713);
		shape34.curveTo(37.193783, 65.54179, 37.27781, 65.46991, 37.317387, 65.3614);
		shape34.curveTo(37.356377, 65.25417, 37.34193, 65.15247, 37.274815, 65.05553);
		shape34.curveTo(37.188675, 64.93386, 36.98973, 64.815895, 36.678696, 64.70268);
		shape34.closePath();
		g.fill(shape34);
	}

	private void paintShapeNode_0_0_0_0_6_10(Graphics2D g) {
		GeneralPath shape35 = new GeneralPath();
		shape35.moveTo(123.909, 56.2274);
		shape35.lineTo(123.75685, 56.029457);
		shape35.curveTo(123.864075, 55.93451, 123.92725, 55.842083, 123.94692, 55.752125);
		shape35.curveTo(123.96636, 55.661816, 123.951675, 55.574383, 123.90259, 55.489296);
		shape35.curveTo(123.8444, 55.388474, 123.76002, 55.323296, 123.64992, 55.29417);
		shape35.curveTo(123.540115, 55.265396, 123.433304, 55.280487, 123.33002, 55.340088);
		shape35.curveTo(123.23166, 55.39687, 123.16889, 55.476025, 123.14211, 55.57708);
		shape35.curveTo(123.11569, 55.678604, 123.13066, 55.77878, 123.18797, 55.878017);
		shape35.curveTo(123.2114, 55.91877, 123.24875, 55.96445, 123.29925, 56.0156);
		shape35.lineTo(123.0972, 56.100155);
		shape35.curveTo(123.09051, 56.085064, 123.08481, 56.073204, 123.07964, 56.064278);
		shape35.curveTo(123.02703, 55.973145, 122.95569, 55.904854, 122.865845, 55.859173);
		shape35.curveTo(122.776, 55.813374, 122.68123, 55.819363, 122.582054, 55.876553);
		shape35.curveTo(122.50355, 55.921825, 122.453636, 55.986534, 122.43308, 56.069386);
		shape35.curveTo(122.41224, 56.152416, 122.425804, 56.236267, 122.47442, 56.32047);
		shape35.curveTo(122.52251, 56.403793, 122.589096, 56.45799, 122.67319, 56.483356);
		shape35.curveTo(122.757744, 56.508724, 122.85733, 56.498917, 122.97142, 56.454292);
		shape35.lineTo(123.056854, 56.69058);
		shape35.curveTo(122.89784, 56.747597, 122.752045, 56.752293, 122.619576, 56.704964);
		shape35.curveTo(122.48687, 56.65746, 122.38041, 56.565037, 122.30055, 56.42675);
		shape35.curveTo(122.24565, 56.331627, 122.215706, 56.231804, 122.21007, 56.128162);
		shape35.curveTo(122.2049, 56.024345, 122.22516, 55.93081, 122.271255, 55.84731);
		shape35.curveTo(122.31758, 55.764103, 122.380005, 55.699455, 122.459625, 55.65342);
		shape35.curveTo(122.534904, 55.610027, 122.61535, 55.59065, 122.70102, 55.595345);
		shape35.curveTo(122.78634, 55.600334, 122.87001, 55.63169, 122.95239, 55.689884);
		shape35.curveTo(122.91646, 55.572975, 122.91975, 55.464287, 122.961845, 55.363464);
		shape35.curveTo(123.00389, 55.263348, 123.08134, 55.18067, 123.19396, 55.115612);
		shape35.curveTo(123.34646, 55.027534, 123.5077, 55.008156, 123.6781, 55.058304);
		shape35.curveTo(123.84839, 55.108097, 123.982445, 55.218315, 124.08057, 55.388187);
		shape35.curveTo(124.169, 55.541504, 124.19707, 55.69517, 124.16389, 55.849194);
		shape35.curveTo(124.131355, 56.003036, 124.04633, 56.12911, 123.90899, 56.227406);
		shape35.closePath();
		shape35.moveTo(122.72628, 55.10269);
		shape35.curveTo(122.519646, 55.222008, 122.34108, 55.29676, 122.19082, 55.326588);
		shape35.curveTo(122.04032, 55.35718, 121.906555, 55.34978, 121.789825, 55.30545);
		shape35.curveTo(121.67315, 55.261116, 121.5782, 55.175564, 121.50498, 55.04873);
		shape35.curveTo(121.45072, 54.954777, 121.422424, 54.86165, 121.41949, 54.76899);
		shape35.curveTo(121.41591, 54.67668, 121.43687, 54.587017, 121.481316, 54.499584);
		shape35.curveTo(121.52594, 54.41262, 121.59159, 54.326363, 121.67867, 54.241455);
		shape35.curveTo(121.76552, 54.15608, 121.89535, 54.06377, 122.06863, 53.963715);
		shape35.curveTo(122.2735, 53.845455, 122.45089, 53.771175, 122.60109, 53.740345);
		shape35.curveTo(122.751526, 53.70987, 122.88512, 53.71674, 123.0022, 53.7609);
		shape35.curveTo(123.119286, 53.80494, 123.21441, 53.89096, 123.2884, 54.019028);
		shape35.curveTo(123.385635, 54.187435, 123.40179, 54.354313, 123.33637, 54.519962);
		shape35.curveTo(123.25763, 54.718903, 123.05399, 54.91344, 122.72628, 55.10269);
		shape35.closePath();
		shape35.moveTo(122.59904, 54.882202);
		shape35.curveTo(122.88559, 54.71667, 123.05687, 54.57293, 123.11318, 54.45085);
		shape35.curveTo(123.1692, 54.329067, 123.1692, 54.219086, 123.11242, 54.120674);
		shape35.curveTo(123.055405, 54.02185, 122.95975, 53.967007, 122.82581, 53.95485);
		shape35.curveTo(122.69193, 53.942753, 122.48172, 54.019028, 122.19593, 54.18403);
		shape35.curveTo(121.908325, 54.35003, 121.73675, 54.494183, 121.68114, 54.61579);
		shape35.curveTo(121.62512, 54.737576, 121.626114, 54.848083, 121.68384, 54.948082);
		shape35.curveTo(121.74091, 55.046906, 121.82806, 55.101223, 121.94549, 55.112083);
		shape35.curveTo(122.09411, 55.124355, 122.31231, 55.047726, 122.59903, 54.8822);
		shape35.closePath();
		g.fill(shape35);
	}

	private void paintShapeNode_0_0_0_0_6_11(Graphics2D g) {
		GeneralPath shape36 = new GeneralPath();
		shape36.moveTo(40.5154, 55.5147);
		shape36.lineTo(40.42004, 55.74547);
		shape36.curveTo(40.28422, 55.700077, 40.17254, 55.691563, 40.084812, 55.719513);
		shape36.curveTo(39.99685, 55.747814, 39.928444, 55.804245, 39.879353, 55.88927);
		shape36.curveTo(39.821163, 55.990093, 39.806892, 56.095726, 39.836662, 56.20565);
		shape36.curveTo(39.86667, 56.315163, 39.93314, 56.400127, 40.036427, 56.459785);
		shape36.curveTo(40.13478, 56.51651, 40.234722, 56.531307, 40.3356, 56.504);
		shape36.curveTo(40.436718, 56.47611, 40.516045, 56.413044, 40.5733, 56.31381);
		shape36.curveTo(40.596848, 56.273174, 40.61781, 56.21798, 40.636776, 56.148632);
		shape36.lineTo(40.811054, 56.281338);
		shape36.curveTo(40.801308, 56.294666, 40.793907, 56.30553, 40.7888, 56.314514);
		shape36.curveTo(40.736187, 56.405586, 40.7127, 56.501595, 40.718044, 56.602238);
		shape36.curveTo(40.72327, 56.70294, 40.775826, 56.78198, 40.875, 56.839344);
		shape36.curveTo(40.95345, 56.884617, 41.034424, 56.89548, 41.116455, 56.871876);
		shape36.curveTo(41.198837, 56.84839, 41.264664, 56.79472, 41.313225, 56.710575);
		shape36.curveTo(41.361374, 56.627193, 41.374996, 56.54246, 41.354916, 56.456966);
		shape36.curveTo(41.3346, 56.37106, 41.27635, 56.289677, 41.180637, 56.213223);
		shape36.lineTo(41.342525, 56.021034);
		shape36.curveTo(41.471413, 56.130253, 41.548397, 56.254208, 41.573643, 56.39261);
		shape36.curveTo(41.598835, 56.531246, 41.572056, 56.669704, 41.492256, 56.807987);
		shape36.curveTo(41.437294, 56.903114, 41.365833, 56.978977, 41.27887, 57.035583);
		shape36.curveTo(41.191555, 57.092014, 41.10042, 57.121197, 41.005, 57.123074);
		shape36.curveTo(40.909874, 57.124542, 40.82262, 57.102814, 40.742935, 57.05684);
		shape36.curveTo(40.667717, 57.013386, 40.6107, 56.953377, 40.571945, 56.876865);
		shape36.curveTo(40.5336, 56.800472, 40.51892, 56.712334, 40.52814, 56.611923);
		shape36.curveTo(40.444878, 56.70147, 40.349106, 56.752968, 40.24071, 56.76694);
		shape36.curveTo(40.13302, 56.780563, 40.022686, 56.754845, 39.910004, 56.689842);
		shape36.curveTo(39.75751, 56.601765, 39.660152, 56.471878, 39.618343, 56.29924);
		shape36.curveTo(39.5763, 56.12684, 39.60472, 55.955616, 39.70278, 55.785683);
		shape36.curveTo(39.79139, 55.632427, 39.910355, 55.53137, 40.06032, 55.483044);
		shape36.curveTo(40.20982, 55.434307, 40.361553, 55.444935, 40.515396, 55.514694);
		shape36.closePath();
		shape36.moveTo(40.13273, 57.10129);
		shape36.curveTo(40.339302, 57.220608, 40.49338, 57.33787, 40.594322, 57.45308);
		shape36.curveTo(40.696083, 57.56817, 40.756565, 57.68772, 40.776527, 57.810913);
		shape36.curveTo(40.796494, 57.934166, 40.769833, 58.05912, 40.69661, 58.186012);
		shape36.curveTo(40.642353, 58.279903, 40.575882, 58.35107, 40.49708, 58.399868);
		shape36.curveTo(40.418926, 58.449135, 40.330788, 58.475792, 40.232903, 58.481075);
		shape36.curveTo(40.135254, 58.48589, 40.027737, 58.47215, 39.910652, 58.43921);
		shape36.curveTo(39.79333, 58.406677, 39.64841, 58.340385, 39.475132, 58.240326);
		shape36.curveTo(39.27026, 58.122066, 39.1173, 58.005566, 39.01542, 57.89089);
		shape36.curveTo(38.913834, 57.775856, 38.85306, 57.656715, 38.832684, 57.533287);
		shape36.curveTo(38.81231, 57.40986, 38.83926, 57.284435, 38.91319, 57.15637);
		shape36.curveTo(39.01037, 56.98796, 39.14683, 56.890488, 39.322933, 56.8643);
		shape36.curveTo(39.534615, 56.83306, 39.80496, 56.912155, 40.13273, 57.10129);
		shape36.closePath();
		shape36.moveTo(40.005367, 57.32178);
		shape36.curveTo(39.718758, 57.15637, 39.508602, 57.079857, 39.37472, 57.09213);
		shape36.curveTo(39.241253, 57.104576, 39.14601, 57.15954, 39.08917, 57.257954);
		shape36.curveTo(39.032093, 57.35672, 39.032444, 57.466934, 39.088875, 57.589073);
		shape36.curveTo(39.145363, 57.711033, 39.31647, 57.854954, 39.602314, 58.019955);
		shape36.curveTo(39.889862, 58.186012, 40.100487, 58.262524, 40.233604, 58.2499);
		shape36.curveTo(40.367073, 58.23751, 40.462257, 58.181374, 40.52004, 58.081436);
		shape36.curveTo(40.577057, 57.98255, 40.58052, 57.879913, 40.531254, 57.772808);
		shape36.curveTo(40.467545, 57.63799, 40.29203, 57.487316, 40.005363, 57.321785);
		shape36.closePath();
		g.fill(shape36);
	}

	private void paintShapeNode_0_0_0_0_6_12(Graphics2D g) {
		GeneralPath shape37 = new GeneralPath();
		shape37.moveTo(119.063, 48.213);
		shape37.lineTo(118.693245, 48.523273);
		shape37.lineTo(119.25554, 49.19338);
		shape37.lineTo(119.081436, 49.33953);
		shape37.lineTo(117.48885, 49.47435);
		shape37.lineTo(117.35879, 49.319332);
		shape37.lineTo(118.35972, 48.47947);
		shape37.lineTo(118.18474, 48.27084);
		shape37.lineTo(118.35884, 48.124744);
		shape37.lineTo(118.53394, 48.333374);
		shape37.lineTo(118.90364, 48.02316);
		shape37.lineTo(119.063, 48.212997);
		shape37.closePath();
		shape37.moveTo(118.51914, 48.669426);
		shape37.lineTo(117.822845, 49.25368);
		shape37.lineTo(118.92471, 49.152744);
		shape37.lineTo(118.51914, 48.669426);
		shape37.closePath();
		shape37.moveTo(117.804115, 48.259857);
		shape37.curveTo(117.62132, 48.41323, 117.458496, 48.517986, 117.31569, 48.57336);
		shape37.curveTo(117.17283, 48.629555, 117.039764, 48.645527, 116.91716, 48.622215);
		shape37.curveTo(116.79462, 48.598846, 116.68622, 48.531025, 116.59203, 48.41887);
		shape37.curveTo(116.52228, 48.335724, 116.47823, 48.24888, 116.45927, 48.158215);
		shape37.curveTo(116.43971, 48.067844, 116.444824, 47.97595, 116.47336, 47.88212);
		shape37.curveTo(116.50219, 47.788696, 116.551926, 47.692337, 116.62292, 47.59363);
		shape37.curveTo(116.69356, 47.494514, 116.80547, 47.38101, 116.95879, 47.252415);
		shape37.curveTo(117.14, 47.100273, 117.30171, 46.996456, 117.44434, 46.93997);
		shape37.curveTo(117.587204, 46.883774, 117.71997, 46.86739, 117.84281, 46.890587);
		shape37.curveTo(117.96583, 46.913605, 118.07452, 46.981777, 118.16964, 47.095104);
		shape37.curveTo(118.29453, 47.244015, 118.339455, 47.405552, 118.30381, 47.580067);
		shape37.curveTo(118.26077, 47.789577, 118.09401, 48.016644, 117.80411, 48.25986);
		shape37.closePath();
		shape37.moveTo(117.640526, 48.06485);
		shape37.curveTo(117.89407, 47.85205, 118.03782, 47.680767, 118.07205, 47.550762);
		shape37.curveTo(118.10605, 47.421112, 118.08691, 47.31283, 118.01392, 47.22575);
		shape37.curveTo(117.94058, 47.138317, 117.83683, 47.100914, 117.70289, 47.112247);
		shape37.curveTo(117.56895, 47.12352, 117.375175, 47.235146, 117.12233, 47.4473);
		shape37.curveTo(116.86796, 47.660686, 116.72404, 47.832497, 116.69039, 47.961857);
		shape37.curveTo(116.656334, 48.091507, 116.67654, 48.200256, 116.750755, 48.28869);
		shape37.curveTo(116.82416, 48.376003, 116.919395, 48.414467, 117.036896, 48.404778);
		shape37.curveTo(117.185394, 48.39098, 117.38698, 48.27759, 117.64053, 48.06485);
		shape37.closePath();
		g.fill(shape37);
	}

	private void paintShapeNode_0_0_0_0_6_13(Graphics2D g) {
		GeneralPath shape38 = new GeneralPath();
		shape38.moveTo(44.5991, 49.2788);
		shape38.lineTo(44.968853, 49.589073);
		shape38.lineTo(45.53115, 48.91891);
		shape38.lineTo(45.70531, 49.065006);
		shape38.lineTo(46.114643, 50.609966);
		shape38.lineTo(45.98458, 50.764984);
		shape38.lineTo(44.98359, 49.92512);
		shape38.lineTo(44.808548, 50.133633);
		shape38.lineTo(44.634502, 49.987537);
		shape38.lineTo(44.809544, 49.778908);
		shape38.lineTo(44.43979, 49.468636);
		shape38.lineTo(44.599094, 49.278797);
		shape38.closePath();
		shape38.moveTo(45.143017, 49.735107);
		shape38.lineTo(45.83931, 50.319363);
		shape38.lineTo(45.54859, 49.251793);
		shape38.lineTo(45.143017, 49.73511);
		shape38.closePath();
		shape38.moveTo(44.863808, 50.510376);
		shape38.curveTo(45.04666, 50.66375, 45.178074, 50.80591, 45.257404, 50.93697);
		shape38.curveTo(45.337555, 51.067856, 45.376427, 51.196156, 45.37467, 51.320934);
		shape38.curveTo(45.372967, 51.445713, 45.32505, 51.56421, 45.230865, 51.676422);
		shape38.curveTo(45.161106, 51.759567, 45.083244, 51.817993, 44.99728, 51.852406);
		shape38.curveTo(44.911728, 51.8874, 44.8203, 51.898323, 44.722885, 51.88652);
		shape38.curveTo(44.62588, 51.874306, 44.52242, 51.84207, 44.412846, 51.78934);
		shape38.curveTo(44.302925, 51.736965, 44.171745, 51.646477, 44.018486, 51.517822);
		shape38.curveTo(43.83722, 51.3658, 43.70692, 51.22458, 43.626476, 51.09387);
		shape38.curveTo(43.546326, 50.962982, 43.50716, 50.83509, 43.508625, 50.710022);
		shape38.curveTo(43.509975, 50.584892, 43.558243, 50.46604, 43.65331, 50.352657);
		shape38.curveTo(43.778263, 50.203804, 43.929523, 50.13152, 44.10756, 50.136333);
		shape38.curveTo(44.321415, 50.142323, 44.574024, 50.267162, 44.863804, 50.510376);
		shape38.closePath();
		shape38.moveTo(44.700214, 50.705383);
		shape38.curveTo(44.446606, 50.49264, 44.25295, 50.380844, 44.11901, 50.369686);
		shape38.curveTo(43.985424, 50.358646, 43.88208, 50.396343, 43.809032, 50.483307);
		shape38.curveTo(43.735634, 50.57074, 43.716785, 50.67943, 43.75125, 50.809376);
		shape38.curveTo(43.78566, 50.939323, 43.92917, 51.110783, 44.182014, 51.322933);
		shape38.curveTo(44.43633, 51.536377, 44.630512, 51.648296, 44.76375, 51.658924);
		shape38.curveTo(44.897392, 51.669964, 45.000916, 51.63121, 45.075134, 51.54272);
		shape38.curveTo(45.148415, 51.455288, 45.16973, 51.35482, 45.139786, 51.240788);
		shape38.curveTo(45.100445, 51.096924, 44.953762, 50.918125, 44.700214, 50.705387);
		shape38.closePath();
		g.fill(shape38);
	}

	private void paintShapeNode_0_0_0_0_6_14(Graphics2D g) {
		GeneralPath shape39 = new GeneralPath();
		shape39.moveTo(113.142, 42.6069);
		shape39.lineTo(112.92855, 42.456577);
		shape39.curveTo(112.995255, 42.347065, 113.02197, 42.245777, 113.00952, 42.151825);
		shape39.curveTo(112.99666, 42.057697, 112.952385, 41.978836, 112.876526, 41.915127);
		shape39.curveTo(112.78522, 41.838497, 112.67888, 41.808258, 112.55833, 41.824112);
		shape39.curveTo(112.43701, 41.839966, 112.328735, 41.90473, 112.233376, 42.018475);
		shape39.curveTo(112.14254, 42.126694, 112.10114, 42.237263, 112.109535, 42.350883);
		shape39.curveTo(112.1177, 42.464447, 112.1709, 42.562508, 112.26955, 42.645126);
		shape39.curveTo(112.33061, 42.696445, 112.39738, 42.728626, 112.469955, 42.742367);
		shape39.curveTo(112.54247, 42.75593, 112.611115, 42.75223, 112.67629, 42.731503);
		shape39.lineTo(112.83513, 42.90396);
		shape39.lineTo(112.01958, 43.57189);
		shape39.lineTo(111.25135, 42.927273);
		shape39.lineTo(111.403496, 42.74595);
		shape39.lineTo(112.01999, 43.263268);
		shape39.lineTo(112.45169, 42.918175);
		shape39.curveTo(112.30454, 42.90508, 112.17988, 42.855698, 112.07818, 42.770203);
		shape39.curveTo(111.943184, 42.65705, 111.86884, 42.515007, 111.85446, 42.34361);
		shape39.curveTo(111.840195, 42.172207, 111.89462, 42.013195, 112.0177, 41.866455);
		shape39.curveTo(112.1352, 41.726467, 112.277534, 41.639915, 112.44424, 41.606388);
		shape39.curveTo(112.648285, 41.564873, 112.83601, 41.615902, 113.00724, 41.75953);
		shape39.curveTo(113.14787, 41.877495, 113.22961, 42.0129, 113.25251, 42.166218);
		shape39.curveTo(113.27518, 42.319653, 113.2383, 42.46645, 113.142, 42.606907);
		shape39.closePath();
		shape39.moveTo(111.642075, 41.95688);
		shape39.curveTo(111.4887, 42.13967, 111.3466, 42.271027, 111.21548, 42.350475);
		shape39.curveTo(111.08465, 42.43051, 110.95635, 42.469383, 110.83151, 42.46774);
		shape39.curveTo(110.70673, 42.465977, 110.58824, 42.418064, 110.47603, 42.323936);
		shape39.curveTo(110.392944, 42.25412, 110.33446, 42.176258, 110.30005, 42.09035);
		shape39.curveTo(110.265114, 42.00468, 110.25413, 41.913254, 110.26588, 41.8159);
		shape39.curveTo(110.27815, 41.718895, 110.31045, 41.61549, 110.36311, 41.50586);
		shape39.curveTo(110.415436, 41.395996, 110.506096, 41.264698, 110.63463, 41.1115);
		shape39.curveTo(110.78665, 40.93029, 110.92793, 40.799816, 111.05858, 40.71949);
		shape39.curveTo(111.18946, 40.639397, 111.3173, 40.600174, 111.44248, 40.60158);
		shape39.curveTo(111.56755, 40.60299, 111.68646, 40.6512, 111.79979, 40.746326);
		shape39.curveTo(111.948586, 40.87128, 112.02087, 41.02266, 112.01617, 41.200638);
		shape39.curveTo(112.01018, 41.414436, 111.885284, 41.666985, 111.64207, 41.956882);
		shape39.closePath();
		shape39.moveTo(111.44707, 41.793285);
		shape39.curveTo(111.659805, 41.539734, 111.77161, 41.345963, 111.78282, 41.212082);
		shape39.curveTo(111.7938, 41.078438, 111.756165, 40.975033, 111.66908, 40.902042);
		shape39.curveTo(111.581764, 40.828644, 111.47302, 40.809795, 111.34307, 40.84426);
		shape39.curveTo(111.21313, 40.87867, 111.0416, 41.02224, 110.82951, 41.275085);
		shape39.curveTo(110.616066, 41.5294, 110.50415, 41.723583, 110.49346, 41.85676);
		shape39.curveTo(110.48248, 41.990402, 110.52123, 42.093925, 110.609665, 42.168087);
		shape39.curveTo(110.6971, 42.241486, 110.79762, 42.26274, 110.91166, 42.232796);
		shape39.curveTo(111.055466, 42.193455, 111.23426, 42.046833, 111.44706, 41.79328);
		shape39.closePath();
		g.fill(shape39);
	}

	private void paintShapeNode_0_0_0_0_6_15(Graphics2D g) {
		GeneralPath shape40 = new GeneralPath();
		shape40.moveTo(51.232, 43.4848);
		shape40.lineTo(51.046917, 43.668884);
		shape40.curveTo(50.95062, 43.58421, 50.85549, 43.54023, 50.760838, 43.536236);
		shape40.curveTo(50.66589, 43.532536, 50.58057, 43.562424, 50.504646, 43.626076);
		shape40.curveTo(50.41328, 43.702705, 50.36507, 43.802174, 50.359726, 43.923664);
		shape40.curveTo(50.354324, 44.04586, 50.399303, 44.16377, 50.49472, 44.27739);
		shape40.curveTo(50.5855, 44.385666, 50.68726, 44.44568, 50.80059, 44.45713);
		shape40.curveTo(50.91386, 44.468815, 51.019672, 44.433407, 51.118145, 44.350613);
		shape40.curveTo(51.17933, 44.299408, 51.22255, 44.239223, 51.248737, 44.17011);
		shape40.curveTo(51.274693, 44.101112, 51.28297, 44.03282, 51.27387, 43.965004);
		shape40.lineTo(51.471283, 43.83858);
		shape40.lineTo(51.987427, 44.757713);
		shape40.lineTo(51.219204, 45.402332);
		shape40.lineTo(51.067062, 45.221066);
		shape40.lineTo(51.683556, 44.703747);
		shape40.lineTo(51.418674, 44.21861);
		shape40.curveTo(51.380272, 44.361237, 51.309925, 44.47545, 51.208107, 44.560825);
		shape40.curveTo(51.07323, 44.674038, 50.920383, 44.722656, 50.7491, 44.707035);
		shape40.curveTo(50.577873, 44.6913, 50.430664, 44.61015, 50.30753, 44.46341);
		shape40.curveTo(50.19015, 44.32342, 50.129612, 44.16823, 50.125557, 43.998234);
		shape40.curveTo(50.120094, 43.790073, 50.20289, 43.614033, 50.374115, 43.47035);
		shape40.curveTo(50.514748, 43.35232, 50.662193, 43.295307, 50.81721, 43.299416);
		shape40.curveTo(50.97223, 43.30376, 51.110455, 43.365532, 51.232002, 43.48479);
		shape40.closePath();
		shape40.moveTo(50.331364, 44.84908);
		shape40.curveTo(50.484737, 45.031815, 50.589436, 45.194584, 50.644924, 45.337505);
		shape40.curveTo(50.70106, 45.48025, 50.71703, 45.613308, 50.69378, 45.736034);
		shape40.curveTo(50.67035, 45.85858, 50.60259, 45.966976, 50.490376, 46.061104);
		shape40.curveTo(50.40723, 46.130802, 50.320385, 46.1749, 50.22978, 46.193867);
		shape40.curveTo(50.13935, 46.21342, 50.047398, 46.20837, 49.95362, 46.179832);
		shape40.curveTo(49.8602, 46.150883, 49.763958, 46.101208, 49.665134, 46.030273);
		shape40.curveTo(49.566017, 45.959633, 49.452454, 45.8476, 49.323917, 45.6944);
		shape40.curveTo(49.171894, 45.51319, 49.0679, 45.35142, 49.01147, 45.208794);
		shape40.curveTo(48.955334, 45.06599, 48.938892, 44.93328, 48.96203, 44.810265);
		shape40.curveTo(48.985104, 44.687305, 49.05328, 44.578617, 49.166607, 44.48355);
		shape40.curveTo(49.315517, 44.358654, 49.477116, 44.313793, 49.65157, 44.34932);
		shape40.curveTo(49.861137, 44.392303, 50.088146, 44.55918, 50.331364, 44.84908);
		shape40.closePath();
		shape40.moveTo(50.136417, 45.012672);
		shape40.curveTo(49.923676, 44.75912, 49.752277, 44.61538, 49.62239, 44.581146);
		shape40.curveTo(49.492622, 44.54709, 49.384285, 44.56623, 49.29726, 44.63928);
		shape40.curveTo(49.209885, 44.71256, 49.172424, 44.81632, 49.183758, 44.950314);
		shape40.curveTo(49.19509, 45.08425, 49.306656, 45.278084, 49.518867, 45.530872);
		shape40.curveTo(49.732254, 45.785244, 49.904064, 45.929165, 50.033367, 45.96281);
		shape40.curveTo(50.16308, 45.99681, 50.27171, 45.97667, 50.36014, 45.902447);
		shape40.curveTo(50.447575, 45.82905, 50.485977, 45.733746, 50.476288, 45.616306);
		shape40.curveTo(50.462547, 45.467804, 50.34916, 45.26628, 50.13642, 45.012672);
		shape40.closePath();
		g.fill(shape40);
	}

	private void paintShapeNode_0_0_0_0_6_16(Graphics2D g) {
		GeneralPath shape41 = new GeneralPath();
		shape41.moveTo(104.575, 37.6847);
		shape41.lineTo(104.7976, 37.791042);
		shape41.curveTo(104.768005, 37.886284, 104.75955, 37.96315, 104.772644, 38.021046);
		shape41.curveTo(104.79525, 38.116173, 104.84927, 38.187927, 104.93424, 38.237015);
		shape41.curveTo(105.00253, 38.276474, 105.07341, 38.29174, 105.147156, 38.283344);
		shape41.curveTo(105.2431, 38.273186, 105.337814, 38.231964, 105.43136, 38.16009);
		shape41.curveTo(105.52437, 38.0881, 105.62243, 37.965202, 105.72501, 37.791336);
		shape41.curveTo(105.62813, 37.839954, 105.531296, 37.86221, 105.43436, 37.85722);
		shape41.curveTo(105.338234, 37.851994, 105.24869, 37.825863, 105.16683, 37.778652);
		shape41.curveTo(105.02379, 37.695976, 104.93243, 37.57343, 104.89279, 37.41001);
		shape41.curveTo(104.85316, 37.246773, 104.8811, 37.08177, 104.97735, 36.915184);
		shape41.curveTo(105.040764, 36.805378, 105.1235, 36.717537, 105.225204, 36.65048);
		shape41.curveTo(105.32655, 36.583775, 105.433365, 36.548954, 105.544876, 36.546547);
		shape41.curveTo(105.65615, 36.54426, 105.76408, 36.573383, 105.869, 36.63398);
		shape41.curveTo(106.04786, 36.737328, 106.15567, 36.886707, 106.19219, 37.08359);
		shape41.curveTo(106.22942, 37.280125, 106.160484, 37.52927, 105.98627, 37.83097);
		shape41.curveTo(105.791435, 38.16843, 105.58756, 38.377766, 105.37424, 38.45903);
		shape41.curveTo(105.18868, 38.529728, 105.00354, 38.51223, 104.81893, 38.405712);
		shape41.curveTo(104.68188, 38.326557, 104.59163, 38.222977, 104.54841, 38.09497);
		shape41.curveTo(104.50501, 37.967434, 104.514, 37.83056, 104.575005, 37.6847);
		shape41.closePath();
		shape41.moveTo(105.884964, 37.43755);
		shape41.curveTo(105.92765, 37.363388, 105.95261, 37.283527, 105.960304, 37.197975);
		shape41.curveTo(105.967995, 37.112305, 105.95361, 37.03585, 105.91791, 36.96809);
		shape41.curveTo(105.88185, 36.900036, 105.832825, 36.847775, 105.770935, 36.81219);
		shape41.curveTo(105.68033, 36.759754, 105.581505, 36.751595, 105.47417, 36.787117);
		shape41.curveTo(105.36689, 36.82282, 105.27705, 36.90332, 105.204765, 37.028454);
		shape41.curveTo(105.13501, 37.14918, 105.11222, 37.262978, 105.1363, 37.369553);
		shape41.curveTo(105.16032, 37.476185, 105.22132, 37.557514, 105.31898, 37.613823);
		shape41.curveTo(105.4161, 37.66996, 105.518326, 37.68282, 105.6259, 37.65217);
		shape41.curveTo(105.733536, 37.621635, 105.81997, 37.550056, 105.88497, 37.43755);
		shape41.closePath();
		shape41.moveTo(104.594604, 37.086056);
		shape41.curveTo(104.47534, 37.29269, 104.35814, 37.446827, 104.242935, 37.547768);
		shape41.curveTo(104.127846, 37.649353, 104.008354, 37.70995, 103.8851, 37.729916);
		shape41.curveTo(103.76191, 37.74988, 103.63683, 37.72322, 103.51006, 37.649998);
		shape41.curveTo(103.416115, 37.5958, 103.34495, 37.529213, 103.29615, 37.450527);
		shape41.curveTo(103.24694, 37.37237, 103.22017, 37.284233, 103.215004, 37.18629);
		shape41.curveTo(103.21007, 37.0887, 103.22387, 36.981068, 103.25681, 36.864098);
		shape41.curveTo(103.28928, 36.74666, 103.35564, 36.601738, 103.455635, 36.42852);
		shape41.curveTo(103.57395, 36.22359, 103.69045, 36.070686, 103.80513, 35.968864);
		shape41.curveTo(103.92016, 35.86722, 104.0393, 35.806446, 104.16273, 35.78607);
		shape41.curveTo(104.286156, 35.765697, 104.41164, 35.79259, 104.5397, 35.866516);
		shape41.curveTo(104.70805, 35.963757, 104.805466, 36.10022, 104.83171, 36.276436);
		shape41.curveTo(104.86295, 36.48794, 104.78391, 36.758347, 104.594604, 37.086056);
		shape41.closePath();
		shape41.moveTo(104.37423, 36.95881);
		shape41.curveTo(104.5397, 36.67208, 104.6161, 36.461987, 104.60394, 36.328106);
		shape41.curveTo(104.59155, 36.194637, 104.53653, 36.099453, 104.43812, 36.042614);
		shape41.curveTo(104.339294, 35.98554, 104.22908, 35.985832, 104.107, 36.04232);
		shape41.curveTo(103.98504, 36.09875, 103.841064, 36.269917, 103.67606, 36.555702);
		shape41.curveTo(103.51006, 36.843307, 103.433495, 37.053936, 103.446175, 37.186993);
		shape41.curveTo(103.458565, 37.32046, 103.51464, 37.415703, 103.6147, 37.473366);
		shape41.curveTo(103.713524, 37.5305, 103.81611, 37.533962, 103.92321, 37.48464);
		shape41.curveTo(104.05809, 37.420986, 104.2087, 37.245476, 104.37423, 36.95881);
		shape41.closePath();
		g.fill(shape41);
	}

	private void paintShapeNode_0_0_0_0_6_17(Graphics2D g) {
		GeneralPath shape42 = new GeneralPath();
		shape42.moveTo(57.4327, 40.5318);
		shape42.lineTo(57.636047, 40.392223);
		shape42.curveTo(57.70375, 40.46539, 57.766052, 40.51119, 57.822773, 40.528805);
		shape42.curveTo(57.91649, 40.556755, 58.005627, 40.54589, 58.09065, 40.49686);
		shape42.curveTo(58.15894, 40.45746, 58.207558, 40.40367, 58.237213, 40.335617);
		shape42.curveTo(58.27638, 40.24742, 58.288006, 40.14478, 58.272564, 40.02787);
		shape42.curveTo(58.25671, 39.911312, 58.19934, 39.764984, 58.100048, 39.58918);
		shape42.curveTo(58.093708, 39.6974, 58.06452, 39.792408, 58.011734, 39.87385);
		shape42.curveTo(57.95918, 39.95447, 57.89177, 40.018948, 57.809975, 40.066216);
		shape42.curveTo(57.666874, 40.148773, 57.515026, 40.166626, 57.35367, 40.11924);
		shape42.curveTo(57.192543, 40.07191, 57.063595, 39.965218, 56.967472, 39.798573);
		shape42.curveTo(56.904057, 39.688766, 56.86935, 39.573208, 56.86213, 39.45154);
		shape42.curveTo(56.855022, 39.330463, 56.878277, 39.22054, 56.931946, 39.122772);
		shape42.curveTo(56.985615, 39.025238, 57.06477, 38.94638, 57.1697, 38.88584);
		shape42.curveTo(57.348618, 38.782555, 57.53194, 38.76394, 57.720722, 38.830704);
		shape42.curveTo(57.909565, 38.896706, 58.090828, 39.081024, 58.26499, 39.382725);
		shape42.curveTo(58.459824, 39.72024, 58.53921, 40.00145, 58.502922, 40.226814);
		shape42.curveTo(58.471333, 40.42282, 58.36364, 40.57449, 58.179085, 40.681065);
		shape42.curveTo(58.042034, 40.76016, 57.907154, 40.786526, 57.774742, 40.759983);
		shape42.curveTo(57.642506, 40.733795, 57.528473, 40.657578, 57.4327, 40.5318);
		shape42.closePath();
		shape42.moveTo(57.873623, 39.2738);
		shape42.curveTo(57.830757, 39.199696, 57.774094, 39.13816, 57.703808, 39.08872);
		shape42.curveTo(57.633522, 39.03922, 57.560062, 39.01344, 57.48355, 39.010506);
		shape42.curveTo(57.40657, 39.007687, 57.33681, 39.02401, 57.27504, 39.05983);
		shape42.curveTo(57.18432, 39.11209, 57.12783, 39.193592, 57.10493, 39.30428);
		shape42.curveTo(57.082207, 39.41508, 57.106987, 39.533108, 57.179268, 39.6583);
		shape42.curveTo(57.24891, 39.779083, 57.33605, 39.855713, 57.44039, 39.888184);
		shape42.curveTo(57.544796, 39.920654, 57.645733, 39.9085, 57.743267, 39.85207);
		shape42.curveTo(57.840446, 39.79605, 57.90269, 39.713963, 57.929993, 39.60545);
		shape42.curveTo(57.9573, 39.496937, 57.938564, 39.38631, 57.873623, 39.273804);
		shape42.closePath();
		shape42.moveTo(56.924072, 40.21548);
		shape42.curveTo(57.04339, 40.422115, 57.118256, 40.600677, 57.148087, 40.75088);
		shape42.curveTo(57.178505, 40.90138, 57.17128, 41.03514, 57.12689, 41.151875);
		shape42.curveTo(57.082615, 41.26855, 56.997, 41.363556, 56.870167, 41.43672);
		shape42.curveTo(56.776215, 41.49092, 56.68297, 41.51928, 56.590427, 41.522217);
		shape42.curveTo(56.49818, 41.52574, 56.408455, 41.504894, 56.321022, 41.460384);
		shape42.curveTo(56.23406, 41.415817, 56.14774, 41.35011, 56.062893, 41.26309);
		shape42.curveTo(55.977455, 41.176243, 55.88515, 41.046295, 55.785152, 40.873074);
		shape42.curveTo(55.666832, 40.6682, 55.592613, 40.49081, 55.561783, 40.340607);
		shape42.curveTo(55.531307, 40.190166, 55.538235, 40.056583, 55.582275, 39.939552);
		shape42.curveTo(55.626373, 39.822407, 55.7124, 39.727165, 55.840466, 39.653236);
		shape42.curveTo(56.008812, 39.556114, 56.175694, 39.539967, 56.341457, 39.60532);
		shape42.curveTo(56.540222, 39.684002, 56.734875, 39.8877, 56.92407, 40.215473);
		shape42.closePath();
		shape42.moveTo(56.70364, 40.342728);
		shape42.curveTo(56.53811, 40.05606, 56.394306, 39.884834, 56.27229, 39.828465);
		shape42.curveTo(56.150566, 39.772446, 56.040585, 39.772446, 55.94217, 39.82929);
		shape42.curveTo(55.843285, 39.886364, 55.78844, 39.98196, 55.776344, 40.1159);
		shape42.curveTo(55.76419, 40.24978, 55.840466, 40.45999, 56.005466, 40.74578);
		shape42.curveTo(56.171524, 41.033386, 56.31568, 41.204967, 56.43723, 41.260513);
		shape42.curveTo(56.559013, 41.316532, 56.66952, 41.315594, 56.76952, 41.257812);
		shape42.curveTo(56.868404, 41.200794, 56.922718, 41.113655, 56.933582, 40.99622);
		shape42.curveTo(56.945854, 40.847603, 56.869167, 40.629402, 56.703636, 40.34273);
		shape42.closePath();
		g.fill(shape42);
	}

	private void paintShapeNode_0_0_0_0_6_18(Graphics2D g) {
		GeneralPath shape43 = new GeneralPath();
		shape43.moveTo(98.0128, 34.6208);
		shape43.lineTo(97.93136, 34.84452);
		shape43.lineTo(96.70483, 34.39808);
		shape43.lineTo(96.770706, 34.217106);
		shape43.curveTo(96.93782, 34.132668, 97.11961, 34.005424, 97.31568, 33.8359);
		shape43.curveTo(97.51134, 33.666317, 97.682915, 33.48041, 97.82906, 33.2793);
		shape43.curveTo(97.933586, 33.137436, 98.0263, 32.974842, 98.107864, 32.790638);
		shape43.lineTo(98.34691, 32.87772);
		shape43.curveTo(98.29001, 33.025867, 98.19541, 33.19533, 98.062416, 33.38623);
		shape43.curveTo(97.92965, 33.576656, 97.77522, 33.751347, 97.599236, 33.909184);
		shape43.curveTo(97.42308, 34.066902, 97.25156, 34.19186, 97.084564, 34.282932);
		shape43.lineTo(98.012794, 34.620804);
		shape43.closePath();
		shape43.moveTo(96.81516, 33.37842);
		shape43.curveTo(96.733604, 33.60261, 96.644875, 33.77472, 96.54893, 33.894096);
		shape43.curveTo(96.45327, 34.01418, 96.346115, 34.094563, 96.2282, 34.135666);
		shape43.curveTo(96.1103, 34.176712, 95.98252, 34.172188, 95.845, 34.1221);
		shape43.curveTo(95.743065, 34.08499, 95.661385, 34.03185, 95.59973, 33.962795);
		shape43.curveTo(95.537605, 33.89433, 95.49597, 33.812122, 95.47384, 33.716644);
		shape43.curveTo(95.45206, 33.621403, 95.44701, 33.513065, 95.45904, 33.392105);
		shape43.curveTo(95.470665, 33.27079, 95.51089, 33.116592, 95.57935, 32.928635);
		shape43.curveTo(95.66021, 32.706326, 95.748344, 32.535393, 95.84359, 32.415253);
		shape43.curveTo(95.939186, 32.29517, 96.046, 32.21467, 96.164024, 32.173153);
		shape43.curveTo(96.28211, 32.131638, 96.41029, 32.136337, 96.54934, 32.18695);
		shape43.curveTo(96.731895, 32.25342, 96.85157, 32.37092, 96.908, 32.539913);
		shape43.curveTo(96.975586, 32.74273, 96.94464, 33.02288, 96.81516, 33.378426);
		shape43.closePath();
		shape43.moveTo(96.576, 33.2914);
		shape43.curveTo(96.68915, 32.980305, 96.727905, 32.75999, 96.69267, 32.63034);
		shape43.curveTo(96.657326, 32.5011, 96.58656, 32.416836, 96.47976, 32.377907);
		shape43.curveTo(96.372536, 32.338974, 96.26402, 32.358353, 96.15363, 32.435215);
		shape43.curveTo(96.0433, 32.511963, 95.93132, 32.7055, 95.818405, 33.015656);
		shape43.curveTo(95.70484, 33.32763, 95.666084, 33.548298, 95.701614, 33.677185);
		shape43.curveTo(95.73696, 33.806484, 95.80884, 33.89051, 95.91735, 33.93003);
		shape43.curveTo(96.024574, 33.969078, 96.126274, 33.954693, 96.2231, 33.88752);
		shape43.curveTo(96.34488, 33.80138, 96.46279, 33.602436, 96.576004, 33.2914);
		shape43.closePath();
		g.fill(shape43);
	}

	private void paintShapeNode_0_0_0_0_6_19(Graphics2D g) {
		GeneralPath shape44 = new GeneralPath();
		shape44.moveTo(65.7876, 37.0194);
		shape44.lineTo(65.86898, 37.243122);
		shape44.lineTo(64.64245, 37.689564);
		shape44.lineTo(64.57657, 37.50859);
		shape44.curveTo(64.65032, 37.336483, 64.70775, 37.1221, 64.74897, 36.86626);
		shape44.curveTo(64.78984, 36.610596, 64.80182, 36.35787, 64.78449, 36.10984);
		shape44.curveTo(64.77339, 35.934036, 64.73987, 35.749832, 64.68397, 35.55635);
		shape44.lineTo(64.92307, 35.46933);
		shape44.curveTo(64.974686, 35.619415, 65.011154, 35.810078, 65.032, 36.041725);
		shape44.curveTo(65.05273, 36.27296, 65.04668, 36.506077, 65.01333, 36.740074);
		shape44.curveTo(64.979744, 36.97419, 64.92871, 37.180115, 64.85931, 37.357212);
		shape44.lineTo(65.7876, 37.0194);
		shape44.closePath();
		shape44.moveTo(64.071526, 36.837547);
		shape44.curveTo(64.153145, 37.06168, 64.19578, 37.25058, 64.199066, 37.40366);
		shape44.curveTo(64.203, 37.55715, 64.17252, 37.687626, 64.108635, 37.794907);
		shape44.curveTo(64.04469, 37.902126, 63.943928, 37.980812, 63.806347, 38.03084);
		shape44.curveTo(63.70441, 38.067894, 63.6077, 38.079697, 63.516037, 38.066425);
		shape44.curveTo(63.424435, 38.053917, 63.339764, 38.01769, 63.26137, 37.958794);
		shape44.curveTo(63.18351, 37.89984, 63.109993, 37.8201, 63.041466, 37.71969);
		shape44.curveTo(62.972355, 37.61934, 62.904064, 37.4753, 62.835712, 37.28734);
		shape44.curveTo(62.754738, 37.06503, 62.712402, 36.87748, 62.708115, 36.72422);
		shape44.curveTo(62.70418, 36.57073, 62.734303, 36.44043, 62.797955, 36.332798);
		shape44.curveTo(62.861725, 36.225048, 62.962955, 36.146248, 63.102, 36.09569);
		shape44.curveTo(63.284557, 36.02922, 63.451794, 36.042316, 63.60364, 36.1355);
		shape44.curveTo(63.78579, 36.24748, 63.9421, 36.481945, 64.07152, 36.837547);
		shape44.closePath();
		shape44.moveTo(63.832363, 36.92457);
		shape44.curveTo(63.719093, 36.613533, 63.607174, 36.419876, 63.49684, 36.34319);
		shape44.curveTo(63.386684, 36.266853, 63.27829, 36.24783, 63.17148, 36.286644);
		shape44.curveTo(63.064316, 36.325752, 62.993618, 36.410366, 62.958447, 36.540195);
		shape44.curveTo(62.923332, 36.669907, 62.96191, 36.890163, 63.074768, 37.200317);
		shape44.curveTo(63.18827, 37.51229, 63.300484, 37.706303, 63.410526, 37.782166);
		shape44.curveTo(63.520683, 37.85844, 63.629784, 37.876644, 63.738358, 37.837185);
		shape44.curveTo(63.84552, 37.798195, 63.914223, 37.7218, 63.945225, 37.608063);
		shape44.curveTo(63.98316, 37.46379, 63.945576, 37.235607, 63.832367, 36.924572);
		shape44.closePath();
		g.fill(shape44);
	}

	private void paintShapeNode_0_0_0_0_6_20(Graphics2D g) {
		GeneralPath shape45 = new GeneralPath();
		shape45.moveTo(89.5815, 31.5512);
		shape45.curveTo(89.67633, 31.606045, 89.74192, 31.671928, 89.77873, 31.749496);
		shape45.curveTo(89.815254, 31.826418, 89.82512, 31.913206, 89.808266, 32.0088);
		shape45.curveTo(89.78284, 32.153484, 89.70962, 32.26558, 89.58824, 32.34573);
		shape45.curveTo(89.466866, 32.42588, 89.320305, 32.451015, 89.14785, 32.420654);
		shape45.curveTo(88.97445, 32.39006, 88.84368, 32.31496, 88.75555, 32.195583);
		shape45.curveTo(88.668, 32.076088, 88.63676, 31.944323, 88.6623, 31.79958);
		shape45.curveTo(88.67845, 31.707624, 88.716736, 31.63176, 88.7768, 31.571867);
		shape45.curveTo(88.8374, 31.512384, 88.91972, 31.472572, 89.02477, 31.453018);
		shape45.curveTo(88.91016, 31.391716, 88.82853, 31.311563, 88.78038, 31.211506);
		shape45.curveTo(88.73258, 31.112095, 88.71937, 31.000586, 88.74098, 30.878038);
		shape45.curveTo(88.77099, 30.707811, 88.85613, 30.575928, 88.99624, 30.481625);
		shape45.curveTo(89.13687, 30.387203, 89.305275, 30.357258, 89.50123, 30.391901);
		shape45.curveTo(89.69735, 30.426428, 89.84491, 30.512041, 89.94462, 30.649208);
		shape45.curveTo(90.04444, 30.786493, 90.07897, 30.941807, 90.04838, 31.115204);
		shape45.curveTo(90.02571, 31.24415, 89.97369, 31.346674, 89.892654, 31.422422);
		shape45.curveTo(89.81215, 31.497936, 89.708275, 31.541035, 89.581505, 31.551193);
		shape45.closePath();
		shape45.moveTo(89.5576, 31.973158);
		shape45.curveTo(89.57422, 31.879324, 89.55725, 31.797176, 89.50763, 31.726889);
		shape45.curveTo(89.4576, 31.65666, 89.3842, 31.61274, 89.287605, 31.59565);
		shape45.curveTo(89.19366, 31.57909, 89.11133, 31.595062, 89.04075, 31.643682);
		shape45.curveTo(88.97028, 31.692303, 88.92783, 31.759653, 88.91274, 31.845207);
		shape45.curveTo(88.89689, 31.934872, 88.914154, 32.015083, 88.96547, 32.08707);
		shape45.curveTo(89.016556, 32.15906, 89.08807, 32.20316, 89.18038, 32.21954);
		shape45.curveTo(89.27339, 32.235924, 89.355896, 32.21942, 89.427826, 32.171097);
		shape45.curveTo(89.49993, 32.1223, 89.54303, 32.05595, 89.557594, 31.973154);
		shape45.closePath();
		shape45.moveTo(89.79812, 31.069702);
		shape45.curveTo(89.81027, 31.000296, 89.80546, 30.92995, 89.78403, 30.85937);
		shape45.curveTo(89.76277, 30.788378, 89.72261, 30.729307, 89.66371, 30.682154);
		shape45.curveTo(89.60476, 30.635178, 89.53817, 30.604704, 89.46319, 30.59155);
		shape45.curveTo(89.34669, 30.57094, 89.24387, 30.591433, 89.154854, 30.6535);
		shape45.curveTo(89.06577, 30.715038, 89.01111, 30.803528, 88.990616, 30.919262);
		shape45.curveTo(88.96995, 31.036642, 88.991615, 31.140575, 89.05632, 31.23112);
		shape45.curveTo(89.12091, 31.321606, 89.21216, 31.37733, 89.32966, 31.397999);
		shape45.curveTo(89.44422, 31.418257, 89.54586, 31.397118, 89.63447, 31.334524);
		shape45.curveTo(89.72331, 31.271519, 89.77804, 31.183382, 89.79812, 31.0697);
		shape45.closePath();
		shape45.moveTo(88.43125, 31.24768);
		shape45.curveTo(88.38979, 31.482676, 88.332306, 31.667524, 88.25861, 31.801756);
		shape45.curveTo(88.18527, 31.936634, 88.09361, 32.03446, 87.98463, 32.09547);
		shape45.curveTo(87.8757, 32.156246, 87.74911, 32.174038, 87.60489, 32.148552);
		shape45.curveTo(87.49796, 32.129704, 87.40842, 32.091595, 87.33566, 32.034283);
		shape45.curveTo(87.26262, 31.977676, 87.20742, 31.903925, 87.16896, 31.813732);
		shape45.curveTo(87.13103, 31.723598, 87.107185, 31.617903, 87.09808, 31.496649);
		shape45.curveTo(87.08851, 31.375217, 87.10126, 31.216322, 87.13602, 31.019379);
		shape45.curveTo(87.177124, 30.78638, 87.23414, 30.602825, 87.30712, 30.467829);
		shape45.curveTo(87.38046, 30.33301, 87.471596, 30.235182, 87.580635, 30.173763);
		shape45.curveTo(87.689735, 30.112461, 87.8168, 30.094786, 87.962425, 30.120445);
		shape45.curveTo(88.15391, 30.154325, 88.29214, 30.249159, 88.37698, 30.405762);
		shape45.curveTo(88.478745, 30.593899, 88.49689, 30.875046, 88.431244, 31.247679);
		shape45.closePath();
		shape45.moveTo(88.18058, 31.203524);
		shape45.curveTo(88.237946, 30.877514, 88.23789, 30.653912, 88.180695, 30.532362);
		shape45.curveTo(88.12338, 30.411165, 88.039, 30.34035, 87.9272, 30.32062);
		shape45.curveTo(87.81481, 30.30083, 87.71129, 30.33888, 87.61599, 30.433596);
		shape45.curveTo(87.520515, 30.528427, 87.443886, 30.738466, 87.38657, 31.063477);
		shape45.curveTo(87.32897, 31.390484, 87.32909, 31.614616, 87.38646, 31.735283);
		shape45.curveTo(87.44365, 31.85648, 87.52903, 31.926767, 87.64283, 31.946848);
		shape45.curveTo(87.755104, 31.966696, 87.85275, 31.93487, 87.9366, 31.851898);
		shape45.curveTo(88.041534, 31.745852, 88.12309, 31.52947, 88.18058, 31.20352);
		shape45.closePath();
		g.fill(shape45);
	}

	private void paintShapeNode_0_0_0_0_6_21(Graphics2D g) {
		GeneralPath shape46 = new GeneralPath();
		shape46.moveTo(73.0906, 34.6224);
		shape46.curveTo(73.19846, 34.641483, 73.28261, 34.680943, 73.343796, 34.741245);
		shape46.curveTo(73.404396, 34.80108, 73.44338, 34.879177, 73.460236, 34.97483);
		shape46.curveTo(73.48578, 35.119457, 73.45536, 35.249813, 73.36869, 35.366665);
		shape46.curveTo(73.28208, 35.483517, 73.152954, 35.557266, 72.9805, 35.58774);
		shape46.curveTo(72.8071, 35.618275, 72.65854, 35.592438, 72.53488, 35.51041);
		shape46.curveTo(72.41175, 35.42803, 72.33729, 35.314934, 72.311806, 35.170193);
		shape46.curveTo(72.2956, 35.07824, 72.30552, 34.99386, 72.341515, 34.917053);
		shape46.curveTo(72.3781, 34.840424, 72.441864, 34.774834, 72.53388, 34.72052);
		shape46.curveTo(72.40523, 34.70214, 72.30106, 34.654697, 72.22167, 34.577187);
		shape46.curveTo(72.14275, 34.500088, 72.09219, 34.399853, 72.070526, 34.27725);
		shape46.curveTo(72.04058, 34.10708, 72.07546, 33.954056, 72.17481, 33.81748);
		shape46.curveTo(72.2747, 33.680664, 72.42267, 33.594933, 72.61867, 33.560463);
		shape46.curveTo(72.81479, 33.525818, 72.98273, 33.555824, 73.12336, 33.650597);
		shape46.curveTo(73.26411, 33.74543, 73.34966, 33.8796, 73.380196, 34.053);
		shape46.curveTo(73.40298, 34.18195, 73.38918, 34.2961, 73.33891, 34.39498);
		shape46.curveTo(73.289116, 34.493397, 73.20627, 34.569496, 73.09059, 34.622402);
		shape46.closePath();
		shape46.moveTo(73.2125, 35.027092);
		shape46.curveTo(73.196, 34.933258, 73.15196, 34.861855, 73.08126, 34.812767);
		shape46.curveTo(73.01027, 34.763855, 72.9263, 34.747646, 72.82965, 34.764675);
		shape46.curveTo(72.7357, 34.781235, 72.66377, 34.824394, 72.61409, 34.89421);
		shape46.curveTo(72.56447, 34.964027, 72.547676, 35.041832, 72.56277, 35.127384);
		shape46.curveTo(72.57851, 35.21705, 72.62219, 35.28657, 72.695, 35.3366);
		shape46.curveTo(72.76764, 35.386806, 72.8499, 35.403774, 72.94227, 35.387627);
		shape46.curveTo(73.03528, 35.371185, 73.107155, 35.3275, 73.15823, 35.257446);
		shape46.curveTo(73.20926, 35.186924, 73.22711, 35.109825, 73.212494, 35.027092);
		shape46.closePath();
		shape46.moveTo(73.12947, 34.095806);
		shape46.curveTo(73.117195, 34.02646, 73.0886, 33.961987, 73.04433, 33.902973);
		shape46.curveTo(73.00005, 33.843548, 72.9421, 33.801743, 72.8707, 33.777607);
		shape46.curveTo(72.79918, 33.75365, 72.72619, 33.747776, 72.65121, 33.761047);
		shape46.curveTo(72.534706, 33.7816, 72.4451, 33.83597, 72.38268, 33.924755);
		shape46.curveTo(72.32003, 34.013012, 72.29889, 34.114887, 72.31927, 34.230625);
		shape46.curveTo(72.34, 34.348003, 72.3959, 34.438255, 72.48762, 34.5012);
		shape46.curveTo(72.57928, 34.56421, 72.68409, 34.585346, 72.80153, 34.56456);
		shape46.curveTo(72.916145, 34.54442, 73.0044, 34.48975, 73.06629, 34.400616);
		shape46.curveTo(73.12824, 34.31107, 73.1495, 34.209545, 73.12947, 34.095802);
		shape46.closePath();
		shape46.moveTo(71.90588, 34.73056);
		shape46.curveTo(71.94734, 34.965553, 71.95656, 35.158916, 71.93325, 35.310295);
		shape46.curveTo(71.91041, 35.462143, 71.857796, 35.585396, 71.77618, 35.679993);
		shape46.curveTo(71.69467, 35.774353, 71.58176, 35.834366, 71.437546, 35.859734);
		shape46.curveTo(71.33062, 35.878582, 71.23344, 35.873417, 71.14548, 35.844467);
		shape46.curveTo(71.05746, 35.816223, 70.98036, 35.765785, 70.91342, 35.694263);
		shape46.curveTo(70.84689, 35.62251, 70.78835, 35.53132, 70.73832, 35.420456);
		shape46.curveTo(70.68782, 35.30971, 70.645485, 35.155983, 70.610725, 34.95904);
		shape46.curveTo(70.56968, 34.726044, 70.56046, 34.53403, 70.58289, 34.382244);
		shape46.curveTo(70.605736, 34.230453, 70.657875, 34.10732, 70.73932, 34.01237);
		shape46.curveTo(70.82088, 33.91748, 70.934265, 33.85741, 71.07989, 33.83169);
		shape46.curveTo(71.27137, 33.797985, 71.43373, 33.839855, 71.567024, 33.957996);
		shape46.curveTo(71.726974, 34.100037, 71.84019, 34.357933, 71.90589, 34.730564);
		shape46.closePath();
		shape46.moveTo(71.65527, 34.774834);
		shape46.curveTo(71.597725, 34.448826, 71.52116, 34.238785, 71.4258, 34.144073);
		shape46.curveTo(71.3305, 34.04977, 71.22703, 34.01213, 71.11523, 34.031803);
		shape46.curveTo(71.00284, 34.05165, 70.91858, 34.122818, 70.86138, 34.244427);
		shape46.curveTo(70.80413, 34.36621, 70.803955, 34.589756, 70.86127, 34.914825);
		shape46.curveTo(70.91899, 35.241776, 70.995735, 35.452343, 71.09092, 35.546116);
		shape46.curveTo(71.186165, 35.64042, 71.29039, 35.677296, 71.40419, 35.65721);
		shape46.curveTo(71.51652, 35.63748, 71.597374, 35.57418, 71.64776, 35.46755);
		shape46.curveTo(71.71012, 35.332024, 71.7127, 35.10079, 71.65527, 34.774837);
		shape46.closePath();
		g.fill(shape46);
	}

	private void paintShapeNode_0_0_0_0_6_22(Graphics2D g) {
		GeneralPath shape47 = new GeneralPath();
		shape47.moveTo(82.3425, 29.5138);
		shape47.lineTo(81.93927, 29.551262);
		shape47.curveTo(81.90504, 29.361893, 81.83986, 29.224373, 81.74356, 29.138466);
		shape47.curveTo(81.64738, 29.053381, 81.524185, 29.010517, 81.372986, 29.010517);
		shape47.curveTo(81.24422, 29.010517, 81.131294, 29.039877, 81.034294, 29.099535);
		shape47.curveTo(80.93717, 29.158371, 80.857666, 29.237057, 80.795715, 29.335705);
		shape47.curveTo(80.73283, 29.434294, 80.68115, 29.56794, 80.639114, 29.734938);
		shape47.curveTo(80.59772, 29.90264, 80.576225, 30.073748, 80.576225, 30.24785);
		shape47.curveTo(80.576225, 30.266111, 80.57699, 30.294767, 80.578636, 30.331291);
		shape47.curveTo(80.662895, 30.198586, 80.77746, 30.089603, 80.9222, 30.006926);
		shape47.curveTo(81.06771, 29.923368, 81.225136, 29.882088, 81.39366, 29.882088);
		shape47.curveTo(81.67675, 29.882088, 81.916145, 29.984612, 82.11174, 30.189837);
		shape47.curveTo(82.30745, 30.395002, 82.40522, 30.664522, 82.40522, 31.000103);
		shape47.curveTo(82.40522, 31.346899, 82.3034, 31.62511, 82.099, 31.836676);
		shape47.curveTo(81.89465, 32.048183, 81.63864, 32.153934, 81.330894, 32.153934);
		shape47.curveTo(81.10899, 32.153934, 80.906235, 32.093513, 80.72174, 31.974312);
		shape47.curveTo(80.53806, 31.855053, 80.39808, 31.683945, 80.30272, 31.462925);
		shape47.curveTo(80.207245, 31.241789, 80.1595, 30.921415, 80.1595, 30.501574);
		shape47.curveTo(80.1595, 30.06576, 80.20641, 29.718199, 80.30107, 29.459013);
		shape47.curveTo(80.39643, 29.200531, 80.53724, 29.004173, 80.724144, 28.868942);
		shape47.curveTo(80.91175, 28.73377, 81.13048, 28.666183, 81.38256, 28.666183);
		shape47.curveTo(81.649734, 28.666183, 81.867584, 28.74017, 82.0371, 28.888788);
		shape47.curveTo(82.20639, 29.03676, 82.308266, 29.245039, 82.3425, 29.513796);
		shape47.closePath();
		shape47.moveTo(80.62555, 31.02159);
		shape47.curveTo(80.62555, 31.261692, 80.68914, 31.453352, 80.81721, 31.594162);
		shape47.curveTo(80.94598, 31.735733, 81.09947, 31.806492, 81.28003, 31.806492);
		shape47.curveTo(81.46605, 31.806492, 81.62835, 31.730215, 81.76669, 31.578308);
		shape47.curveTo(81.90504, 31.425638, 81.97415, 31.2284, 81.97415, 30.986593);
		shape47.curveTo(81.97415, 30.768686, 81.90815, 30.592176, 81.77697, 30.456242);
		shape47.curveTo(81.64579, 30.320189, 81.48443, 30.252544, 81.29195, 30.252544);
		shape47.curveTo(81.09706, 30.252544, 80.93799, 30.320189, 80.81315, 30.456242);
		shape47.curveTo(80.68749, 30.592176, 80.62554, 30.780607, 80.62554, 31.021591);
		shape47.closePath();
		shape47.moveTo(79.75157, 30.4085);
		shape47.curveTo(79.75157, 30.81249, 79.710175, 31.137676, 79.62673, 31.383417);
		shape47.curveTo(79.54394, 31.629978, 79.41992, 31.819996, 79.256096, 31.95364);
		shape47.curveTo(79.09233, 32.08717, 78.88634, 32.15393, 78.63831, 32.15393);
		shape47.curveTo(78.45458, 32.15393, 78.29392, 32.116646, 78.15558, 32.042656);
		shape47.curveTo(78.01718, 31.969492, 77.903435, 31.862858, 77.812836, 31.723694);
		shape47.curveTo(77.72288, 31.584646, 77.65212, 31.415182, 77.60127, 31.215538);
		shape47.curveTo(77.5496, 31.01607, 77.52418, 30.747192, 77.52418, 30.4085);
		shape47.curveTo(77.52418, 30.007742, 77.56546, 29.684786, 77.64743, 29.438341);
		shape47.curveTo(77.73005, 29.19178, 77.85342, 29.001705, 78.01719, 28.867296);
		shape47.curveTo(78.181015, 28.733006, 78.387825, 28.666183, 78.63832, 28.666183);
		shape47.curveTo(78.9675, 28.666183, 79.22598, 28.783857, 79.41359, 29.019968);
		shape47.curveTo(79.6386, 29.303877, 79.75158, 29.767523, 79.75158, 30.408499);
		shape47.closePath();
		shape47.moveTo(79.32051, 30.4085);
		shape47.curveTo(79.32051, 29.847908, 79.25451, 29.474865, 79.123276, 29.288721);
		shape47.curveTo(78.992096, 29.103579, 78.830734, 29.01051, 78.63831, 29.01051);
		shape47.curveTo(78.44501, 29.01051, 78.28353, 29.104225, 78.1523, 29.290365);
		shape47.curveTo(78.02112, 29.476446, 77.95512, 29.849373, 77.95512, 30.408495);
		shape47.curveTo(77.95512, 30.970732, 78.02112, 31.34448, 78.1523, 31.528915);
		shape47.curveTo(78.28354, 31.714233, 78.446655, 31.806482, 78.6422, 31.806482);
		shape47.curveTo(78.8355, 31.806482, 78.98899, 31.72451, 79.104256, 31.561563);
		shape47.curveTo(79.24818, 31.35405, 79.32052, 30.969145, 79.32052, 30.408493);
		shape47.closePath();
		g.fill(shape47);
	}

	private void paintShapeNode_0_0_0_0_6_23(Graphics2D g) {
		GeneralPath shape48 = new GeneralPath();
		shape48.moveTo(72.5814, 30.0432);
		shape48.lineTo(72.82508, 30.000277);
		shape48.lineTo(73.09901, 31.553867);
		shape48.curveTo(73.14792, 31.487633, 73.2148, 31.418108, 73.30012, 31.345356);
		shape48.curveTo(73.386086, 31.272602, 73.46412, 31.21541, 73.53511, 31.174189);
		shape48.lineTo(73.57662, 31.409653);
		shape48.curveTo(73.45155, 31.497908, 73.345856, 31.596733, 73.25977, 31.705715);
		shape48.curveTo(73.174164, 31.815344, 73.11726, 31.916342, 73.08978, 32.00988);
		shape48.lineTo(72.93301, 32.037537);
		shape48.lineTo(72.5814, 30.043198);
		shape48.closePath();
		shape48.moveTo(72.12943, 31.13262);
		shape48.curveTo(72.17083, 31.367615, 72.18011, 31.560976, 72.15674, 31.712414);
		shape48.curveTo(72.133896, 31.864262, 72.08128, 31.987396, 71.999725, 32.08205);
		shape48.curveTo(71.91817, 32.17647, 71.805305, 32.236362, 71.66103, 32.26185);
		shape48.curveTo(71.55416, 32.280697, 71.456924, 32.275414, 71.368965, 32.246582);
		shape48.curveTo(71.280945, 32.218338, 71.20385, 32.1679, 71.13691, 32.096317);
		shape48.curveTo(71.07038, 32.024563, 71.011894, 31.933311, 70.96181, 31.822628);
		shape48.curveTo(70.91131, 31.711824, 70.86897, 31.558098, 70.83421, 31.361036);
		shape48.curveTo(70.79317, 31.128038, 70.78401, 30.936085, 70.806496, 30.784355);
		shape48.curveTo(70.82922, 30.632507, 70.881424, 30.509432, 70.96287, 30.414425);
		shape48.curveTo(71.04437, 30.319475, 71.15775, 30.259464, 71.3035, 30.233746);
		shape48.curveTo(71.494865, 30.199982, 71.65728, 30.241907, 71.79051, 30.359991);
		shape48.curveTo(71.95052, 30.502033, 72.063736, 30.759987, 72.12944, 31.13262);
		shape48.closePath();
		shape48.moveTo(71.87876, 31.176836);
		shape48.curveTo(71.82127, 30.850945, 71.744705, 30.640787, 71.649345, 30.546074);
		shape48.curveTo(71.55405, 30.45183, 71.45064, 30.41419, 71.33872, 30.433979);
		shape48.curveTo(71.22633, 30.453768, 71.142075, 30.524818, 71.08488, 30.646484);
		shape48.curveTo(71.02769, 30.76821, 71.02745, 30.99187, 71.08476, 31.316942);
		shape48.curveTo(71.14249, 31.643833, 71.21923, 31.854458, 71.314476, 31.948174);
		shape48.curveTo(71.40972, 32.042534, 71.513885, 32.079353, 71.627686, 32.05927);
		shape48.curveTo(71.74001, 32.03954, 71.82087, 31.976122, 71.871254, 31.869606);
		shape48.curveTo(71.933556, 31.7342, 71.93626, 31.502905, 71.87877, 31.176836);
		shape48.closePath();
		shape48.moveTo(70.58635, 31.404783);
		shape48.curveTo(70.62781, 31.639719, 70.637085, 31.83308, 70.613716, 31.984518);
		shape48.curveTo(70.590935, 32.13625, 70.53826, 32.25956, 70.4567, 32.354095);
		shape48.curveTo(70.375145, 32.448517, 70.26222, 32.50847, 70.11801, 32.53395);
		shape48.curveTo(70.0112, 32.552742, 69.91396, 32.547516, 69.82588, 32.518566);
		shape48.curveTo(69.737976, 32.490322, 69.66082, 32.44, 69.59394, 32.368362);
		shape48.curveTo(69.52741, 32.296665, 69.46887, 32.205418, 69.41884, 32.094612);
		shape48.curveTo(69.368225, 31.983809, 69.326004, 31.830082, 69.291245, 31.633137);
		shape48.curveTo(69.25014, 31.400139, 69.24104, 31.208126, 69.26341, 31.056396);
		shape48.curveTo(69.286255, 30.904665, 69.338394, 30.781473, 69.41984, 30.686523);
		shape48.curveTo(69.501396, 30.591574, 69.614784, 30.531504, 69.76041, 30.505844);
		shape48.curveTo(69.951775, 30.47208, 70.11419, 30.514006, 70.24754, 30.632149);
		shape48.curveTo(70.40749, 30.774073, 70.520645, 31.032085, 70.58636, 31.404778);
		shape48.closePath();
		shape48.moveTo(70.33568, 31.449);
		shape48.curveTo(70.27819, 31.12293, 70.20162, 30.912834, 70.10632, 30.818178);
		shape48.curveTo(70.01096, 30.723875, 69.907555, 30.686235, 69.79575, 30.706024);
		shape48.curveTo(69.683365, 30.725872, 69.599106, 30.79692, 69.54191, 30.918646);
		shape48.curveTo(69.48472, 31.040312, 69.48448, 31.263914, 69.541794, 31.588926);
		shape48.curveTo(69.599396, 31.915934, 69.67615, 32.126503, 69.771385, 32.220333);
		shape48.curveTo(69.866745, 32.314518, 69.97097, 32.35145, 70.084656, 32.33143);
		shape48.curveTo(70.197044, 32.311523, 70.2779, 32.248283, 70.328224, 32.141647);
		shape48.curveTo(70.39059, 32.006184, 70.39323, 31.774887, 70.33568, 31.448996);
		shape48.closePath();
		g.fill(shape48);
	}

	private void paintShapeNode_0_0_0_0_6_24(Graphics2D g) {
		GeneralPath shape49 = new GeneralPath();
		shape49.moveTo(89.5542, 33.9766);
		shape49.lineTo(89.79783, 34.019585);
		shape49.lineTo(89.52384, 35.573235);
		shape49.curveTo(89.59248, 35.527668, 89.67909, 35.48527, 89.7842, 35.44605);
		shape49.curveTo(89.88984, 35.407116, 89.982735, 35.380047, 90.06353, 35.365543);
		shape49.lineTo(90.02202, 35.601067);
		shape49.curveTo(89.87434, 35.64117, 89.741165, 35.697895, 89.623024, 35.77088);
		shape49.curveTo(89.50506, 35.84463, 89.41704, 35.920086, 89.35926, 35.998596);
		shape49.lineTo(89.20242, 35.97094);
		shape49.lineTo(89.55421, 33.9766);
		shape49.closePath();
		shape49.moveTo(88.75679, 34.845764);
		shape49.curveTo(88.71539, 35.080757, 88.65797, 35.265606, 88.58421, 35.399956);
		shape49.curveTo(88.51081, 35.534775, 88.41927, 35.63254, 88.31023, 35.69361);
		shape49.curveTo(88.2013, 35.754444, 88.07476, 35.772118, 87.93049, 35.746693);
		shape49.curveTo(87.82356, 35.727844, 87.73402, 35.689617, 87.66126, 35.632423);
		shape49.curveTo(87.58816, 35.57576, 87.53302, 35.502068, 87.49456, 35.411873);
		shape49.curveTo(87.45663, 35.32168, 87.432846, 35.215927, 87.42363, 35.094788);
		shape49.curveTo(87.414055, 34.973415, 87.42686, 34.81446, 87.461624, 34.6174);
		shape49.curveTo(87.50273, 34.38446, 87.559746, 34.200905, 87.63279, 34.06603);
		shape49.curveTo(87.70607, 33.931152, 87.797264, 33.833324, 87.9063, 33.771904);
		shape49.curveTo(88.01529, 33.71054, 88.14236, 33.69293, 88.288155, 33.718586);
		shape49.curveTo(88.47952, 33.75235, 88.617805, 33.847298, 88.7026, 34.003845);
		shape49.curveTo(88.80442, 34.19204, 88.82256, 34.47313, 88.7568, 34.84576);
		shape49.closePath();
		shape49.moveTo(88.50618, 34.80161);
		shape49.curveTo(88.56361, 34.47566, 88.56349, 34.252056, 88.506355, 34.13039);
		shape49.curveTo(88.44899, 34.00925, 88.36466, 33.938496, 88.25275, 33.918827);
		shape49.curveTo(88.14036, 33.89898, 88.036896, 33.936913, 87.941536, 34.031685);
		shape49.curveTo(87.846176, 34.12652, 87.769485, 34.336613, 87.71218, 34.661625);
		shape49.curveTo(87.65458, 34.988575, 87.65463, 35.212765, 87.71207, 35.33343);
		shape49.curveTo(87.76932, 35.454628, 87.85464, 35.524914, 87.96844, 35.544937);
		shape49.curveTo(88.08071, 35.564785, 88.17841, 35.532898, 88.26221, 35.450047);
		shape49.curveTo(88.36702, 35.34406, 88.4487, 35.12762, 88.50619, 34.80161);
		shape49.closePath();
		shape49.moveTo(87.21377, 34.57372);
		shape49.curveTo(87.17231, 34.808712, 87.11488, 34.99356, 87.04113, 35.12791);
		shape49.curveTo(86.96785, 35.262672, 86.87619, 35.360497, 86.767204, 35.421448);
		shape49.curveTo(86.65828, 35.48228, 86.531685, 35.500015, 86.387405, 35.474648);
		shape49.curveTo(86.280594, 35.455738, 86.19105, 35.417572, 86.11818, 35.360264);
		shape49.curveTo(86.04525, 35.303658, 85.98994, 35.229965, 85.95159, 35.139774);
		shape49.curveTo(85.9136, 35.049698, 85.88976, 34.943886, 85.88066, 34.82269);
		shape49.curveTo(85.87103, 34.70126, 85.88389, 34.542362, 85.918655, 34.34542);
		shape49.curveTo(85.9597, 34.112423, 86.01683, 33.928806, 86.08971, 33.79387);
		shape49.curveTo(86.16311, 33.659107, 86.25418, 33.561165, 86.36322, 33.499863);
		shape49.curveTo(86.47232, 33.4385, 86.59945, 33.420826, 86.74507, 33.446545);
		shape49.curveTo(86.93644, 33.48031, 87.07472, 33.575256, 87.15963, 33.731804);
		shape49.curveTo(87.26133, 33.919884, 87.27947, 34.201088, 87.21377, 34.57372);
		shape49.closePath();
		shape49.moveTo(86.9631, 34.529564);
		shape49.curveTo(87.020584, 34.203556, 87.02046, 33.979893, 86.96327, 33.858345);
		shape49.curveTo(86.90596, 33.73709, 86.82164, 33.666393, 86.70984, 33.64672);
		shape49.curveTo(86.59739, 33.626934, 86.49393, 33.664925, 86.39857, 33.759697);
		shape49.curveTo(86.30321, 33.85447, 86.22652, 34.06451, 86.16921, 34.38952);
		shape49.curveTo(86.11149, 34.716526, 86.11161, 34.940598, 86.16904, 35.061386);
		shape49.curveTo(86.22635, 35.182526, 86.31167, 35.25287, 86.42535, 35.272892);
		shape49.curveTo(86.537796, 35.29268, 86.63538, 35.260853, 86.71912, 35.177883);
		shape49.curveTo(86.824104, 35.071953, 86.90567, 34.855457, 86.9631, 34.529564);
		shape49.closePath();
		g.fill(shape49);
	}

	private void paintShapeNode_0_0_0_0_6_25(Graphics2D g) {
		GeneralPath shape50 = new GeneralPath();
		shape50.moveTo(63.9328, 32.1146);
		shape50.lineTo(64.16539, 32.029987);
		shape50.lineTo(64.70496, 33.512417);
		shape50.curveTo(64.74161, 33.438667, 64.795395, 33.35869, 64.86685, 33.272255);
		shape50.curveTo(64.93872, 33.185642, 65.00572, 33.11577, 65.0685, 33.062862);
		shape50.lineTo(65.15023, 33.28752);
		shape50.curveTo(65.042366, 33.396152, 64.95552, 33.51171, 64.88963, 33.63414);
		shape50.curveTo(64.82434, 33.756924, 64.78582, 33.866318, 64.77502, 33.963142);
		shape50.lineTo(64.62552, 34.017635);
		shape50.lineTo(63.932808, 32.114605);
		shape50.closePath();
		shape50.moveTo(62.65677, 32.57907);
		shape50.lineTo(62.889355, 32.494457);
		shape50.lineTo(63.428925, 33.976887);
		shape50.curveTo(63.465626, 33.903137, 63.519352, 33.8231, 63.590813, 33.736668);
		shape50.curveTo(63.662804, 33.649998, 63.729744, 33.58012, 63.792454, 33.527332);
		shape50.lineTo(63.87419, 33.751934);
		shape50.curveTo(63.766384, 33.860504, 63.679478, 33.976242, 63.613655, 34.098495);
		shape50.curveTo(63.548416, 34.22128, 63.50978, 34.330673, 63.499096, 34.427616);
		shape50.lineTo(63.349422, 34.48199);
		shape50.lineTo(62.65677, 32.57907);
		shape50.closePath();
		shape50.moveTo(62.40093, 33.730442);
		shape50.curveTo(62.48249, 33.95463, 62.525238, 34.14353, 62.528526, 34.296673);
		shape50.curveTo(62.532402, 34.450047, 62.501926, 34.58064, 62.43804, 34.687862);
		shape50.curveTo(62.374096, 34.795082, 62.273335, 34.873707, 62.135696, 34.923855);
		shape50.curveTo(62.03376, 34.960964, 61.93717, 34.97265, 61.845448, 34.959496);
		shape50.curveTo(61.753902, 34.946873, 61.66923, 34.91064, 61.590782, 34.851746);
		shape50.curveTo(61.51292, 34.792793, 61.439346, 34.71299, 61.370876, 34.612583);
		shape50.curveTo(61.30188, 34.51229, 61.233532, 34.368195, 61.165005, 34.180233);
		shape50.curveTo(61.08415, 33.957924, 61.04187, 33.77049, 61.037525, 33.617172);
		shape50.curveTo(61.03365, 33.463737, 61.063595, 33.33338, 61.127365, 33.22575);
		shape50.curveTo(61.191193, 33.118057, 61.292423, 33.039257, 61.431355, 32.988644);
		shape50.curveTo(61.61403, 32.922173, 61.781204, 32.93527, 61.933052, 33.028397);
		shape50.curveTo(62.115257, 33.14049, 62.27145, 33.374897, 62.40093, 33.730442);
		shape50.closePath();
		shape50.moveTo(62.161766, 33.817524);
		shape50.curveTo(62.048553, 33.506428, 61.936638, 33.31289, 61.826244, 33.236145);
		shape50.curveTo(61.71603, 33.15987, 61.60763, 33.140785, 61.50094, 33.179657);
		shape50.curveTo(61.39372, 33.218647, 61.32302, 33.30332, 61.287907, 33.43309);
		shape50.curveTo(61.252674, 33.56286, 61.291313, 33.783173, 61.40423, 34.09333);
		shape50.curveTo(61.517673, 34.405247, 61.629887, 34.599316, 61.739925, 34.67518);
		shape50.curveTo(61.850082, 34.751453, 61.95924, 34.76972, 62.067696, 34.730198);
		shape50.curveTo(62.174915, 34.69115, 62.243618, 34.614758, 62.27468, 34.500957);
		shape50.curveTo(62.312496, 34.356743, 62.274975, 34.12856, 62.161762, 33.817524);
		shape50.closePath();
		g.fill(shape50);
	}

	private void paintShapeNode_0_0_0_0_6_26(Graphics2D g) {
		GeneralPath shape51 = new GeneralPath();
		shape51.moveTo(97.3601, 36.2995);
		shape51.lineTo(97.59269, 36.384174);
		shape51.lineTo(97.053116, 37.866604);
		shape51.curveTo(97.12857, 37.833664, 97.22123, 37.807003, 97.331505, 37.786686);
		shape51.curveTo(97.44225, 37.766544, 97.53849, 37.756035, 97.62058, 37.755917);
		shape51.lineTo(97.53879, 37.980576);
		shape51.curveTo(97.38635, 37.994434, 97.24548, 38.02714, 97.11636, 38.078518);
		shape51.curveTo(96.98742, 38.130604, 96.887596, 38.189674, 96.81707, 38.25691);
		shape51.lineTo(96.66751, 38.202534);
		shape51.lineTo(97.36011, 36.299503);
		shape51.closePath();
		shape51.moveTo(96.08407, 35.835087);
		shape51.lineTo(96.31666, 35.91976);
		shape51.lineTo(95.77703, 37.40219);
		shape51.curveTo(95.8526, 37.36931, 95.945206, 37.342472, 96.055534, 37.322216);
		shape51.curveTo(96.166336, 37.302135, 96.26258, 37.291622, 96.34455, 37.291508);
		shape51.lineTo(96.26282, 37.51611);
		shape51.curveTo(96.11044, 37.52997, 95.96946, 37.562733, 95.84045, 37.614113);
		shape51.curveTo(95.7115, 37.6662, 95.61162, 37.72521, 95.54109, 37.79256);
		shape51.lineTo(95.39153, 37.738068);
		shape51.lineTo(96.08407, 35.8351);
		shape51.closePath();
		shape51.moveTo(95.148026, 36.552635);
		shape51.curveTo(95.066345, 36.776825, 94.977684, 36.948933, 94.88179, 37.068367);
		shape51.curveTo(94.78613, 37.18839, 94.67892, 37.268833, 94.561066, 37.309937);
		shape51.curveTo(94.4431, 37.35092, 94.315384, 37.3464, 94.17775, 37.296314);
		shape51.curveTo(94.07581, 37.259262, 93.994255, 37.20612, 93.93248, 37.13701);
		shape51.curveTo(93.870415, 37.068542, 93.82884, 36.986393, 93.80665, 36.890858);
		shape51.curveTo(93.78492, 36.795616, 93.779816, 36.68722, 93.79191, 36.566257);
		shape51.curveTo(93.803535, 36.44506, 93.84376, 36.290806, 93.91211, 36.102787);
		shape51.curveTo(93.99308, 35.880478, 94.08116, 35.70972, 94.17641, 35.589462);
		shape51.curveTo(94.272064, 35.46944, 94.378815, 35.388817, 94.49684, 35.347363);
		shape51.curveTo(94.61493, 35.30591, 94.74311, 35.310604, 94.882095, 35.36116);
		shape51.curveTo(95.06477, 35.42763, 95.18444, 35.545128, 95.24087, 35.71406);
		shape51.curveTo(95.308334, 35.917053, 95.27739, 36.19703, 95.14803, 36.55263);
		shape51.closePath();
		shape51.moveTo(94.908806, 36.465553);
		shape51.curveTo(95.02207, 36.15452, 95.06077, 35.93432, 95.02554, 35.80455);
		shape51.curveTo(94.990135, 35.67531, 94.91932, 35.59099, 94.81263, 35.552174);
		shape51.curveTo(94.70541, 35.513126, 94.59684, 35.532562, 94.4865, 35.609367);
		shape51.curveTo(94.376114, 35.686172, 94.26408, 35.87977, 94.151215, 36.189926);
		shape51.curveTo(94.03765, 36.501842, 93.99889, 36.722626, 94.03436, 36.851456);
		shape51.curveTo(94.06977, 36.980698, 94.14159, 37.064842, 94.25015, 37.1043);
		shape51.curveTo(94.357376, 37.14329, 94.459076, 37.128963, 94.55596, 37.06173);
		shape51.curveTo(94.67769, 36.97553, 94.79559, 36.77659, 94.908806, 36.465553);
		shape51.closePath();
		g.fill(shape51);
	}

	private void paintShapeNode_0_0_0_0_6_27(Graphics2D g) {
		GeneralPath shape52 = new GeneralPath();
		shape52.moveTo(55.9818, 35.5707);
		shape52.lineTo(56.196068, 35.44698);
		shape52.lineTo(56.9849, 36.813198);
		shape52.curveTo(57.008156, 36.734222, 57.047203, 36.646084, 57.102573, 36.54855);
		shape52.curveTo(57.158356, 36.45072, 57.212204, 36.370277, 57.264816, 36.307274);
		shape52.lineTo(57.38437, 36.514317);
		shape52.curveTo(57.296936, 36.640095, 57.231464, 36.76904, 57.187954, 36.900925);
		shape52.curveTo(57.144913, 37.03316, 57.125946, 37.147606, 57.13223, 37.244904);
		shape52.lineTo(56.994297, 37.324467);
		shape52.lineTo(55.981796, 35.570698);
		shape52.closePath();
		shape52.moveTo(54.42433, 36.744793);
		shape52.lineTo(54.30525, 36.53869);
		shape52.lineTo(55.45973, 35.872166);
		shape52.curveTo(55.491142, 35.922783, 55.511696, 35.97704, 55.520798, 36.03523);
		shape52.curveTo(55.537064, 36.131058, 55.534595, 36.23552, 55.513985, 36.34932);
		shape52.curveTo(55.492847, 36.463, 55.450214, 36.60516, 55.385918, 36.775562);
		shape52.curveTo(55.286095, 37.040974, 55.228607, 37.238445, 55.214634, 37.36798);
		shape52.curveTo(55.20025, 37.49716, 55.216747, 37.602566, 55.263428, 37.683537);
		shape52.curveTo(55.312458, 37.768505, 55.38433, 37.822468, 55.47869, 37.846012);
		shape52.curveTo(55.573288, 37.868973, 55.66941, 37.852413, 55.766945, 37.7961);
		shape52.curveTo(55.870293, 37.736443, 55.93506, 37.657875, 55.960896, 37.559933);
		shape52.curveTo(55.987377, 37.46234, 55.969173, 37.358524, 55.906933, 37.248604);
		shape52.lineTo(56.140106, 37.143673);
		shape52.curveTo(56.220142, 37.31707, 56.23564, 37.4752, 56.186905, 37.61824);
		shape52.curveTo(56.138287, 37.76116, 56.030712, 37.88089, 55.864773, 37.976604);
		shape52.curveTo(55.697308, 38.073315, 55.537647, 38.103203, 55.386505, 38.066856);
		shape52.curveTo(55.234776, 38.030216, 55.11951, 37.94337, 55.04012, 37.80591);
		shape52.curveTo(54.99972, 37.735916, 54.974354, 37.658993, 54.964252, 37.57485);
		shape52.curveTo(54.953682, 37.490997, 54.96026, 37.39223, 54.98357, 37.279667);
		shape52.curveTo(55.006588, 37.166454, 55.05773, 37.00034, 55.13665, 36.78132);
		shape52.curveTo(55.202766, 36.598938, 55.242992, 36.478092, 55.25726, 36.418373);
		shape52.curveTo(55.27182, 36.35901, 55.279633, 36.30299, 55.2811, 36.250202);
		shape52.lineTo(54.424328, 36.744793);
		shape52.closePath();
		shape52.moveTo(54.572773, 37.532394);
		shape52.curveTo(54.69209, 37.73903, 54.766956, 37.91759, 54.796787, 38.067856);
		shape52.curveTo(54.827206, 38.218353, 54.819862, 38.352116, 54.77559, 38.46885);
		shape52.curveTo(54.731316, 38.585526, 54.64564, 38.680473, 54.518867, 38.753754);
		shape52.curveTo(54.42486, 38.807953, 54.33173, 38.836254, 54.23919, 38.83919);
		shape52.curveTo(54.146824, 38.842773, 54.057102, 38.82181, 53.96967, 38.77736);
		shape52.curveTo(53.882706, 38.73279, 53.796448, 38.667084, 53.711597, 38.580063);
		shape52.curveTo(53.62616, 38.493217, 53.533794, 38.36333, 53.433796, 38.19005);
		shape52.curveTo(53.315536, 37.985176, 53.241257, 37.807785, 53.210487, 37.65758);
		shape52.curveTo(53.179955, 37.50714, 53.186825, 37.373558, 53.23098, 37.25647);
		shape52.curveTo(53.275078, 37.139385, 53.361042, 37.04426, 53.489227, 36.970272);
		shape52.curveTo(53.657516, 36.873093, 53.824398, 36.856884, 53.9901, 36.9223);
		shape52.curveTo(54.188984, 37.00104, 54.383636, 37.204678, 54.572773, 37.53239);
		shape52.closePath();
		shape52.moveTo(54.35234, 37.6597);
		shape52.curveTo(54.18687, 37.37309, 54.043007, 37.201805, 53.921047, 37.145493);
		shape52.curveTo(53.799263, 37.089474, 53.689167, 37.089474, 53.59081, 37.146255);
		shape52.curveTo(53.491985, 37.203274, 53.43708, 37.298927, 53.424984, 37.432865);
		shape52.curveTo(53.41283, 37.566746, 53.489223, 37.77696, 53.654224, 38.062748);
		shape52.curveTo(53.820225, 38.350353, 53.96438, 38.521935, 54.08581, 38.57754);
		shape52.curveTo(54.207596, 38.63356, 54.318222, 38.63256, 54.41822, 38.57484);
		shape52.curveTo(54.517044, 38.517765, 54.57136, 38.430626, 54.582283, 38.313187);
		shape52.curveTo(54.594555, 38.16457, 54.517868, 37.94637, 54.352337, 37.6597);
		shape52.closePath();
		g.fill(shape52);
	}

	private void paintShapeNode_0_0_0_0_6_28(Graphics2D g) {
		GeneralPath shape53 = new GeneralPath();
		shape53.moveTo(104.742, 40.037);
		shape53.lineTo(104.95626, 40.160778);
		shape53.lineTo(104.16754, 41.526997);
		shape53.curveTo(104.24752, 41.507618, 104.34341, 41.497402, 104.45556, 41.49658);
		shape53.curveTo(104.568184, 41.49599, 104.66477, 41.50239, 104.74563, 41.516426);
		shape53.lineTo(104.626076, 41.72353);
		shape53.curveTo(104.473465, 41.71067, 104.32902, 41.718422, 104.19308, 41.746666);
		shape53.curveTo(104.05702, 41.775555, 103.948456, 41.816364, 103.8673, 41.870445);
		shape53.lineTo(103.72943, 41.790764);
		shape53.lineTo(104.74198, 40.036995);
		shape53.closePath();
		shape53.moveTo(102.94648, 39.27529);
		shape53.lineTo(103.065445, 39.06907);
		shape53.lineTo(104.219864, 39.73559);
		shape53.curveTo(104.191795, 39.788147, 104.15504, 39.833126, 104.10918, 39.87006);
		shape53.curveTo(104.03431, 39.932068, 103.94265, 39.982155, 103.83384, 40.021202);
		shape53.curveTo(103.72474, 40.059723, 103.580345, 40.093895, 103.400665, 40.123432);
		shape53.curveTo(103.120865, 40.169704, 102.921104, 40.218616, 102.8019, 40.271286);
		shape53.curveTo(102.68288, 40.32343, 102.59979, 40.390427, 102.55305, 40.471344);
		shape53.curveTo(102.50401, 40.55625, 102.49316, 40.645504, 102.51993, 40.738987);
		shape53.curveTo(102.54735, 40.83241, 102.60977, 40.907394, 102.7073, 40.963707);
		shape53.curveTo(102.810646, 41.023308, 102.91106, 41.04016, 103.00882, 41.01356);
		shape53.curveTo(103.10659, 40.987667, 103.187386, 40.92002, 103.25145, 40.811157);
		shape53.lineTo(103.45891, 40.960598);
		shape53.curveTo(103.348755, 41.116673, 103.21957, 41.2091, 103.071304, 41.238457);
		shape53.curveTo(102.92322, 41.267815, 102.765785, 41.234463, 102.59991, 41.138634);
		shape53.curveTo(102.43238, 41.04198, 102.326744, 40.91867, 102.282646, 40.76958);
		shape53.curveTo(102.23849, 40.619907, 102.25605, 40.47663, 102.335434, 40.33917);
		shape53.curveTo(102.37577, 40.269176, 102.429794, 40.208694, 102.49762, 40.1579);
		shape53.curveTo(102.56491, 40.106815, 102.653755, 40.063187, 102.762856, 40.027016);
		shape53.curveTo(102.87243, 39.990376, 103.04183, 39.95156, 103.27101, 39.910458);
		shape53.curveTo(103.46197, 39.87652, 103.5868, 39.850918, 103.64564, 39.83342);
		shape53.curveTo(103.70435, 39.816334, 103.75674, 39.795074, 103.803185, 39.769943);
		shape53.lineTo(102.94647, 39.27529);
		shape53.closePath();
		shape53.moveTo(102.338615, 39.797657);
		shape53.curveTo(102.2193, 40.00429, 102.1021, 40.158367, 101.986885, 40.25937);
		shape53.curveTo(101.871735, 40.36101, 101.75224, 40.421494, 101.62905, 40.441517);
		shape53.curveTo(101.50586, 40.461483, 101.38078, 40.434765, 101.25395, 40.3616);
		shape53.curveTo(101.16, 40.307285, 101.08889, 40.240814, 101.04009, 40.16213);
		shape53.curveTo(100.99083, 40.083973, 100.96411, 39.995777, 100.958824, 39.897835);
		shape53.curveTo(100.95401, 39.800186, 100.96775, 39.69267, 101.00075, 39.57564);
		shape53.curveTo(101.03322, 39.458263, 101.09951, 39.313343, 101.19957, 39.140064);
		shape53.curveTo(101.31783, 38.935192, 101.434326, 38.78223, 101.549065, 38.68047);
		shape53.curveTo(101.66404, 38.578766, 101.78318, 38.517994, 101.90661, 38.497616);
		shape53.curveTo(102.0301, 38.4773, 102.155464, 38.504192, 102.283585, 38.57818);
		shape53.curveTo(102.451935, 38.6753, 102.54941, 38.811764, 102.57565, 38.98798);
		shape53.curveTo(102.606834, 39.199543, 102.527855, 39.469948, 102.3386, 39.79766);
		shape53.closePath();
		shape53.moveTo(102.11819, 39.67041);
		shape53.curveTo(102.2836, 39.383743, 102.360054, 39.173527, 102.34784, 39.039764);
		shape53.curveTo(102.33545, 38.906296, 102.28037, 38.810936, 102.182014, 38.754154);
		shape53.curveTo(102.08325, 38.69708, 101.97298, 38.697372, 101.8509, 38.75386);
		shape53.curveTo(101.72888, 38.81029, 101.585014, 38.981514, 101.42001, 39.2673);
		shape53.curveTo(101.25395, 39.554848, 101.177444, 39.765472, 101.19001, 39.898533);
		shape53.curveTo(101.2024, 40.032, 101.25859, 40.127243, 101.358536, 40.185024);
		shape53.curveTo(101.45742, 40.2421, 101.56006, 40.245506, 101.66716, 40.19624);
		shape53.curveTo(101.80198, 40.132587, 101.95265, 39.957077, 102.11818, 39.67041);
		shape53.closePath();
		g.fill(shape53);
	}

	private void paintShapeNode_0_0_0_0_6_29(Graphics2D g) {
		GeneralPath shape54 = new GeneralPath();
		shape54.moveTo(48.7342, 40.4246);
		shape54.lineTo(48.923744, 40.26547);
		shape54.lineTo(49.937885, 41.47403);
		shape54.curveTo(49.947044, 41.39206, 49.97018, 41.29852, 50.00782, 41.19282);
		shape54.curveTo(50.045753, 41.086834, 50.0848, 40.998283, 50.12573, 40.927116);
		shape54.lineTo(50.27934, 41.11026);
		shape54.curveTo(50.21516, 41.24925, 50.173058, 41.38765, 50.15309, 41.525112);
		shape54.curveTo(50.133713, 41.662807, 50.134888, 41.778778, 50.15785, 41.873493);
		shape54.lineTo(50.036007, 41.97584);
		shape54.lineTo(48.734196, 40.42459);
		shape54.closePath();
		shape54.moveTo(48.589573, 41.24103);
		shape54.lineTo(48.421047, 41.425232);
		shape54.curveTo(48.308952, 41.336155, 48.20696, 41.289944, 48.115, 41.286243);
		shape54.curveTo(48.022636, 41.28278, 47.93902, 41.31243, 47.86386, 41.375496);
		shape54.curveTo(47.774727, 41.450363, 47.725105, 41.544724, 47.71542, 41.658173);
		shape54.curveTo(47.7062, 41.771324, 47.73955, 41.87391, 47.816242, 41.965332);
		shape54.curveTo(47.88929, 42.052296, 47.97813, 42.100388, 48.08224, 42.109196);
		shape54.curveTo(48.18688, 42.11759, 48.283, 42.085415, 48.370728, 42.01178);
		shape54.curveTo(48.406662, 41.981598, 48.4453, 41.93703, 48.486816, 41.87831);
		shape54.lineTo(48.605255, 42.062572);
		shape54.curveTo(48.59157, 42.071793, 48.58077, 42.079426, 48.572903, 42.08606);
		shape54.curveTo(48.49234, 42.153706, 48.437553, 42.23597, 48.408077, 42.33233);
		shape54.curveTo(48.37854, 42.428745, 48.400913, 42.521053, 48.474487, 42.608723);
		shape54.curveTo(48.532738, 42.67819, 48.605137, 42.71612, 48.69028, 42.721935);
		shape54.curveTo(48.775658, 42.7281, 48.855812, 42.70015, 48.930386, 42.637672);
		shape54.curveTo(49.00402, 42.575783, 49.045826, 42.500797, 49.05622, 42.41366);
		shape54.curveTo(49.066498, 42.32593, 49.039543, 42.229572, 48.975777, 42.124992);
		shape54.lineTo(49.193684, 41.9998);
		shape54.curveTo(49.277416, 42.146423, 49.307365, 42.289406, 49.283817, 42.427925);
		shape54.curveTo(49.260094, 42.566914, 49.18752, 42.687874, 49.065205, 42.790398);
		shape54.curveTo(48.98106, 42.86104, 48.88793, 42.907894, 48.786934, 42.931442);
		shape54.curveTo(48.685585, 42.95458, 48.58987, 42.95082, 48.499622, 42.919933);
		shape54.curveTo(48.409664, 42.888695, 48.33509, 42.83849, 48.27602, 42.768024);
		shape54.curveTo(48.22012, 42.701435, 48.18706, 42.625572, 48.176903, 42.54043);
		shape54.curveTo(48.167038, 42.45546, 48.183304, 42.36762, 48.226284, 42.276485);
		shape54.curveTo(48.117477, 42.33221, 48.009785, 42.34777, 47.90327, 42.323814);
		shape54.curveTo(47.797283, 42.29986, 47.70245, 42.23797, 47.618774, 42.13832);
		shape54.curveTo(47.50568, 42.003384, 47.458588, 41.847954, 47.478317, 41.6715);
		shape54.curveTo(47.49781, 41.49511, 47.583073, 41.343964, 47.733395, 41.217896);
		shape54.curveTo(47.86892, 41.1041, 48.015366, 41.04978, 48.17279, 41.055653);
		shape54.curveTo(48.32992, 41.061054, 48.468853, 41.122826, 48.58958, 41.241028);
		shape54.closePath();
		shape54.moveTo(47.687233, 42.601143);
		shape54.curveTo(47.840668, 42.78388, 47.945423, 42.946823, 48.000793, 43.08957);
		shape54.curveTo(48.056988, 43.232372, 48.07296, 43.36549, 48.04959, 43.488155);
		shape54.curveTo(48.02628, 43.610703, 47.958458, 43.71916, 47.846306, 43.813286);
		shape54.curveTo(47.7631, 43.882927, 47.676254, 43.927082, 47.58565, 43.94605);
		shape54.curveTo(47.49528, 43.965485, 47.403385, 43.960495, 47.309555, 43.931957);
		shape54.curveTo(47.21619, 43.903126, 47.119892, 43.853275, 47.02107, 43.7824);
		shape54.curveTo(46.92195, 43.711758, 46.808445, 43.599724, 46.67985, 43.446526);
		shape54.curveTo(46.52771, 43.265316, 46.423893, 43.103607, 46.367405, 42.960976);
		shape54.curveTo(46.31121, 42.81811, 46.294827, 42.68535, 46.318024, 42.56239);
		shape54.curveTo(46.34104, 42.43943, 46.409214, 42.3308, 46.52254, 42.235676);
		shape54.curveTo(46.67145, 42.11078, 46.83299, 42.06586, 47.00756, 42.1015);
		shape54.curveTo(47.217014, 42.144424, 47.44408, 42.311245, 47.687237, 42.601143);
		shape54.closePath();
		shape54.moveTo(47.492287, 42.764736);
		shape54.curveTo(47.279606, 42.511185, 47.108204, 42.367558, 46.9782, 42.33321);
		shape54.curveTo(46.84855, 42.299152, 46.74027, 42.318356, 46.653187, 42.3914);
		shape54.curveTo(46.565754, 42.46468, 46.52829, 42.56838, 46.539684, 42.702435);
		shape54.curveTo(46.550957, 42.836433, 46.662582, 43.030148, 46.874676, 43.282993);
		shape54.curveTo(47.08812, 43.537247, 47.259933, 43.681286, 47.38929, 43.71493);
		shape54.curveTo(47.51894, 43.74899, 47.62769, 43.72873, 47.71612, 43.654568);
		shape54.curveTo(47.803436, 43.58117, 47.8419, 43.485928, 47.83221, 43.36849);
		shape54.curveTo(47.818413, 43.21993, 47.705143, 43.018406, 47.492283, 42.764736);
		shape54.closePath();
		g.fill(shape54);
	}

	private void paintShapeNode_0_0_0_0_6_30(Graphics2D g) {
		GeneralPath shape55 = new GeneralPath();
		shape55.moveTo(111.263, 44.9563);
		shape55.lineTo(111.45261, 45.115368);
		shape55.lineTo(110.43852, 46.32393);
		shape55.curveTo(110.52084, 46.31876, 110.616974, 46.325336, 110.7276, 46.34401);
		shape55.curveTo(110.83858, 46.362976, 110.93259, 46.38605, 111.00974, 46.413944);
		shape55.lineTo(110.85602, 46.597088);
		shape55.curveTo(110.708046, 46.55798, 110.564415, 46.540543, 110.425545, 46.54477);
		shape55.curveTo(110.28661, 46.549583, 110.172585, 46.5709, 110.08333, 46.609947);
		shape55.lineTo(109.961365, 46.507717);
		shape55.lineTo(111.26299, 44.9563);
		shape55.closePath();
		shape55.moveTo(110.43382, 44.955654);
		shape55.lineTo(110.2232, 44.821716);
		shape55.curveTo(110.29143, 44.695824, 110.31921, 44.587368, 110.30687, 44.496178);
		shape55.curveTo(110.294304, 44.404633, 110.25056, 44.32736, 110.1754, 44.264297);
		shape55.curveTo(110.086205, 44.18955, 109.98468, 44.157078, 109.87123, 44.167236);
		shape55.curveTo(109.758255, 44.177807, 109.66296, 44.22848, 109.586205, 44.319847);
		shape55.curveTo(109.513275, 44.40693, 109.48133, 44.502758, 109.49078, 44.606808);
		shape55.curveTo(109.50065, 44.711327, 109.549034, 44.800404, 109.63682, 44.87398);
		shape55.curveTo(109.67275, 44.904163, 109.72337, 44.934464, 109.78837, 44.96517);
		shape55.lineTo(109.62748, 45.11373);
		shape55.curveTo(109.61609, 45.10187, 109.606636, 45.092594, 109.59876, 45.085957);
		shape55.curveTo(109.51814, 45.01837, 109.4276, 44.978737, 109.3276, 44.96646);
		shape55.curveTo(109.22754, 44.95407, 109.14046, 44.992123, 109.06694, 45.07979);
		shape55.curveTo(109.00863, 45.149254, 108.98386, 45.227116, 108.99295, 45.312023);
		shape55.curveTo(109.00164, 45.397106, 109.0431, 45.47121, 109.117615, 45.533806);
		shape55.curveTo(109.19131, 45.595577, 109.2724, 45.623764, 109.36007, 45.61889);
		shape55.curveTo(109.448265, 45.61372, 109.53846, 45.570503, 109.630356, 45.48947);
		shape55.lineTo(109.79148, 45.682365);
		shape55.curveTo(109.66165, 45.79029, 109.52607, 45.84461, 109.3855, 45.845486);
		shape55.curveTo(109.244576, 45.84619, 109.11281, 45.79575, 108.99061, 45.69311);
		shape55.curveTo(108.9064, 45.62253, 108.8441, 45.538914, 108.80335, 45.443554);
		shape55.curveTo(108.762955, 45.347782, 108.75009, 45.252834, 108.76477, 45.15865);
		shape55.curveTo(108.77992, 45.06464, 108.816444, 44.98249, 108.87557, 44.91203);
		shape55.curveTo(108.93141, 44.84544, 109.00041, 44.799698, 109.0825, 44.77492);
		shape55.curveTo(109.16447, 44.750435, 109.253784, 44.751198, 109.35102, 44.77774);
		shape55.curveTo(109.27727, 44.680267, 109.24321, 44.576862, 109.24832, 44.46782);
		shape55.curveTo(109.25349, 44.35925, 109.298, 44.25514, 109.38155, 44.155434);
		shape55.curveTo(109.49482, 44.020615, 109.63974, 43.947216, 109.816956, 43.936058);
		shape55.curveTo(109.99405, 43.924606, 110.15771, 43.98233, 110.30797, 44.10846);
		shape55.curveTo(110.44355, 44.2222, 110.52247, 44.357018, 110.54402, 44.513035);
		shape55.curveTo(110.56598, 44.6687, 110.52922, 44.81626, 110.43381, 44.95566);
		shape55.closePath();
		shape55.moveTo(108.93771, 44.303223);
		shape55.curveTo(108.7844, 44.486076, 108.64212, 44.61749, 108.51118, 44.69682);
		shape55.curveTo(108.38023, 44.77697, 108.25199, 44.81584, 108.12709, 44.81414);
		shape55.curveTo(108.00231, 44.812378, 107.88382, 44.764465, 107.77161, 44.670338);
		shape55.curveTo(107.688576, 44.60052, 107.630035, 44.52266, 107.59563, 44.436695);
		shape55.curveTo(107.560745, 44.351143, 107.549706, 44.259716, 107.56151, 44.162357);
		shape55.curveTo(107.57372, 44.06541, 107.60602, 43.96189, 107.65869, 43.85226);
		shape55.curveTo(107.71107, 43.742397, 107.801674, 43.61116, 107.93021, 43.457962);
		shape55.curveTo(108.08223, 43.276638, 108.22345, 43.14634, 108.35416, 43.065952);
		shape55.curveTo(108.48504, 42.9858, 108.61293, 42.946575, 108.73812, 42.948044);
		shape55.curveTo(108.86319, 42.949394, 108.98198, 42.99766, 109.09537, 43.092728);
		shape55.curveTo(109.24422, 43.21774, 109.316505, 43.369003, 109.31169, 43.547096);
		shape55.curveTo(109.30582, 43.760834, 109.180984, 44.013443, 108.937706, 44.303223);
		shape55.closePath();
		shape55.moveTo(108.74277, 44.13963);
		shape55.curveTo(108.955505, 43.88614, 109.06719, 43.692425, 109.07847, 43.558426);
		shape55.curveTo(109.08945, 43.42484, 109.05181, 43.321495, 108.96473, 43.248447);
		shape55.curveTo(108.87736, 43.17505, 108.76872, 43.1562, 108.638664, 43.190727);
		shape55.curveTo(108.50872, 43.22508, 108.33726, 43.368645, 108.12511, 43.621433);
		shape55.curveTo(107.91178, 43.875748, 107.79974, 44.06999, 107.789116, 44.203224);
		shape55.curveTo(107.778076, 44.33681, 107.81695, 44.44033, 107.90532, 44.514553);
		shape55.curveTo(107.99275, 44.587833, 108.09322, 44.60915, 108.20719, 44.579205);
		shape55.curveTo(108.35111, 44.539864, 108.529915, 44.3933, 108.74277, 44.139633);
		shape55.closePath();
		g.fill(shape55);
	}

	private void paintShapeNode_0_0_0_0_6_31(Graphics2D g) {
		GeneralPath shape56 = new GeneralPath();
		shape56.moveTo(42.3238, 46.422);
		shape56.lineTo(42.48293, 46.232456);
		shape56.lineTo(43.69149, 47.246418);
		shape56.curveTo(43.686264, 47.16427, 43.692898, 47.068027, 43.711456, 46.957462);
		shape56.curveTo(43.73054, 46.846424, 43.75356, 46.752533, 43.78145, 46.675316);
		shape56.lineTo(43.964592, 46.828983);
		shape56.curveTo(43.925484, 46.977013, 43.908104, 47.120525, 43.912216, 47.259514);
		shape56.curveTo(43.91709, 47.398384, 43.938347, 47.512478, 43.977455, 47.60167);
		shape56.lineTo(43.875168, 47.72369);
		shape56.lineTo(42.323807, 46.422);
		shape56.closePath();
		shape56.moveTo(41.406136, 47.515762);
		shape56.lineTo(41.77589, 47.825916);
		shape56.lineTo(42.33807, 47.155872);
		shape56.lineTo(42.51229, 47.301907);
		shape56.lineTo(42.921623, 48.846867);
		shape56.lineTo(42.79156, 49.001884);
		shape56.lineTo(41.79057, 48.16202);
		shape56.lineTo(41.615585, 48.37065);
		shape56.lineTo(41.441364, 48.2245);
		shape56.lineTo(41.616467, 48.01587);
		shape56.lineTo(41.24671, 47.705597);
		shape56.lineTo(41.406136, 47.51576);
		shape56.closePath();
		shape56.moveTo(41.949993, 47.97201);
		shape56.lineTo(42.64623, 48.556328);
		shape56.lineTo(42.355568, 47.488697);
		shape56.lineTo(41.949993, 47.972015);
		shape56.closePath();
		shape56.moveTo(41.670784, 48.74734);
		shape56.curveTo(41.85352, 48.900715, 41.98499, 49.042816, 42.06438, 49.173878);
		shape56.curveTo(42.14453, 49.304764, 42.183346, 49.433064, 42.181644, 49.557903);
		shape56.curveTo(42.179882, 49.682682, 42.13197, 49.801178, 42.03784, 49.91339);
		shape56.curveTo(41.968025, 49.99642, 41.890163, 50.054962, 41.804256, 50.089375);
		shape56.curveTo(41.718704, 50.124313, 41.627277, 50.135292, 41.52992, 50.123432);
		shape56.curveTo(41.432915, 50.111217, 41.32939, 50.078983, 41.21988, 50.02631);
		shape56.curveTo(41.10984, 49.973873, 40.978718, 49.883327, 40.825462, 49.75479);
		shape56.curveTo(40.644196, 49.602768, 40.51384, 49.461487, 40.433453, 49.330837);
		shape56.curveTo(40.353302, 49.19995, 40.314194, 49.07206, 40.315487, 48.94693);
		shape56.curveTo(40.316895, 48.821857, 40.36516, 48.70295, 40.46023, 48.589622);
		shape56.curveTo(40.585186, 48.440826, 40.736507, 48.368546, 40.9146, 48.37324);
		shape56.curveTo(41.128338, 48.37923, 41.38089, 48.50407, 41.670788, 48.74734);
		shape56.closePath();
		shape56.moveTo(41.507133, 48.942287);
		shape56.curveTo(41.253582, 48.72949, 41.059868, 48.617805, 40.925987, 48.60659);
		shape56.curveTo(40.7924, 48.59561, 40.688995, 48.633247, 40.615948, 48.72033);
		shape56.curveTo(40.54255, 48.807644, 40.52382, 48.916336, 40.55811, 49.046337);
		shape56.curveTo(40.59258, 49.176285, 40.736206, 49.347683, 40.98899, 49.559895);
		shape56.curveTo(41.243305, 49.77322, 41.43749, 49.885258, 41.570724, 49.895943);
		shape56.curveTo(41.70431, 49.90692, 41.80783, 49.868168, 41.88211, 49.77974);
		shape56.curveTo(41.955334, 49.692307, 41.97665, 49.591778, 41.9467, 49.477745);
		shape56.curveTo(41.907417, 49.333942, 41.760677, 49.155025, 41.50713, 48.942284);
		shape56.closePath();
		g.fill(shape56);
	}

	private void paintShapeNode_0_0_0_0_6_32(Graphics2D g) {
		GeneralPath shape57 = new GeneralPath();
		shape57.moveTo(116.845, 50.8774);
		shape57.lineTo(117.004074, 51.067005);
		shape57.lineTo(115.79563, 52.081146);
		shape57.curveTo(115.877426, 52.090305, 115.97102, 52.1135, 116.07672, 52.150963);
		shape57.curveTo(116.18277, 52.189014, 116.271255, 52.22806, 116.34237, 52.26887);
		shape57.lineTo(116.15928, 52.4226);
		shape57.curveTo(116.020294, 52.35836, 115.88195, 52.31632, 115.74443, 52.296234);
		shape57.curveTo(115.6068, 52.276917, 115.49076, 52.27803, 115.39605, 52.30111);
		shape57.lineTo(115.2937, 52.17915);
		shape57.lineTo(116.845, 50.8774);
		shape57.closePath();
		shape57.moveTo(115.927216, 49.78375);
		shape57.lineTo(115.55758, 50.09402);
		shape57.lineTo(116.11976, 50.764008);
		shape57.lineTo(115.94572, 50.910217);
		shape57.lineTo(114.353134, 51.045094);
		shape57.lineTo(114.22307, 50.890076);
		shape57.lineTo(115.22406, 50.0501);
		shape57.lineTo(115.04896, 49.841526);
		shape57.lineTo(115.22312, 49.695374);
		shape57.lineTo(115.39816, 49.904003);
		shape57.lineTo(115.76792, 49.59379);
		shape57.lineTo(115.92722, 49.783745);
		shape57.closePath();
		shape57.moveTo(115.383415, 50.240055);
		shape57.lineTo(114.68712, 50.82431);
		shape57.lineTo(115.78899, 50.72343);
		shape57.lineTo(115.383415, 50.240055);
		shape57.closePath();
		shape57.moveTo(114.668335, 49.830486);
		shape57.curveTo(114.4856, 49.98386, 114.32283, 50.088615, 114.17997, 50.144047);
		shape57.curveTo(114.03716, 50.20024, 113.904106, 50.216213, 113.78139, 50.19284);
		shape57.curveTo(113.658844, 50.16941, 113.550446, 50.10165, 113.456314, 49.989494);
		shape57.curveTo(113.38661, 49.90635, 113.342575, 49.819504, 113.32355, 49.728897);
		shape57.curveTo(113.30399, 49.638527, 113.309044, 49.546574, 113.33764, 49.4528);
		shape57.curveTo(113.36653, 49.35938, 113.41627, 49.26302, 113.48714, 49.16437);
		shape57.curveTo(113.55789, 49.065136, 113.669815, 48.95163, 113.82301, 48.823097);
		shape57.curveTo(114.00422, 48.670956, 114.16599, 48.56708, 114.308624, 48.51065);
		shape57.curveTo(114.45143, 48.454456, 114.58414, 48.438133, 114.70721, 48.46115);
		shape57.curveTo(114.83011, 48.484226, 114.9388, 48.55246, 115.03392, 48.66573);
		shape57.curveTo(115.15876, 48.81464, 115.203674, 48.97624, 115.16809, 49.15075);
		shape57.curveTo(115.12511, 49.360203, 114.95829, 49.587208, 114.66833, 49.830482);
		shape57.closePath();
		shape57.moveTo(114.5048, 49.63548);
		shape57.curveTo(114.75841, 49.422737, 114.90204, 49.25134, 114.936325, 49.121452);
		shape57.curveTo(114.97032, 48.99186, 114.95118, 48.883465, 114.8782, 48.796383);
		shape57.curveTo(114.80492, 48.70895, 114.7011, 48.671604, 114.56716, 48.68282);
		shape57.curveTo(114.43317, 48.69421, 114.23945, 48.805897, 113.98654, 49.01793);
		shape57.curveTo(113.732285, 49.231373, 113.58825, 49.403126, 113.5546, 49.532547);
		shape57.curveTo(113.52061, 49.66214, 113.54075, 49.77083, 113.61497, 49.85932);
		shape57.curveTo(113.68837, 49.946697, 113.78367, 49.98504, 113.90117, 49.97535);
		shape57.curveTo(114.049614, 49.961727, 114.25125, 49.84828, 114.50481, 49.635483);
		shape57.closePath();
		g.fill(shape57);
	}

	private void paintShapeNode_0_0_0_0_6_33(Graphics2D g) {
		GeneralPath shape58 = new GeneralPath();
		shape58.moveTo(37.0567, 53.4469);
		shape58.lineTo(37.180367, 53.23263);
		shape58.lineTo(38.546646, 54.021465);
		shape58.curveTo(38.527267, 53.941315, 38.516994, 53.84548, 38.51617, 53.733273);
		shape58.curveTo(38.515583, 53.620647, 38.522102, 53.524113, 38.536137, 53.44326);
		shape58.lineTo(38.74318, 53.562756);
		shape58.curveTo(38.73032, 53.715427, 38.73807, 53.859875, 38.766315, 53.995926);
		shape58.curveTo(38.795204, 54.131863, 38.836014, 54.240494, 38.890038, 54.32164);
		shape58.lineTo(38.810474, 54.459454);
		shape58.lineTo(37.056705, 53.446903);
		shape58.closePath();
		shape58.moveTo(37.19704, 54.260338);
		shape58.lineTo(37.086002, 54.496742);
		shape58.curveTo(36.966625, 54.45, 36.862225, 54.441193, 36.771854, 54.46985);
		shape58.curveTo(36.68137, 54.4988, 36.611492, 54.556107, 36.56193, 54.641838);
		shape58.curveTo(36.502274, 54.745186, 36.490883, 54.855106, 36.527405, 54.971077);
		shape58.curveTo(36.564163, 55.087753, 36.646782, 55.18317, 36.775257, 55.257393);
		shape58.curveTo(36.89763, 55.328033, 37.013775, 55.34952, 37.12417, 55.321632);
		shape58.curveTo(37.23468, 55.2938, 37.321877, 55.224335, 37.386173, 55.112946);
		shape58.curveTo(37.4261, 55.043774, 37.446186, 54.97249, 37.447063, 54.898678);
		shape58.curveTo(37.447887, 54.82481, 37.432323, 54.757927, 37.40056, 54.69727);
		shape58.lineTo(37.542892, 54.511013);
		shape58.lineTo(38.342297, 55.198086);
		shape58.lineTo(37.840775, 56.066605);
		shape58.lineTo(37.635845, 55.948345);
		shape58.lineTo(38.03825, 55.251408);
		shape58.lineTo(37.62346, 54.886116);
		shape58.curveTo(37.636143, 55.033264, 37.609074, 55.16462, 37.542603, 55.27965);
		shape58.curveTo(37.454582, 55.432144, 37.32757, 55.53009, 37.161278, 55.574013);
		shape58.curveTo(36.994984, 55.617817, 36.82887, 55.59186, 36.662987, 55.49615);
		shape58.curveTo(36.504738, 55.404785, 36.394875, 55.279537, 36.332867, 55.121227);
		shape58.curveTo(36.25659, 54.92757, 36.274147, 54.7338, 36.38589, 54.54026);
		shape58.curveTo(36.477726, 54.38125, 36.59681, 54.277138, 36.743843, 54.228107);
		shape58.curveTo(36.89099, 54.179077, 37.04202, 54.18988, 37.197037, 54.260345);
		shape58.closePath();
		shape58.moveTo(36.817303, 55.850338);
		shape58.curveTo(37.023937, 55.969597, 37.178074, 56.0868, 37.278954, 56.202065);
		shape58.curveTo(37.380596, 56.317097, 37.441196, 56.43665, 37.46116, 56.55996);
		shape58.curveTo(37.481125, 56.68315, 37.454464, 56.80811, 37.381184, 56.934998);
		shape58.curveTo(37.326984, 57.02889, 37.2604, 57.1, 37.181774, 57.14891);
		shape58.curveTo(37.10362, 57.198116, 37.015423, 57.224777, 36.917538, 57.23006);
		shape58.curveTo(36.81989, 57.234875, 36.712315, 57.221077, 36.595287, 57.18825);
		shape58.curveTo(36.47791, 57.155663, 36.33299, 57.08931, 36.159767, 56.98931);
		shape58.curveTo(35.954838, 56.87099, 35.801933, 56.754612, 35.70011, 56.63993);
		shape58.curveTo(35.59847, 56.524784, 35.537693, 56.4057, 35.51732, 56.28221);
		shape58.curveTo(35.496944, 56.158844, 35.523777, 56.03336, 35.597763, 55.905293);
		shape58.curveTo(35.695004, 55.736885, 35.831467, 55.63947, 36.00762, 55.613224);
		shape58.curveTo(36.219185, 55.581985, 36.48959, 55.66114, 36.817303, 55.85033);
		shape58.closePath();
		shape58.moveTo(36.69, 56.070827);
		shape58.curveTo(36.40333, 55.905296, 36.193233, 55.828846, 36.059353, 55.841);
		shape58.curveTo(35.925884, 55.85339, 35.830643, 55.908527, 35.77386, 56.00694);
		shape58.curveTo(35.71673, 56.105644, 35.71708, 56.21598, 35.773567, 56.33806);
		shape58.curveTo(35.829937, 56.46002, 36.001163, 56.604, 36.28695, 56.769);
		shape58.curveTo(36.574554, 56.935, 36.785183, 57.01157, 36.91824, 56.998886);
		shape58.curveTo(37.05171, 56.986496, 37.146893, 56.93042, 37.204613, 56.83036);
		shape58.curveTo(37.26169, 56.731537, 37.26521, 56.628838, 37.215885, 56.52179);
		shape58.curveTo(37.152176, 56.38697, 36.976723, 56.236298, 36.689995, 56.070824);
		shape58.closePath();
		g.fill(shape58);
	}

	private void paintShapeNode_0_0_0_0_6_34(Graphics2D g) {
		GeneralPath shape59 = new GeneralPath();
		shape59.moveTo(121.323, 57.6821);
		shape59.lineTo(121.44672, 57.896305);
		shape59.lineTo(120.08044, 58.68514);
		shape59.curveTo(120.15953, 58.70845, 120.247665, 58.74744, 120.34526, 58.80287);
		shape59.curveTo(120.44303, 58.858654, 120.523415, 58.91256, 120.58642, 58.96517);
		shape59.lineTo(120.37937, 59.084724);
		shape59.curveTo(120.253654, 58.997234, 120.12465, 58.93176, 119.99271, 58.88819);
		shape59.curveTo(119.86053, 58.845207, 119.746086, 58.82624, 119.64879, 58.832466);
		shape59.lineTo(119.56917, 58.694653);
		shape59.lineTo(121.323, 57.682102);
		shape59.closePath();
		shape59.moveTo(120.54837, 57.3969);
		shape59.lineTo(120.39916, 57.182575);
		shape59.curveTo(120.49934, 57.10254, 120.55911, 57.016518, 120.57949, 56.923977);
		shape59.curveTo(120.59969, 56.83108, 120.58501, 56.741947, 120.53551, 56.65616);
		shape59.curveTo(120.475845, 56.55281, 120.38636, 56.487988, 120.26763, 56.46168);
		shape59.curveTo(120.148254, 56.43514, 120.02424, 56.45898, 119.89576, 56.533142);
		shape59.curveTo(119.77339, 56.60384, 119.69676, 56.693623, 119.665695, 56.803135);
		shape59.curveTo(119.634514, 56.912766, 119.65108, 57.023098, 119.71543, 57.13443);
		shape59.curveTo(119.75536, 57.2036, 119.80703, 57.256626, 119.87051, 57.294323);
		shape59.curveTo(119.934044, 57.3319, 119.99975, 57.351868, 120.068214, 57.354744);
		shape59.lineTo(120.15835, 57.571125);
		shape59.lineTo(119.16364, 57.919918);
		shape59.lineTo(118.66218, 57.051285);
		shape59.lineTo(118.86711, 56.932964);
		shape59.lineTo(119.269516, 57.6299);
		shape59.lineTo(119.793175, 57.45333);
		shape59.curveTo(119.65942, 57.390736, 119.55918, 57.30166, 119.49283, 57.18657);
		shape59.curveTo(119.40475, 57.034077, 119.38344, 56.875122, 119.42859, 56.70912);
		shape59.curveTo(119.47375, 56.543182, 119.57932, 56.412354, 119.74515, 56.316525);
		shape59.curveTo(119.903336, 56.22516, 120.06681, 56.192627, 120.234924, 56.21811);
		shape59.curveTo(120.440735, 56.24888, 120.59975, 56.360973, 120.71155, 56.554512);
		shape59.curveTo(120.80333, 56.713524, 120.83398, 56.868717, 120.80292, 57.02057);
		shape59.curveTo(120.771736, 57.172478, 120.68689, 57.2979, 120.54837, 57.3969);
		shape59.closePath();
		shape59.moveTo(119.36124, 56.27307);
		shape59.curveTo(119.15466, 56.392387, 118.9761, 56.467255, 118.825775, 56.497025);
		shape59.curveTo(118.67539, 56.5275, 118.54151, 56.520218, 118.42478, 56.475887);
		shape59.curveTo(118.308105, 56.431553, 118.21322, 56.346, 118.13994, 56.21911);
		shape59.curveTo(118.08574, 56.125217, 118.057434, 56.03197, 118.05444, 55.93943);
		shape59.curveTo(118.050865, 55.847122, 118.071884, 55.757458, 118.11627, 55.670025);
		shape59.curveTo(118.1609, 55.58306, 118.22666, 55.496742, 118.31357, 55.411835);
		shape59.curveTo(118.400475, 55.326458, 118.53042, 55.23415, 118.70364, 55.13415);
		shape59.curveTo(118.90858, 55.01583, 119.085846, 54.94155, 119.23605, 54.910725);
		shape59.curveTo(119.38655, 54.880306, 119.52008, 54.887177, 119.63722, 54.93128);
		shape59.curveTo(119.75425, 54.97532, 119.849495, 55.06134, 119.92342, 55.189407);
		shape59.curveTo(120.02066, 55.357815, 120.03675, 55.524754, 119.9714, 55.69046);
		shape59.curveTo(119.892715, 55.889225, 119.68896, 56.083878, 119.361244, 56.27307);
		shape59.closePath();
		shape59.moveTo(119.23393, 56.05258);
		shape59.curveTo(119.5206, 55.88705, 119.69189, 55.74331, 119.74825, 55.62129);
		shape59.curveTo(119.80427, 55.499565, 119.804214, 55.389523, 119.747375, 55.29111);
		shape59.curveTo(119.690414, 55.192287, 119.5947, 55.137444, 119.46077, 55.125286);
		shape59.curveTo(119.32689, 55.113132, 119.11668, 55.189465, 118.83083, 55.354465);
		shape59.curveTo(118.54329, 55.520523, 118.371704, 55.66462, 118.31616, 55.78623);
		shape59.curveTo(118.26015, 55.90795, 118.261086, 56.01852, 118.31886, 56.11846);
		shape59.curveTo(118.37588, 56.217346, 118.463135, 56.271717, 118.58046, 56.282524);
		shape59.curveTo(118.72908, 56.294796, 118.94728, 56.21811, 119.23395, 56.052578);
		shape59.closePath();
		g.fill(shape59);
	}

	private void paintShapeNode_0_0_0_0_6_35(Graphics2D g) {
		GeneralPath shape60 = new GeneralPath();
		shape60.moveTo(33.1022, 61.2821);
		shape60.lineTo(33.186874, 61.04963);
		shape60.lineTo(34.669304, 61.5892);
		shape60.curveTo(34.636364, 61.513626, 34.609646, 61.421085, 34.589386, 61.310753);
		shape60.curveTo(34.569187, 61.19995, 34.558735, 61.10371, 34.5585, 61.02168);
		shape60.lineTo(34.78322, 61.103416);
		shape60.curveTo(34.797077, 61.25585, 34.82984, 61.396835, 34.88122, 61.5259);
		shape60.curveTo(34.933365, 61.65473, 34.99238, 61.754673, 35.059612, 61.825134);
		shape60.lineTo(35.005238, 61.974808);
		shape60.lineTo(33.102207, 61.282097);
		shape60.closePath();
		shape60.moveTo(33.876823, 63.60609);
		shape60.lineTo(33.942825, 63.368393);
		shape60.curveTo(34.04171, 63.3809, 34.118923, 63.37597, 34.17371, 63.35295);
		shape60.curveTo(34.263317, 63.31408, 34.324738, 63.248547, 34.358383, 63.1563);
		shape60.curveTo(34.385334, 63.082195, 34.388035, 63.009853, 34.367073, 62.93857);
		shape60.curveTo(34.340298, 62.84591, 34.28328, 62.75977, 34.19626, 62.680145);
		shape60.curveTo(34.109238, 62.60099, 33.971188, 62.525833, 33.782112, 62.454956);
		shape60.curveTo(33.846878, 62.54186, 33.885517, 62.633404, 33.897377, 62.729763);
		shape60.curveTo(33.90906, 62.82536, 33.898785, 62.918133, 33.86655, 63.00686);
		shape60.curveTo(33.810062, 63.162052, 33.70513, 63.273384, 33.55111, 63.340736);
		shape60.curveTo(33.397205, 63.408028, 33.229855, 63.409264, 33.049118, 63.34338);
		shape60.curveTo(32.929916, 63.300045, 32.829098, 63.23375, 32.74536, 63.14532);
		shape60.curveTo(32.662098, 63.057064, 32.60925, 62.95783, 32.587524, 62.848553);
		shape60.curveTo(32.565857, 62.739334, 32.57596, 62.628002, 32.617355, 62.514145);
		shape60.curveTo(32.688053, 62.320076, 32.816532, 62.1879, 33.003963, 62.11773);
		shape60.curveTo(33.191044, 62.046974, 33.44841, 62.071636, 33.77583, 62.190777);
		shape60.curveTo(34.141945, 62.32407, 34.383514, 62.488483, 34.50054, 62.68443);
		shape60.curveTo(34.60236, 62.854893, 34.617393, 63.040268, 34.544464, 63.24056);
		shape60.curveTo(34.490326, 63.389294, 34.404007, 63.496162, 34.285397, 63.56099);
		shape60.curveTo(34.167313, 63.625874, 34.031082, 63.64073, 33.876827, 63.606087);
		shape60.closePath();
		shape60.moveTo(33.405838, 62.35889);
		shape60.curveTo(33.325512, 62.329647, 33.24248, 62.318962, 33.156925, 62.326244);
		shape60.curveTo(33.071255, 62.333527, 32.998383, 62.361008, 32.9379, 62.407925);
		shape60.curveTo(32.877125, 62.455254, 32.8342, 62.51262, 32.809834, 62.57974);
		shape60.curveTo(32.773956, 62.678154, 32.78306, 62.77686, 32.83667, 62.87639);
		shape60.curveTo(32.890457, 62.9758, 32.985287, 63.050373, 33.121166, 63.099815);
		shape60.curveTo(33.25211, 63.147495, 33.368137, 63.150196, 33.46896, 63.10798);
		shape60.curveTo(33.569782, 63.06576, 33.639362, 62.991596, 33.677883, 62.88567);
		shape60.curveTo(33.71623, 62.78027, 33.711117, 62.677334, 33.662323, 62.576748);
		shape60.curveTo(33.61347, 62.475986, 33.52803, 62.40335, 33.405838, 62.3589);
		shape60.closePath();
		shape60.moveTo(33.28388, 63.69064);
		shape60.curveTo(33.508068, 63.7722, 33.680176, 63.86081, 33.799553, 63.956757);
		shape60.curveTo(33.919636, 64.05247, 34.00002, 64.15969, 34.041122, 64.277596);
		shape60.curveTo(34.08217, 64.3955, 34.077644, 64.52316, 34.027557, 64.6608);
		shape60.curveTo(33.990448, 64.76273, 33.937305, 64.84429, 33.86825, 64.90607);
		shape60.curveTo(33.799786, 64.96813, 33.717636, 65.009766, 33.6221, 65.03196);
		shape60.curveTo(33.52674, 65.05363, 33.418465, 65.05868, 33.29756, 65.0467);
		shape60.curveTo(33.176247, 65.03501, 33.02205, 64.99479, 32.83409, 64.926445);
		shape60.curveTo(32.611782, 64.845474, 32.440907, 64.757454, 32.32071, 64.662094);
		shape60.curveTo(32.200626, 64.5665, 32.120064, 64.45975, 32.07861, 64.34166);
		shape60.curveTo(32.037094, 64.223694, 32.041794, 64.095505, 32.092407, 63.956463);
		shape60.curveTo(32.15888, 63.77379, 32.276375, 63.654175, 32.44525, 63.597687);
		shape60.curveTo(32.648186, 63.53022, 32.92822, 63.561165, 33.28388, 63.69064);
		shape60.closePath();
		shape60.moveTo(33.196857, 63.929802);
		shape60.curveTo(32.88576, 63.81659, 32.66551, 63.777893, 32.535797, 63.813126);
		shape60.curveTo(32.406555, 63.848476, 32.322178, 63.91923, 32.283363, 64.025925);
		shape60.curveTo(32.244316, 64.13315, 32.26381, 64.241776, 32.340557, 64.35205);
		shape60.curveTo(32.41742, 64.46244, 32.61096, 64.57448, 32.921055, 64.68734);
		shape60.curveTo(33.23309, 64.80084, 33.453754, 64.839714, 33.582645, 64.80413);
		shape60.curveTo(33.711945, 64.76884, 33.79597, 64.69697, 33.83549, 64.588455);
		shape60.curveTo(33.87454, 64.48123, 33.860035, 64.37953, 33.79298, 64.28271);
		shape60.curveTo(33.70684, 64.16093, 33.507896, 64.043015, 33.19686, 63.929806);
		shape60.closePath();
		g.fill(shape60);
	}

	private void paintShapeNode_0_0_0_0_6_36(Graphics2D g) {
		GeneralPath shape61 = new GeneralPath();
		shape61.moveTo(124.554, 65.1769);
		shape61.lineTo(124.63856, 65.40943);
		shape61.lineTo(123.15613, 65.949005);
		shape61.curveTo(123.229935, 65.9857, 123.30991, 66.03944, 123.39635, 66.11089);
		shape61.curveTo(123.48302, 66.18282, 123.552895, 66.24976, 123.6058, 66.31254);
		shape61.lineTo(123.38108, 66.39433);
		shape61.curveTo(123.27251, 66.28647, 123.15683, 66.19956, 123.03446, 66.133736);
		shape61.curveTo(122.911736, 66.0685, 122.802284, 66.02986, 122.70546, 66.01912);
		shape61.lineTo(122.65097, 65.86956);
		shape61.lineTo(124.554, 65.17691);
		shape61.closePath();
		shape61.moveTo(122.466774, 63.89453);
		shape61.lineTo(122.569, 64.11902);
		shape61.curveTo(122.48521, 64.173096, 122.429245, 64.22642, 122.40206, 64.27932);
		shape61.curveTo(122.358376, 64.36669, 122.3535, 64.45636, 122.38703, 64.54867);
		shape61.curveTo(122.41399, 64.62271, 122.458435, 64.67991, 122.520325, 64.72101);
		shape61.curveTo(122.60036, 64.774796, 122.699356, 64.80415, 122.81727, 64.809204);
		shape61.curveTo(122.93482, 64.8139, 123.088905, 64.78272, 123.279274, 64.715485);
		shape61.curveTo(123.17375, 64.69053, 123.08533, 64.645256, 123.014336, 64.57908);
		shape61.curveTo(122.94393, 64.513374, 122.8922, 64.435684, 122.85985, 64.34696);
		shape61.curveTo(122.80336, 64.191765, 122.812164, 64.03904, 122.886856, 63.888424);
		shape61.curveTo(122.96149, 63.737984, 123.088905, 63.62953, 123.26977, 63.563766);
		shape61.curveTo(123.38891, 63.520374, 123.50876, 63.50634, 123.629715, 63.520256);
		shape61.curveTo(123.750206, 63.53429, 123.85443, 63.576393, 123.9414, 63.64615);
		shape61.curveTo(124.02818, 63.715847, 124.09201, 63.807625, 124.13347, 63.921482);
		shape61.curveTo(124.20411, 64.11555, 124.190605, 64.2994, 124.09213, 64.47362);
		shape61.curveTo(123.99431, 64.64813, 123.781334, 64.79463, 123.45391, 64.91378);
		shape61.curveTo(123.08774, 65.04707, 122.79702, 65.07638, 122.58146, 65.00151);
		shape61.curveTo(122.39385, 64.93633, 122.263145, 64.80404, 122.19033, 64.603745);
		shape61.curveTo(122.13619, 64.45501, 122.133606, 64.317604, 122.182816, 64.19177);
		shape61.curveTo(122.23155, 64.06612, 122.326385, 63.96717, 122.46678, 63.894535);
		shape61.closePath();
		shape61.moveTo(123.62924, 64.54726);
		shape61.curveTo(123.70957, 64.51795, 123.78009, 64.4728, 123.84093, 64.4122);
		shape61.curveTo(123.90188, 64.35155, 123.94005, 64.28372, 123.95619, 64.208855);
		shape61.curveTo(123.97234, 64.133575, 123.968346, 64.062, 123.94392, 63.99494);
		shape61.curveTo(123.9081, 63.896526, 123.83763, 63.82671, 123.73264, 63.78496);
		shape61.curveTo(123.62748, 63.74339, 123.50693, 63.747204, 123.37111, 63.796707);
		shape61.curveTo(123.240105, 63.84433, 123.149506, 63.91679, 123.09942, 64.01397);
		shape61.curveTo(123.04933, 64.11109, 123.04369, 64.21268, 123.082275, 64.31855);
		shape61.curveTo(123.120674, 64.42395, 123.19073, 64.49952, 123.292786, 64.545204);
		shape61.curveTo(123.3949, 64.591, 123.50711, 64.59177, 123.62925, 64.54726);
		shape61.closePath();
		shape61.moveTo(122.86665, 63.448677);
		shape61.curveTo(122.64246, 63.530296, 122.45368, 63.572987, 122.300545, 63.576275);
		shape61.curveTo(122.14705, 63.580093, 122.01658, 63.549618, 121.9093, 63.48579);
		shape61.curveTo(121.80208, 63.421844, 121.72346, 63.32114, 121.67337, 63.183502);
		shape61.curveTo(121.63626, 63.081566, 121.62452, 62.984913, 121.637726, 62.893196);
		shape61.curveTo(121.65029, 62.801594, 121.68646, 62.716923, 121.745415, 62.63853);
		shape61.curveTo(121.80449, 62.56067, 121.88417, 62.487152, 121.98452, 62.418625);
		shape61.curveTo(122.08493, 62.34963, 122.22891, 62.28128, 122.41687, 62.21287);
		shape61.curveTo(122.63918, 62.131958, 122.82673, 62.08956, 122.98005, 62.085335);
		shape61.curveTo(123.133545, 62.0814, 123.26384, 62.111347, 123.371475, 62.175175);
		shape61.curveTo(123.47911, 62.238884, 123.55791, 62.34012, 123.60853, 62.479164);
		shape61.curveTo(123.674995, 62.66184, 123.6619, 62.828957, 123.56883, 62.980804);
		shape61.curveTo(123.45673, 63.162952, 123.22233, 63.319202, 122.86667, 63.44868);
		shape61.closePath();
		shape61.moveTo(122.77957, 63.209454);
		shape61.curveTo(123.09061, 63.09624, 123.28426, 62.984325, 123.360954, 62.87399);
		shape61.curveTo(123.43729, 62.763832, 123.45643, 62.655376, 123.41756, 62.548683);
		shape61.curveTo(123.37857, 62.441463, 123.29384, 62.370766, 123.16412, 62.33559);
		shape61.curveTo(123.034294, 62.30042, 122.81398, 62.339054, 122.503944, 62.451855);
		shape61.curveTo(122.19191, 62.565475, 121.9979, 62.67763, 121.922035, 62.78767);
		shape61.curveTo(121.8457, 62.897827, 121.8275, 63.006927, 121.86696, 63.11544);
		shape61.curveTo(121.906006, 63.22266, 121.98246, 63.291306, 122.096085, 63.32237);
		shape61.curveTo(122.24036, 63.36024, 122.468544, 63.32272, 122.77958, 63.20945);
		shape61.closePath();
		g.fill(shape61);
	}

	private void paintShapeNode_0_0_0_0_6_37(Graphics2D g) {
		GeneralPath shape62 = new GeneralPath();
		shape62.moveTo(30.5902, 69.7766);
		shape62.lineTo(30.633123, 69.532974);
		shape62.lineTo(32.186714, 69.8069);
		shape62.curveTo(32.14121, 69.7382, 32.098812, 69.65165, 32.059647, 69.54654);
		shape62.curveTo(32.02054, 69.4409, 31.993587, 69.34801, 31.979084, 69.26721);
		shape62.lineTo(32.214607, 69.308784);
		shape62.curveTo(32.25471, 69.45652, 32.311436, 69.58958, 32.384483, 69.707726);
		shape62.curveTo(32.458176, 69.82563, 32.53363, 69.91377, 32.612198, 69.97149);
		shape62.lineTo(32.58454, 70.12833);
		shape62.lineTo(30.590202, 69.776596);
		shape62.closePath();
		shape62.moveTo(32.202812, 70.72175);
		shape62.lineTo(32.43728, 70.763084);
		shape62.lineTo(32.21062, 72.04856);
		shape62.lineTo(32.021015, 72.01515);
		shape62.curveTo(31.908745, 71.86524, 31.751787, 71.70828, 31.550966, 71.54458);
		shape62.curveTo(31.34991, 71.3814, 31.136995, 71.2447, 30.913628, 71.13572);
		shape62.curveTo(30.75579, 71.05733, 30.579515, 70.994385, 30.383923, 70.94594);
		shape62.lineTo(30.428139, 70.69538);
		shape62.curveTo(30.58398, 70.72574, 30.767242, 70.78945, 30.978397, 70.887276);
		shape62.curveTo(31.189024, 70.984985, 31.387789, 71.106766, 31.57381, 71.252625);
		shape62.curveTo(31.759716, 71.39872, 31.912504, 71.54598, 32.031292, 71.6946);
		shape62.lineTo(32.202812, 70.72174);
		shape62.closePath();
		shape62.moveTo(31.187323, 72.11698);
		shape62.curveTo(31.4222, 72.15838, 31.607048, 72.215805, 31.741457, 72.28956);
		shape62.curveTo(31.876276, 72.36296, 31.974045, 72.45456, 32.034996, 72.5636);
		shape62.curveTo(32.09583, 72.67246, 32.113564, 72.799065, 32.08814, 72.94334);
		shape62.curveTo(32.06923, 73.05015, 32.03112, 73.139755, 31.973812, 73.21251);
		shape62.curveTo(31.917206, 73.285614, 31.843573, 73.34081, 31.75332, 73.37921);
		shape62.curveTo(31.663187, 73.417145, 31.557491, 73.440926, 31.436178, 73.45009);
		shape62.curveTo(31.314806, 73.45972, 31.155851, 73.446915, 30.958908, 73.412155);
		shape62.curveTo(30.72597, 73.37105, 30.542355, 73.31398, 30.407417, 73.24099);
		shape62.curveTo(30.272655, 73.16771, 30.174713, 73.07658, 30.11341, 72.967476);
		shape62.curveTo(30.05205, 72.85849, 30.034374, 72.73137, 30.060152, 72.585625);
		shape62.curveTo(30.093857, 72.39432, 30.188807, 72.256035, 30.345293, 72.17118);
		shape62.curveTo(30.533487, 72.06936, 30.814577, 72.05128, 31.187326, 72.11698);
		shape62.closePath();
		shape62.moveTo(31.143106, 72.36765);
		shape62.curveTo(30.817097, 72.310104, 30.593494, 72.31023, 30.471886, 72.36754);
		shape62.curveTo(30.350689, 72.42473, 30.279932, 72.50911, 30.260262, 72.62097);
		shape62.curveTo(30.240414, 72.73336, 30.278406, 72.83688, 30.373178, 72.93224);
		shape62.curveTo(30.46801, 73.02754, 30.678108, 73.10423, 31.00306, 73.1616);
		shape62.curveTo(31.330067, 73.2192, 31.55414, 73.21915, 31.674866, 73.16171);
		shape62.curveTo(31.796062, 73.1044, 31.866407, 73.0192, 31.886374, 72.9054);
		shape62.curveTo(31.906221, 72.793076, 31.874395, 72.695366, 31.791424, 72.61157);
		shape62.curveTo(31.685436, 72.5067, 31.468996, 72.42514, 31.143105, 72.36765);
		shape62.closePath();
		g.fill(shape62);
	}

	private void paintShapeNode_0_0_0_0_6_38(Graphics2D g) {
		GeneralPath shape63 = new GeneralPath();
		shape63.moveTo(126.43, 73.1377);
		shape63.lineTo(126.472984, 73.381386);
		shape63.lineTo(124.919395, 73.65537);
		shape63.curveTo(124.985634, 73.704346, 125.05509, 73.771164, 125.127846, 73.856544);
		shape63.curveTo(125.200714, 73.94239, 125.25785, 74.020485, 125.29907, 74.09142);
		shape63.lineTo(125.06355, 74.132935);
		shape63.curveTo(124.97536, 74.007866, 124.87653, 73.90223, 124.76749, 73.81615);
		shape63.curveTo(124.65791, 73.73054, 124.55686, 73.673515, 124.46332, 73.64621);
		shape63.lineTo(124.43566, 73.48937);
		shape63.lineTo(126.43, 73.1377);
		shape63.closePath();
		shape63.moveTo(124.59138, 72.801125);
		shape63.lineTo(124.35691, 72.84252);
		shape63.lineTo(124.13026, 71.556984);
		shape63.lineTo(124.3198, 71.523575);
		shape63.curveTo(124.47664, 71.62604, 124.67776, 71.71987, 124.92244, 71.805016);
		shape63.curveTo(125.16718, 71.88963, 125.41404, 71.945175, 125.66125, 71.97119);
		shape63.curveTo(125.83635, 71.99086, 126.023544, 71.989746, 126.22384, 71.968376);
		shape63.lineTo(126.268, 72.21893);
		shape63.curveTo(126.11122, 72.24371, 125.917206, 72.24659, 125.685326, 72.22686);
		shape63.curveTo(125.45397, 72.20707, 125.22561, 72.16062, 125.00084, 72.087166);
		shape63.curveTo(124.77624, 72.01341, 124.58223, 71.92733, 124.419815, 71.82833);
		shape63.lineTo(124.59139, 72.80113);
		shape63.closePath();
		shape63.moveTo(125.06841, 71.14278);
		shape63.curveTo(124.833534, 71.18417, 124.640175, 71.19345, 124.48868, 71.17008);
		shape63.curveTo(124.33689, 71.14724, 124.2137, 71.09457, 124.1191, 71.01301);
		shape63.curveTo(124.02474, 70.93145, 123.96473, 70.81859, 123.93931, 70.67438);
		shape63.curveTo(123.92052, 70.567505, 123.92568, 70.47027, 123.954636, 70.38225);
		shape63.curveTo(123.98288, 70.29423, 124.0332, 70.217186, 124.10484, 70.15019);
		shape63.curveTo(124.1766, 70.083725, 124.26779, 70.02524, 124.378654, 69.97515);
		shape63.curveTo(124.4894, 69.92459, 124.64313, 69.882256, 124.84007, 69.84756);
		shape63.curveTo(125.073074, 69.80651, 125.26508, 69.79741, 125.41687, 69.819786);
		shape63.curveTo(125.5686, 69.84257, 125.691795, 69.894714, 125.78668, 69.97627);
		shape63.curveTo(125.88163, 70.05772, 125.9417, 70.171104, 125.96736, 70.31684);
		shape63.curveTo(126.00112, 70.50821, 125.9592, 70.67056, 125.84112, 70.803856);
		shape63.curveTo(125.699135, 70.96387, 125.44118, 71.07702, 125.06843, 71.142784);
		shape63.closePath();
		shape63.moveTo(125.024254, 70.892105);
		shape63.curveTo(125.350266, 70.83462, 125.56036, 70.75805, 125.655014, 70.66263);
		shape63.curveTo(125.74931, 70.567444, 125.78696, 70.46392, 125.767166, 70.35206);
		shape63.curveTo(125.747375, 70.23967, 125.67627, 70.15541, 125.554665, 70.09821);
		shape63.curveTo(125.43294, 70.04108, 125.20922, 70.0409, 124.88427, 70.0981);
		shape63.curveTo(124.55732, 70.15582, 124.346756, 70.23257, 124.252975, 70.32775);
		shape63.curveTo(124.158676, 70.42305, 124.12168, 70.52722, 124.14188, 70.64096);
		shape63.curveTo(124.161606, 70.75329, 124.22496, 70.83427, 124.33154, 70.88459);
		shape63.curveTo(124.467064, 70.94689, 124.6983, 70.94953, 125.024254, 70.892105);
		shape63.closePath();
		g.fill(shape63);
	}

	private void paintCompositeGraphicsNode_0_0_0_0_6(Graphics2D g) {
		// _0_0_0_0_6_0
		AffineTransform trans_0_0_0_0_6_0 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_0(g);
		g.setTransform(trans_0_0_0_0_6_0);
		// _0_0_0_0_6_1
		AffineTransform trans_0_0_0_0_6_1 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_1(g);
		g.setTransform(trans_0_0_0_0_6_1);
		// _0_0_0_0_6_2
		AffineTransform trans_0_0_0_0_6_2 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintCompositeGraphicsNode_0_0_0_0_6_2(g);
		g.setTransform(trans_0_0_0_0_6_2);
		// _0_0_0_0_6_3
		AffineTransform trans_0_0_0_0_6_3 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_3(g);
		g.setTransform(trans_0_0_0_0_6_3);
		// _0_0_0_0_6_4
		AffineTransform trans_0_0_0_0_6_4 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_4(g);
		g.setTransform(trans_0_0_0_0_6_4);
		// _0_0_0_0_6_5
		AffineTransform trans_0_0_0_0_6_5 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_5(g);
		g.setTransform(trans_0_0_0_0_6_5);
		// _0_0_0_0_6_6
		AffineTransform trans_0_0_0_0_6_6 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_6(g);
		g.setTransform(trans_0_0_0_0_6_6);
		// _0_0_0_0_6_7
		AffineTransform trans_0_0_0_0_6_7 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_7(g);
		g.setTransform(trans_0_0_0_0_6_7);
		// _0_0_0_0_6_8
		AffineTransform trans_0_0_0_0_6_8 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_8(g);
		g.setTransform(trans_0_0_0_0_6_8);
		// _0_0_0_0_6_9
		AffineTransform trans_0_0_0_0_6_9 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_9(g);
		g.setTransform(trans_0_0_0_0_6_9);
		// _0_0_0_0_6_10
		AffineTransform trans_0_0_0_0_6_10 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_10(g);
		g.setTransform(trans_0_0_0_0_6_10);
		// _0_0_0_0_6_11
		AffineTransform trans_0_0_0_0_6_11 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_11(g);
		g.setTransform(trans_0_0_0_0_6_11);
		// _0_0_0_0_6_12
		AffineTransform trans_0_0_0_0_6_12 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_12(g);
		g.setTransform(trans_0_0_0_0_6_12);
		// _0_0_0_0_6_13
		AffineTransform trans_0_0_0_0_6_13 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_13(g);
		g.setTransform(trans_0_0_0_0_6_13);
		// _0_0_0_0_6_14
		AffineTransform trans_0_0_0_0_6_14 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_14(g);
		g.setTransform(trans_0_0_0_0_6_14);
		// _0_0_0_0_6_15
		AffineTransform trans_0_0_0_0_6_15 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_15(g);
		g.setTransform(trans_0_0_0_0_6_15);
		// _0_0_0_0_6_16
		AffineTransform trans_0_0_0_0_6_16 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_16(g);
		g.setTransform(trans_0_0_0_0_6_16);
		// _0_0_0_0_6_17
		AffineTransform trans_0_0_0_0_6_17 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_17(g);
		g.setTransform(trans_0_0_0_0_6_17);
		// _0_0_0_0_6_18
		AffineTransform trans_0_0_0_0_6_18 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_18(g);
		g.setTransform(trans_0_0_0_0_6_18);
		// _0_0_0_0_6_19
		AffineTransform trans_0_0_0_0_6_19 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_19(g);
		g.setTransform(trans_0_0_0_0_6_19);
		// _0_0_0_0_6_20
		AffineTransform trans_0_0_0_0_6_20 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_20(g);
		g.setTransform(trans_0_0_0_0_6_20);
		// _0_0_0_0_6_21
		AffineTransform trans_0_0_0_0_6_21 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_21(g);
		g.setTransform(trans_0_0_0_0_6_21);
		// _0_0_0_0_6_22
		AffineTransform trans_0_0_0_0_6_22 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_22(g);
		g.setTransform(trans_0_0_0_0_6_22);
		// _0_0_0_0_6_23
		AffineTransform trans_0_0_0_0_6_23 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_23(g);
		g.setTransform(trans_0_0_0_0_6_23);
		// _0_0_0_0_6_24
		AffineTransform trans_0_0_0_0_6_24 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_24(g);
		g.setTransform(trans_0_0_0_0_6_24);
		// _0_0_0_0_6_25
		AffineTransform trans_0_0_0_0_6_25 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_25(g);
		g.setTransform(trans_0_0_0_0_6_25);
		// _0_0_0_0_6_26
		AffineTransform trans_0_0_0_0_6_26 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_26(g);
		g.setTransform(trans_0_0_0_0_6_26);
		// _0_0_0_0_6_27
		AffineTransform trans_0_0_0_0_6_27 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_27(g);
		g.setTransform(trans_0_0_0_0_6_27);
		// _0_0_0_0_6_28
		AffineTransform trans_0_0_0_0_6_28 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_28(g);
		g.setTransform(trans_0_0_0_0_6_28);
		// _0_0_0_0_6_29
		AffineTransform trans_0_0_0_0_6_29 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_29(g);
		g.setTransform(trans_0_0_0_0_6_29);
		// _0_0_0_0_6_30
		AffineTransform trans_0_0_0_0_6_30 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_30(g);
		g.setTransform(trans_0_0_0_0_6_30);
		// _0_0_0_0_6_31
		AffineTransform trans_0_0_0_0_6_31 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_31(g);
		g.setTransform(trans_0_0_0_0_6_31);
		// _0_0_0_0_6_32
		AffineTransform trans_0_0_0_0_6_32 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_32(g);
		g.setTransform(trans_0_0_0_0_6_32);
		// _0_0_0_0_6_33
		AffineTransform trans_0_0_0_0_6_33 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_33(g);
		g.setTransform(trans_0_0_0_0_6_33);
		// _0_0_0_0_6_34
		AffineTransform trans_0_0_0_0_6_34 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_34(g);
		g.setTransform(trans_0_0_0_0_6_34);
		// _0_0_0_0_6_35
		AffineTransform trans_0_0_0_0_6_35 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_35(g);
		g.setTransform(trans_0_0_0_0_6_35);
		// _0_0_0_0_6_36
		AffineTransform trans_0_0_0_0_6_36 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_36(g);
		g.setTransform(trans_0_0_0_0_6_36);
		// _0_0_0_0_6_37
		AffineTransform trans_0_0_0_0_6_37 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_37(g);
		g.setTransform(trans_0_0_0_0_6_37);
		// _0_0_0_0_6_38
		AffineTransform trans_0_0_0_0_6_38 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_6_38(g);
		g.setTransform(trans_0_0_0_0_6_38);
	}

	private void paintShapeNode_0_0_0_0_7(Graphics2D g) {
		GeneralPath shape64 = new GeneralPath();
		shape64.moveTo(79.0951, 17.7881);
		shape64.curveTo(79.10103, 17.785164, 79.105965, 17.779232, 79.1089, 17.772305);
		shape64.curveTo(79.1119, 17.765434, 79.11384, 17.759504, 79.11683, 17.752575);
		shape64.curveTo(79.11977, 17.745646, 79.124695, 17.739716, 79.13163, 17.734783);
		shape64.curveTo(79.13856, 17.72985, 79.14842, 17.727913, 79.162224, 17.727913);
		shape64.curveTo(79.17015, 17.76244, 79.18788, 17.794031, 79.21754, 17.821688);
		shape64.curveTo(79.24613, 17.849344, 79.278725, 17.874006, 79.317245, 17.893736);
		shape64.curveTo(79.35477, 17.914463, 79.394226, 17.932255, 79.43668, 17.948051);
		shape64.curveTo(79.47914, 17.962849, 79.5186, 17.975708, 79.555176, 17.985573);
		shape64.curveTo(79.57684, 17.99444, 79.59763, 18.001368, 79.61736, 18.005302);
		shape64.curveTo(79.63809, 18.009237, 79.65582, 18.017164, 79.67068, 18.029024);
		shape64.curveTo(79.68841, 18.035894, 79.711136, 18.041824, 79.73779, 18.044762);
		shape64.curveTo(79.76445, 18.047756, 79.789116, 18.050692, 79.809845, 18.054686);
		shape64.curveTo(79.86416, 18.062553, 79.907616, 18.073418, 79.94014, 18.088215);
		shape64.curveTo(79.97273, 18.102072, 80.011246, 18.117868, 80.05564, 18.135601);
		shape64.curveTo(80.13661, 18.16226, 80.21759, 18.18892, 80.29751, 18.213581);
		shape64.curveTo(80.37749, 18.238302, 80.45546, 18.265959, 80.5335, 18.294556);
		shape64.curveTo(80.564095, 18.31035, 80.601616, 18.328142, 80.647, 18.34981);
		shape64.curveTo(80.69239, 18.370596, 80.72498, 18.389328, 80.743774, 18.404125);
		shape64.curveTo(80.757576, 18.417982, 80.77537, 18.429785, 80.7961, 18.440649);
		shape64.curveTo(80.817825, 18.451511, 80.83949, 18.456444, 80.86321, 18.456444);
		shape64.curveTo(80.87208, 18.456444, 80.88594, 18.457441, 80.90367, 18.459438);
		shape64.curveTo(80.92046, 18.461376, 80.93333, 18.462374, 80.94119, 18.462374);
		shape64.curveTo(80.94319, 18.462374, 80.94712, 18.463371, 80.954056, 18.463371);
		shape64.curveTo(80.96092, 18.464369, 80.96886, 18.465368, 80.97872, 18.465368);
		shape64.curveTo(80.98759, 18.467306, 80.99945, 18.468304, 81.01325, 18.468304);
		shape64.lineTo(81.03891, 18.468304);
		shape64.curveTo(81.06463, 18.468304, 81.09322, 18.465368, 81.12582, 18.459436);
		shape64.curveTo(81.15841, 18.453506, 81.190994, 18.446577, 81.22259, 18.439648);
		shape64.curveTo(81.25412, 18.43178, 81.284775, 18.422913, 81.31437, 18.414986);
		shape64.curveTo(81.343025, 18.406118, 81.36768, 18.398191, 81.38941, 18.390324);
		shape64.curveTo(81.402214, 18.386389, 81.424934, 18.379461, 81.45553, 18.368597);
		shape64.curveTo(81.48612, 18.357735, 81.51871, 18.34687, 81.5523, 18.337006);
		shape64.curveTo(81.58583, 18.326143, 81.618416, 18.316278, 81.64807, 18.307411);
		shape64.curveTo(81.677666, 18.299484, 81.698395, 18.294552, 81.71026, 18.294552);
		shape64.lineTo(81.72212, 18.294552);
		shape64.curveTo(81.73299, 18.294552, 81.74579, 18.298487, 81.757645, 18.306414);
		shape64.curveTo(81.77045, 18.314281, 81.776375, 18.324146, 81.776375, 18.338005);
		shape64.curveTo(81.776375, 18.364664, 81.76457, 18.389326, 81.74085, 18.41205);
		shape64.curveTo(81.71619, 18.433777, 81.68754, 18.454504, 81.654945, 18.472237);
		shape64.curveTo(81.62241, 18.491026, 81.58783, 18.506823, 81.55131, 18.52162);
		shape64.curveTo(81.514786, 18.536417, 81.48419, 18.54728, 81.461464, 18.555206);
		shape64.curveTo(81.4368, 18.561079, 81.401276, 18.566011, 81.35583, 18.571001);
		shape64.curveTo(81.31044, 18.575933, 81.265045, 18.581806, 81.2196, 18.589733);
		shape64.curveTo(81.17421, 18.59766, 81.13569, 18.608465, 81.103096, 18.62332);
		shape64.curveTo(81.069565, 18.63712, 81.05377, 18.657846, 81.05377, 18.685503);
		shape64.curveTo(81.05377, 18.692432, 81.05771, 18.70623, 81.065575, 18.723965);
		shape64.curveTo(81.0725, 18.742754, 81.08137, 18.761486, 81.09123, 18.782215);
		shape64.curveTo(81.10116, 18.802004, 81.11002, 18.822731, 81.11989, 18.84246);
		shape64.curveTo(81.12975, 18.863188, 81.13668, 18.878984, 81.14062, 18.890846);
		shape64.curveTo(81.148544, 18.905643, 81.15248, 18.925373, 81.15348, 18.950094);
		shape64.curveTo(81.15448, 18.973759, 81.15542, 18.99842, 81.15642, 19.022142);
		shape64.curveTo(81.15742, 19.045807, 81.15842, 19.068531, 81.160355, 19.08826);
		shape64.curveTo(81.16235, 19.108988, 81.16828, 19.122847, 81.17815, 19.130714);
		shape64.curveTo(81.184074, 19.136644, 81.19001, 19.139639, 81.19694, 19.14058);
		shape64.curveTo(81.203804, 19.141577, 81.21174, 19.142576, 81.219604, 19.144571);
		shape64.curveTo(81.22847, 19.14651, 81.2364, 19.151442, 81.243324, 19.158371);
		shape64.curveTo(81.25019, 19.1643, 81.25613, 19.175165, 81.262054, 19.187965);
		shape64.curveTo(81.26998, 19.207754, 81.27492, 19.226484, 81.27785, 19.246216);
		shape64.curveTo(81.280785, 19.265945, 81.283775, 19.285734, 81.28771, 19.306461);
		shape64.curveTo(81.29165, 19.328188, 81.29564, 19.347918, 81.299576, 19.367647);
		shape64.curveTo(81.30351, 19.387377, 81.3065, 19.407166, 81.30844, 19.428833);
		shape64.curveTo(81.31243, 19.445627, 81.318306, 19.46242, 81.32817, 19.479214);
		shape64.curveTo(81.3381, 19.495008, 81.343025, 19.51374, 81.343025, 19.535467);
		shape64.lineTo(81.343025, 19.544392);
		shape64.lineTo(81.322296, 19.769463);
		shape64.lineTo(81.28178, 19.769463);
		shape64.lineTo(81.25612, 19.769463);
		shape64.curveTo(81.24825, 19.769463, 81.23933, 19.766468, 81.22846, 19.759598);
		shape64.curveTo(81.21766, 19.752668, 81.20773, 19.744741, 81.19787, 19.734877);
		shape64.curveTo(81.189, 19.725012, 81.17914, 19.716145, 81.17121, 19.70628);
		shape64.curveTo(81.162346, 19.696415, 81.15542, 19.689486, 81.15148, 19.685553);
		shape64.curveTo(81.13075, 19.652964, 81.11695, 19.607573, 81.11102, 19.550264);
		shape64.curveTo(81.109024, 19.5404, 81.109024, 19.5276, 81.109024, 19.510805);
		shape64.curveTo(81.109024, 19.49401, 81.109024, 19.477219, 81.109024, 19.460424);
		shape64.curveTo(81.109024, 19.442692, 81.109024, 19.426895, 81.109024, 19.412098);
		shape64.curveTo(81.109024, 19.398241, 81.108025, 19.388376, 81.10609, 19.382444);
		shape64.curveTo(81.10215, 19.37258, 81.09522, 19.361717, 81.087296, 19.349915);
		shape64.curveTo(81.07843, 19.338993, 81.069565, 19.331125, 81.06263, 19.32719);
		shape64.curveTo(81.04484, 19.32126, 81.03397, 19.314331, 81.029045, 19.305464);
		shape64.curveTo(81.02411, 19.296597, 81.01524, 19.292604, 81.00145, 19.292604);
		shape64.curveTo(80.989586, 19.292604, 80.97773, 19.299534, 80.96393, 19.314331);
		shape64.curveTo(80.95007, 19.328188, 80.93527, 19.336058, 80.91754, 19.336058);
		shape64.curveTo(80.907616, 19.336058, 80.89481, 19.332123, 80.87702, 19.324196);
		shape64.curveTo(80.86023, 19.316328, 80.8415, 19.306463, 80.82177, 19.295599);
		shape64.curveTo(80.80298, 19.283737, 80.78425, 19.272875, 80.767456, 19.262012);
		shape64.curveTo(80.749725, 19.25115, 80.7388, 19.24328, 80.732925, 19.23735);
		shape64.curveTo(80.70527, 19.216623, 80.67861, 19.1929, 80.65295, 19.168238);
		shape64.curveTo(80.62729, 19.142578, 80.60556, 19.117916, 80.58777, 19.093195);
		shape64.curveTo(80.57098, 19.060606, 80.55424, 19.028076, 80.54038, 18.997482);
		shape64.curveTo(80.52558, 18.966831, 80.50386, 18.938234, 80.475266, 18.913513);
		shape64.curveTo(80.4654, 18.903648, 80.45055, 18.89578, 80.43082, 18.890848);
		shape64.curveTo(80.41009, 18.884918, 80.392296, 18.879986, 80.3765, 18.87605);
		shape64.curveTo(80.3637, 18.868124, 80.34397, 18.860256, 80.31925, 18.853327);
		shape64.curveTo(80.30745, 18.849392, 80.29752, 18.847397, 80.28966, 18.848394);
		shape64.curveTo(80.28273, 18.849392, 80.271866, 18.848394, 80.258064, 18.844402);
		shape64.curveTo(80.237335, 18.840467, 80.213615, 18.83947, 80.18895, 18.841467);
		shape64.curveTo(80.16329, 18.843464, 80.140564, 18.842464, 80.119835, 18.838531);
		shape64.curveTo(80.09617, 18.83454, 80.07544, 18.832602, 80.05566, 18.831602);
		shape64.curveTo(80.03693, 18.830605, 80.0172, 18.825672, 79.998405, 18.817802);
		shape64.curveTo(79.97081, 18.81287, 79.94808, 18.80694, 79.93029, 18.802008);
		shape64.curveTo(79.91156, 18.797016, 79.89477, 18.792084, 79.87897, 18.786213);
		shape64.curveTo(79.86417, 18.780283, 79.84937, 18.773354, 79.83452, 18.766424);
		shape64.curveTo(79.81972, 18.758556, 79.802925, 18.74963, 79.784195, 18.739824);
		shape64.curveTo(79.7516, 18.724968, 79.71902, 18.708174, 79.68842, 18.690441);
		shape64.curveTo(79.65783, 18.673647, 79.626236, 18.657852, 79.59265, 18.643055);
		shape64.curveTo(79.56012, 18.628258, 79.524536, 18.616396, 79.48608, 18.60753);
		shape64.curveTo(79.447556, 18.597666, 79.404106, 18.592674, 79.35572, 18.592674);
		shape64.lineTo(79.34979, 18.592674);
		shape64.curveTo(79.327126, 18.592674, 79.30147, 18.592674, 79.27381, 18.592674);
		shape64.curveTo(79.245155, 18.592674, 79.21756, 18.591734, 79.1899, 18.589739);
		shape64.curveTo(79.16125, 18.587742, 79.13459, 18.584806, 79.10893, 18.579874);
		shape64.curveTo(79.082275, 18.574942, 79.06061, 18.568012, 79.043816, 18.561085);
		shape64.curveTo(79.00236, 18.54529, 78.964836, 18.526558, 78.92925, 18.50583);
		shape64.curveTo(78.89273, 18.484104, 78.85127, 18.470306, 78.80289, 18.462378);
		shape64.curveTo(78.78216, 18.452513, 78.7575, 18.446583, 78.730835, 18.443645);
		shape64.curveTo(78.70418, 18.44065, 78.67851, 18.439653, 78.65585, 18.439653);
		shape64.curveTo(78.63806, 18.437716, 78.62421, 18.434721, 78.612404, 18.430786);
		shape64.curveTo(78.60054, 18.426851, 78.589676, 18.422918, 78.58081, 18.420921);
		shape64.curveTo(78.570946, 18.417986, 78.56108, 18.414991, 78.551155, 18.412054);
		shape64.curveTo(78.54129, 18.40906, 78.530426, 18.406124, 78.516624, 18.404127);
		shape64.curveTo(78.48403, 18.401192, 78.456375, 18.399195, 78.434654, 18.400192);
		shape64.curveTo(78.41199, 18.40119, 78.38827, 18.397198, 78.3636, 18.387333);
		shape64.curveTo(78.334946, 18.373533, 78.321144, 18.35674, 78.32414, 18.335014);
		shape64.curveTo(78.32708, 18.314287, 78.32908, 18.289625, 78.32908, 18.262966);
		shape64.lineTo(78.372475, 18.262966);
		shape64.lineTo(78.985565, 18.294558);
		shape64.lineTo(78.9885, 18.294558);
		shape64.curveTo(78.99543, 18.294558, 79.005295, 18.287628, 79.01616, 18.274828);
		shape64.curveTo(79.02702, 18.260971, 79.03195, 18.250166, 79.03195, 18.242239);
		shape64.curveTo(79.03195, 18.238304, 79.02109, 18.234312, 78.99837, 18.22938);
		shape64.curveTo(78.97664, 18.224447, 78.962845, 18.220512, 78.95691, 18.21658);
		shape64.curveTo(78.94311, 18.21065, 78.93424, 18.206715, 78.92925, 18.204718);
		shape64.curveTo(78.92432, 18.20278, 78.92039, 18.201782, 78.91645, 18.200783);
		shape64.curveTo(78.91251, 18.199785, 78.907585, 18.198786, 78.90165, 18.197847);
		shape64.curveTo(78.89572, 18.19685, 78.88586, 18.194853, 78.87299, 18.190918);
		shape64.curveTo(78.85919, 18.186926, 78.84534, 18.182991, 78.830536, 18.179056);
		shape64.curveTo(78.816734, 18.175121, 78.80488, 18.167194, 78.798004, 18.155392);
		shape64.curveTo(78.794014, 18.148462, 78.78708, 18.135603, 78.777214, 18.11787);
		shape64.curveTo(78.76735, 18.101076, 78.76241, 18.090214, 78.76241, 18.086279);
		shape64.curveTo(78.76241, 18.057625, 78.76834, 18.035898, 78.780205, 18.023096);
		shape64.curveTo(78.79207, 18.00924, 78.80487, 17.994442, 78.82066, 17.979643);
		shape64.curveTo(78.82859, 17.967781, 78.83353, 17.957916, 78.83646, 17.94999);
		shape64.curveTo(78.839455, 17.94312, 78.84338, 17.93619, 78.84832, 17.93026);
		shape64.curveTo(78.85326, 17.92433, 78.859184, 17.919397, 78.86511, 17.914465);
		shape64.curveTo(78.87198, 17.909533, 78.88284, 17.9046, 78.89864, 17.89867);
		shape64.curveTo(78.91444, 17.890802, 78.93323, 17.881876, 78.955894, 17.871073);
		shape64.curveTo(78.979614, 17.86021, 78.994415, 17.851284, 79.00234, 17.843416);
		shape64.curveTo(79.00628, 17.83549, 79.00921, 17.825624, 79.01121, 17.811825);
		shape64.curveTo(79.01321, 17.798025, 79.014145, 17.786165, 79.014145, 17.777239);
		shape64.curveTo(79.018135, 17.761444, 79.02207, 17.749641, 79.02601, 17.740715);
		shape64.curveTo(79.029945, 17.731848, 79.033936, 17.722982, 79.03787, 17.713118);
		shape64.curveTo(79.06647, 17.726917, 79.08626, 17.75158, 79.09512, 17.788101);
		shape64.closePath();
		shape64.moveTo(79.5759, 19.82771);
		shape64.lineTo(79.364624, 19.82771);
		shape64.lineTo(79.364624, 19.196829);
		shape64.curveTo(79.364624, 19.186024, 79.35969, 19.180094, 79.34977, 19.180094);
		shape64.lineTo(79.31824, 19.180094);
		shape64.curveTo(79.305374, 19.180094, 79.29845, 19.186024, 79.29845, 19.196829);
		shape64.lineTo(79.29845, 19.82771);
		shape64.lineTo(79.0961, 19.82771);
		shape64.lineTo(79.0961, 19.196829);
		shape64.curveTo(79.0961, 19.186024, 79.09116, 19.180094, 79.0813, 19.180094);
		shape64.lineTo(79.049706, 19.180094);
		shape64.curveTo(79.03784, 19.180094, 79.03191, 19.186024, 79.03191, 19.196829);
		shape64.lineTo(79.03191, 19.82771);
		shape64.lineTo(78.82064, 19.82771);
		shape64.lineTo(78.82064, 19.15044);
		shape64.curveTo(78.82064, 19.142572, 78.82164, 19.134645, 78.82364, 19.125778);
		shape64.curveTo(78.82558, 19.11791, 78.830505, 19.108984, 78.83844, 19.101116);
		shape64.curveTo(78.847305, 19.098122, 78.85717, 19.09319, 78.8651, 19.08726);
		shape64.curveTo(78.87396, 19.08133, 78.88383, 19.078392, 78.89269, 19.078392);
		shape64.lineTo(79.503784, 19.078392);
		shape64.curveTo(79.51271, 19.078392, 79.52058, 19.081327, 79.52651, 19.08726);
		shape64.curveTo(79.53244, 19.09319, 79.54031, 19.098122, 79.55023, 19.101116);
		shape64.curveTo(79.56696, 19.11691, 79.57589, 19.13664, 79.57589, 19.159367);
		shape64.lineTo(79.57589, 19.82771);
		shape64.closePath();
		shape64.moveTo(79.88588, 19.82771);
		shape64.lineTo(79.671616, 19.82771);
		shape64.lineTo(79.671616, 19.078394);
		shape64.lineTo(79.88588, 19.078394);
		shape64.lineTo(79.88588, 19.82771);
		shape64.closePath();
		shape64.moveTo(80.470314, 19.82771);
		shape64.lineTo(79.98065, 19.82771);
		shape64.lineTo(79.98065, 19.734875);
		shape64.lineTo(80.250114, 19.180096);
		shape64.lineTo(79.98065, 19.180096);
		shape64.lineTo(79.98065, 19.078394);
		shape64.lineTo(80.470314, 19.078394);
		shape64.lineTo(80.470314, 19.180096);
		shape64.lineTo(80.191864, 19.734875);
		shape64.lineTo(80.470314, 19.734875);
		shape64.lineTo(80.470314, 19.82771);
		shape64.closePath();
		shape64.moveTo(80.764496, 19.82771);
		shape64.lineTo(80.5443, 19.82771);
		shape64.lineTo(80.5443, 19.078394);
		shape64.lineTo(80.764496, 19.078394);
		shape64.lineTo(80.764496, 19.82771);
		shape64.closePath();
		shape64.moveTo(79.99838, 16.72686);
		shape64.curveTo(80.55422, 16.72686, 81.03002, 16.925272, 81.4259, 17.321157);
		shape64.curveTo(81.821785, 17.717043, 82.0202, 18.192844, 82.0202, 18.748678);
		shape64.curveTo(82.0202, 19.302519, 81.82179, 19.777319, 81.4259, 20.173208);
		shape64.curveTo(81.03002, 20.569094, 80.554214, 20.767506, 79.99838, 20.767506);
		shape64.curveTo(79.44454, 20.767506, 78.96974, 20.569094, 78.57385, 20.173208);
		shape64.curveTo(78.17797, 19.777323, 77.97955, 19.302519, 77.97955, 18.748678);
		shape64.curveTo(77.97955, 18.19284, 78.17796, 17.717037, 78.57385, 17.321157);
		shape64.curveTo(78.969734, 16.925272, 79.44454, 16.72686, 79.99838, 16.72686);
		shape64.closePath();
		shape64.moveTo(79.99838, 20.48419);
		shape64.curveTo(80.47324, 20.48419, 80.880936, 20.313375, 81.221565, 19.971806);
		shape64.curveTo(81.56313, 19.631233, 81.733894, 19.223488, 81.733894, 18.748686);
		shape64.curveTo(81.733894, 18.273823, 81.56314, 17.866137, 81.221565, 17.525505);
		shape64.curveTo(80.880936, 17.183935, 80.47324, 17.01318, 79.99838, 17.01318);
		shape64.curveTo(79.523575, 17.01318, 79.11583, 17.183935, 78.77526, 17.525505);
		shape64.curveTo(78.43369, 17.866135, 78.26288, 18.273823, 78.26288, 18.748686);
		shape64.curveTo(78.26288, 19.22349, 78.43369, 19.631235, 78.77526, 19.971806);
		shape64.curveTo(79.11583, 20.313375, 79.52358, 20.48419, 79.99838, 20.48419);
		shape64.closePath();

		if ( dMode == MODE_ROTATE ) {
			g.setPaint(new Color(255, 100, 100, 255));
		} else {
			g.setPaint(new Color(197, 198, 198, 255));
		}
		g.fill(shape64);
	}

	private void paintCompositeGraphicsNode_0_0_0_0(Graphics2D g) {
		// _0_0_0_0_0
		AffineTransform trans_0_0_0_0_0 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_0(g);
		g.setTransform(trans_0_0_0_0_0);
		// _0_0_0_0_1
		AffineTransform trans_0_0_0_0_1 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_1(g);
		g.setTransform(trans_0_0_0_0_1);
		// _0_0_0_0_2
		AffineTransform trans_0_0_0_0_2 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_2(g);
		g.setTransform(trans_0_0_0_0_2);
		// _0_0_0_0_3
		AffineTransform trans_0_0_0_0_3 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_3(g);
		g.setTransform(trans_0_0_0_0_3);
		// _0_0_0_0_4
		AffineTransform trans_0_0_0_0_4 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_4(g);
		g.setTransform(trans_0_0_0_0_4);
		// _0_0_0_0_5
		AffineTransform trans_0_0_0_0_5 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintCompositeGraphicsNode_0_0_0_0_5(g);
		g.setTransform(trans_0_0_0_0_5);
		// _0_0_0_0_6
		AffineTransform trans_0_0_0_0_6 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintCompositeGraphicsNode_0_0_0_0_6(g);
		g.setTransform(trans_0_0_0_0_6);
		// _0_0_0_0_7
		AffineTransform trans_0_0_0_0_7 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintShapeNode_0_0_0_0_7(g);
		g.setTransform(trans_0_0_0_0_7);
	}

	private void paintCompositeGraphicsNode_0_0_0(Graphics2D g) {
		// _0_0_0_0
		AffineTransform trans_0_0_0_0 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintCompositeGraphicsNode_0_0_0_0(g);
		g.setTransform(trans_0_0_0_0);
	}

	private void paintCanvasGraphicsNode_0_0(Graphics2D g) {
		// _0_0_0
		AffineTransform trans_0_0_0 = g.getTransform();
		g.transform(new AffineTransform(1.0f, 0.0f, 0.0f, 1.0f, 0.0f, 0.0f));
		paintCompositeGraphicsNode_0_0_0(g);
		g.setTransform(trans_0_0_0);
	}

	private void paintRootGraphicsNode_0(Graphics2D g) {
		// _0_0
		g.setComposite(AlphaComposite.getInstance(3, 1.0f * origAlpha));
		AffineTransform trans_0_0 = g.getTransform();
		g.transform(new AffineTransform(3.7749905586242676f, 0.0f, 0.0f, 3.7749905586242676f, 0.5007611148284923f, -0.0f));
		paintCanvasGraphicsNode_0_0(g);
		g.setTransform(trans_0_0);
	}
	
}
